/* !!Copyright begin */
/*
War Software series
Copyright (C) 1996 - 1998 by Jarle (jgaa) Aase (www.jgaa.com)
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation version 2
of the License - WITH THE FOLLOWING RESTRICTIONS:

  * GOVERNMENTS, GOVERNMENTAL AGENCIES OR COMMERCIAL 
    COMPANIES OWNED 50% OR MORE BY THE GOVERNMENT ARE
    NOT ALLOWED TO USE THE SOFTWARE OR SORCE CODE FOR
    ANY PURPOSE. THIS RESTRICTION ALSO APPLIES FOR
    PROGRAMS DERIVED FROM THE SOURCE CODE, OR PROGRAMS/
    SOURCE CODE THAT USE SPECIAL TECHNICS/ALOGRITHMS
    INVENTED IN THIS SOFTWARE.

  * THE FULL SOURCE CODE IS WITHELD FROM A PUBLIC RELEASE
    UNTIL THE LINUX VERISON IS RELEASED.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
 
/* !!Copyright end */

/*c++Doc ===================================================
class WarLinkedListItem;
Abstract: Data container for a WarLinkedList item
This class is usually used as a transparent pointer
to a data object in a WarLinkedList
	;
Arguments: ;
Note: ;
Group:GenData;
Project: ;
Return: ;
Sample: ;
See also: WarLinkedList;
==========================================================*/


class WarLinkedListItem
{
public:
	WarLinkedListItem();
	~WarLinkedListItem();

	WarLinkedListItem *m_Next;
	WarLinkedListItem *m_Prev;
	LPVOID m_Ptr; // Data
};


/*c++Doc ===================================================
class WarLinkedList;
Abstract: Generic double linked list;
Arguments: ;
Note: If the list is not emptyed before the object is destructed,
the destructor will give an assertion.;
Group:GenData;
Project: ;
Return: ;
Sample:{ 

WarLinkedList List;

for(int Index = 0; Index < 10; Index++)
	List.AddFirst((LPVOID)new CFoo);

WarLinkedListItem *Item;
CFoo *pFoo;

for(Item = List.First(); Item; Item = List.Next(Item))
{
	pFoo = (CFoo *)List.Ptr(Item);
	...
}

};
See also: ;
==========================================================*/

class WarLinkedList
{

private:
	WarLinkedListItem *m_First;
	WarLinkedListItem *m_Last;

protected:
	int m_NumItems;	// Number of items in list

public:
	WarLinkedList();
	~WarLinkedList();
	virtual void Kill(); // KillAll + delete the this pointer
	virtual void KillAll(); // Kill all items, including the pointer.
	WarLinkedListItem *First();
	WarLinkedListItem *Last();
	LPVOID FirstPtr();
	LPVOID LastPtr();
	LPVOID Ptr(WarLinkedListItem *Item);
	WarLinkedListItem *Next(WarLinkedListItem *Item);
	WarLinkedListItem *Prev(WarLinkedListItem *Item);
	WarLinkedListItem *FindPtr(LPVOID Ptr);
	BOOL DeletePtr(LPVOID Ptr);
	void DeleteItem(WarLinkedListItem *Item);
	void AddFirst(LPVOID Ptr);
	void AddLast(LPVOID Ptr);
	void AddPtr(WarLinkedListItem *Next, WarLinkedListItem *Prev, LPVOID Ptr);
	LPVOID GetAndDeleteFirst();
	LPVOID GetAndDeleteLast();
	void Sort(int (*SortFunc)(const void *, const void *));
	int GetItemCount() {return m_NumItems;}
	int IsEmpty() {return m_NumItems == 0;}
	void Empty();
	void Cat(WarLinkedList& List);
	void operator = (WarLinkedList &List);
	void operator += (WarLinkedList &List) {Cat(List);}
};

