
//@{

#ifndef _WAR_CMD_ARG_H
#define _WAR_CMD_ARG_H

//@}


/** Simple command-line argument parser.

    Gerenerally works as GNU getopt_long(), but
    is implemented as a class, and does not write
    to stdout or stderr.

    The option datatype belongs to the class, as does the
    _argument values. 

    The POSIXLY_CORRECT feature is not implemented.
*/
class WarCmdArg
{
public:
    /// Option has_arg values
    enum ArgFlagE
    {
        /// Equalent to 0 (zero)
        no_argument,
        /// 
        required_argument,
        ///
        optional_argument,
    };

    /// As in GNU getopt.h: typedef .. option
    typedef struct option_def
    {
        /// Name of the long argument without "//" ..
        const char *name;
        /// \Ref{ArgFlagE}
        int has_arg;
        /// Normally 0
        int *flag;
        /// Normally the return value if the argument match
        int val;
    } option;

    /** Creates and initializes the parser. No error checking is
        performed.a

        @param argc main()'s argc
        @param argv main()'s argv
        @param optstring A string with valid options. Hyphen is
            reserved. If a letter is followed by a colon, the option
            has an argument. If the colon is followed by another colon,
            the argument is optional.
        @param longopts a array of structure \Ref{option} defining the
            long argument names and return values.
    */
    WarCmdArg(int argc,  char * const *argv, 
        const char * optstring, const option * longopts);

    /** Parse the command line options and return the next
        argument

        Arguments like -x -y and -xy are considered equal.

        @return If the option is given in short-form, the option letter
            is returned. If the option is given in long-form, the
            option.val is normally returned (if option.flag is NULL).
    */
    int GetArg();

    /** The current index into argv. Used to parse the first
        non-option argument.
    */
    int GetOptind() const
    {
        return m_ArgIndex;
    }

    const char * GetArgument() const
    {
        return m_pOptArg;
    }

    const char * GetCurrArg() const
    {
        return &m_argv[m_ArgIndex][m_OptIdx];
    }

private:
    const int m_argc;
    char * const *m_argv;
    const char * m_optstring;
    const option * m_longopts;
    int m_ArgIndex;
    int m_OptIdx;
    const char * m_pOptArg;
    int m_Flag;
};  


#endif // /_WAR_CMD_ARG_H

