/* !!Copyright begin */
/*
War Software series
Copyright (C) 1996 - 1998 by Jarle (jgaa) Aase (www.jgaa.com)
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation version 2
of the License - WITH THE FOLLOWING RESTRICTIONS:

  * GOVERNMENTS, GOVERNMENTAL AGENCIES OR COMMERCIAL 
    COMPANIES OWNED 50% OR MORE BY THE GOVERNMENT ARE
    NOT ALLOWED TO USE THE SOFTWARE OR SORCE CODE FOR
    ANY PURPOSE. THIS RESTRICTION ALSO APPLIES FOR
    PROGRAMS DERIVED FROM THE SOURCE CODE, OR PROGRAMS/
    SOURCE CODE THAT USE SPECIAL TECHNICS/ALOGRITHMS
    INVENTED IN THIS SOFTWARE.

  * THE FULL SOURCE CODE IS WITHELD FROM A PUBLIC RELEASE
    UNTIL THE LINUX VERISON IS RELEASED.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
 
/* !!Copyright end */

// WarFile.h

// We try to stay as close to UNIX as possible when working with permissions.
// However, we need an extra set of permissons pr. class for free and hide
// bits. We also use our own bits for links and directories so that we can
// distinguish between links to files and linke to directorioes.
// And - since we are not a "real" file system - we need to know if the file/dir
// is read-only.

/* Bit layout for the file mode (m_Flags)

          30                                      20                                      10
| 2 | 1 | 0 | 9 | 8 | 7 | 6 | 5 | 4 | 3 | 2 | 1 | 0 | 9 | 8 | 7 | 6 | 5 | 4 | 3 | 2 | 1 | 0 | 9 | 8 | 7 | 6 | 5 | 4 | 3 | 2 | 1 |
+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
|   |   | w | W | W | W | W | W | W | W | W | W | W | W | W | W | X | X | X | X | X | X | X | X | X | X | X | X | X | X | X | X |
+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
|   |   | S | s | s | s | s | p | p | p | p | p | p | p | p | p | s | s | s | s | s | s | s | p | p | p | p | p | p | p | p | p |
|   |   |dsk|vir|dir|lnk|sro| U | U | U | G | G | G | O | O | O |   |   |   |   |   |   |   | U | U | U | G | G | G | O | O | O |
|   |   |   |   |   |   |   | f | h | c | f | h | c | f | h | c |reg|dir| cs| np|uox|gox| sb| r | w | x | r | w | x | r | w | x |
|   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |---|  bls  |   |   |   |   |   |   |   |   |   |   |   |   |   |
|   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |lnk --- lnk|   |   |   |   |   |   |   |   |   |   |   |   |   |
|   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |  soc  |---|   |   |   |   |   |   |   |   |   |   |   |   |   |
|   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   who     |   |   |   |sav|   |   |   |   |   |   |   |   |   |

X = UNIX     U = user      G = group    O = other   W = War
r = read     w = write     x = exec     f = free    h = hide
s = system   c = create

dsk = Drive
vir = Virtual file (Only exist in memory)
sro = System Read Only (or CD-ROM)
uox = S_ISUID	set user id on execution 
gox = S_ISGID	set group id on execution 
sb  = S_ISTXT	sticky bit 
np  = S_IFIFO	named pipe (fifo)
cs  = S_IFCHR	character special
dir = S_IFDIR	directory
bls = S_IFBLK	block special
reg = S_IFREG	regular
lnk = S_IFLNK	symbolic link
soc = S_IFSOCK socket
sav = S_ISVTX	save swapped text even after use
who = S_IFWHT whiteout

*/

#ifndef S_ISUID
#define	S_ISUID	0004000			/* set user id on execution */
#define	S_ISGID	0002000			/* set group id on execution */
#ifndef _POSIX_SOURCE
#define	S_ISTXT	0001000			/* sticky bit */
#endif

#define	S_IRWXU	0000700			/* RWX mask for owner */
#define	S_IRUSR	0000400			/* R for owner */
#define	S_IWUSR	0000200			/* W for owner */
#define	S_IXUSR	0000100			/* X for owner */

#ifndef _POSIX_SOURCE
#ifdef S_IREAD // <sys/stat> is included
#undef S_IREAD
#undef S_IWRITE
#undef S_IEXEC
#undef S_IFMT
#undef S_IFCHR
#undef S_IFDIR
#undef S_IFREG
#endif // <sys/stat>
#define	S_IREAD		S_IRUSR
#define	S_IWRITE	S_IWUSR
#define	S_IEXEC		S_IXUSR
#endif // Posix

#define	S_IRWXG	0000070			/* RWX mask for group */
#define	S_IRGRP	0000040			/* R for group */
#define	S_IWGRP	0000020			/* W for group */
#define	S_IXGRP	0000010			/* X for group */

#define	S_IRWXO	0000007			/* RWX mask for other */
#define	S_IROTH	0000004			/* R for other */
#define	S_IWOTH	0000002			/* W for other */
#define	S_IXOTH	0000001			/* X for other */

// Currently not used by War
#ifndef _POSIX_SOURCE
#define	S_IFMT	 0170000		/* type of file mask */
#define	S_IFIFO	 0010000		/* named pipe (fifo) */
#define	S_IFCHR	 0020000		/* character special */
#define	S_IFDIR	 0040000		/* directory */
#define	S_IFBLK	 0060000		/* block special */
#define	S_IFREG	 0100000		/* regular */
#define	S_IFLNK	 0120000		/* symbolic link */
#define	S_IFSOCK 0140000		/* socket */
#define	S_IFWHT  0160000		/* whiteout */
#define	S_ISVTX	 0001000		/* save swapped text even after use */
#endif

// Currently not used by War
#define	S_ISDIR(m)	(((m) & 0170000) == 0040000)	/* directory */
#define	S_ISCHR(m)	(((m) & 0170000) == 0020000)	/* char special */
#define	S_ISBLK(m)	(((m) & 0170000) == 0060000)	/* block special */
#define	S_ISREG(m)	(((m) & 0170000) == 0100000)	/* regular file */
#define	S_ISFIFO(m)	(((m) & 0170000) == 0010000 || \
			 ((m) & 0170000) == 0140000)	/* fifo or socket */

// Currently not used  by War
#ifndef _POSIX_SOURCE
#define	S_ISLNK(m)	(((m) & 0170000) == 0120000)	/* symbolic link */
#define	S_ISSOCK(m)	(((m) & 0170000) == 0010000 || \
			 ((m) & 0170000) == 0140000)	/* fifo or socket */
#define	S_ISWHT(m)	(((m) & 0170000) == 0160000)	/* whiteout */
#endif

// Currently not used by War
#ifndef _POSIX_SOURCE
#define	ACCESSPERMS	(S_IRWXU|S_IRWXG|S_IRWXO)	/* 0777 */
							/* 7777 */
#define	ALLPERMS	(S_ISUID|S_ISGID|S_ISTXT|S_IRWXU|S_IRWXG|S_IRWXO)
							/* 0666 */
#define	DEFFILEMODE	(S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH)

#define S_BLKSIZE	512		/* block size used in the stat struct */
#endif
#endif // S_ISUID

// Inode flags

#define NODE_DIR			(1 << 27)	// Directory
#define NODE_VIRTUAL	(1 << 28)	// No dir, just a name
#define NODE_READONLY	(1 << 25)	// Operating system read only
#define NODE_LINK     (1 << 26) // Link
#define NODE_DRIVE		(1 << 29) // Drive

// Node permissions
#define NODE_OREAD		S_IRUSR	// Owner read
#define NODE_0WRITE		S_IWUSR	// Owner write
#define NODE_OEXEC		S_IXUSR	// Owner execute
#define NODE_OFREE		(1 << 24) // Owner free download
#define NODE_OHIDE    (1 << 23) // Hide from owner
#define NODE_OCREA		(1 << 22) // Create

#define NODE_GREAD		S_IRGRP	// Group read
#define NODE_GWRITE		S_IWGRP	// Group write
#define NODE_GEXEC		S_IXGRP	// Group execute
#define NODE_GFREE		(1 << 21) // Group free download
#define NODE_GHIDE    (1 << 20) // Hide from group
#define NODE_GCREA		(1 << 19) // Create

#define NODE_AREAD		S_IROTH	// All read
#define NODE_AWRITE		S_IWOTH	// All write
#define NODE_AEXEC		S_IXOTH	// All execute
#define NODE_AFREE		(1 << 18) // All (other) free download
#define NODE_AHIDE    (1 << 17) // Hide 
#define NODE_ACREA		(1 << 16) // Create

#define NODE_READ			0x0001	// Request read access
#define NODE_WRITE		0x0002	// Request write/delete access
#define NODE_EXEC			0x0004	// Request execute/chdir access
#define NODE_ANY			0x0008	// Request any access
#define NODE_CREATE		0x0010	// Request permission to create
#define NODE_HIDE			0x0020	// Is node hidden for user?
#define NODE_FREE			0x0040	// Is node free for user?

#define NODE_MBFILE		0				// Must be file
#define NODE_MBDIR		1				// Must be directory
#define NODE_MBANY		2				// Can be anything

#define NODE_WRITEMSK (NODE_0WRITE|NODE_GWRITE|NODE_AWRITE)

#define NODE_DEFAULT_FILE 0666
#define NODE_DEFAULT_DIR 0777
#define NODE_SETTABLE 0x1B61FFF // Mask of bits that the user can chmod...

#define	ISDOT(a)	(a[0] == '.')
#define IS_READONLY(a) (a->m_Flags & NODE_READONLY)
#define IS_DIRECTORY(a) (a->m_Flags & NODE_DIR)
#define IS_DRIVE(a) (a->m_Flags & NODE_DRIVE)
#define IS_LINK(a) (a->m_Flags & NODE_LINK)
#define IS_EXEC(a) (a->m_Flags & (NODE_OEXEC | NODE_GEXEC | NODE_AEXEC))




class WarFile;

class WarFStat
{
public:
	WarFStat();
	~WarFStat();

	LPCSTR FileName() {return m_Name;}
	void MakeFileNameASCII();
	int GetFileNameLen() {return strlen(m_Name);}
	int GetFileBlocks() {return (int)(m_size / 512);}
	unsigned GetInode() {return m_Inode;}
	unsigned GetLinks() {return m_Links;}
	FLEN GetFileLength() {return m_size;}
	LPCSTR GetUserName() {return "root";}
	LPCSTR GetGroupName() {return "root";}
	BOOL IsHidden() {return m_Perms & NODE_AHIDE;}
	LPCSTR GetComment();
	int GetDLC() {return m_DLC;}
	BOOL IsDirectory() {return m_Perms & NODE_DIR;}
	BOOL IsLink() {return m_Perms & NODE_LINK;}
	WarTime GetAccessTime() {return m_atime;}
	WarTime GetModifyTime() {return m_mtime;}
	WarTime GetCreationTime() {return m_ctime;}
	WarTime GetLastModeTime() {return m_mtime;}

	LPSTR m_Name; // File name
	LPSTR m_Title; // Document title
	WarTime m_ctime; // Creation time
	WarTime m_mtime; // Last modification time
	WarTime m_atime; // Last access time
	FLEN m_size; // Size, as defined by PositionModes
	DWORD m_Perms; // VFSYS/UNIX perms
	user m_Owner; // Owner (Depending on protocol)
	user m_Group; // Owner (Depending on protocol)
	user *m_MoreGroups; // VfSys/UNIX
	unsigned m_Links : 16;
	unsigned m_dev : 15;
	unsigned m_IsFsAccessNode : 1;
	int m_Inode;
	int m_DLC; /* Download count */
	LPVOID m_pFather; // Used by WarFileList to identify the father node
};

class WarFileList : public WarLinkedList
{
public:
	WarFileList(LPCSTR RootPath);
	~WarFileList();

	LPCSTR GetPath() {return m_RootPath;}

	WarLinkedListItem *Add(WarFStat *pFS, WarLinkedListItem *pFather);
	void AddUnique(WarFStat *pFS);
	WarString GetFulllPath(WarLinkedListItem *pItem);
	WarFStat *Find(LPCSTR Name);
	void Clear();

	BOOL m_bSupressTotal;

	WarString m_RootPath;
};

// Infor about names
class WarStatInfo
{
public:
	WarStatInfo();
	~WarStatInfo();

	LPCSTR m_Owner;
	LPCSTR m_Group;
	LPCSTR *m_MoreGroups;
	LPSTR m_buffer; 
};

// Overridable object to receive callback notifications
class WarFileCallback
{
public:
	WarFileCallback();
	~WarFileCallback();

	void SetRef(WarFile *pFile);

	WarFile *m_pFile; // Pointer to the war file object

	virtual void OnOpen(BOOL Success) {;}
	virtual void OnRead(BOOL Success, DWORD BytesRead) {;}
	virtual void OnWrite(BOOL Success, DWORD BytesWritten) {;}
	virtual void OnScan(BOOL Success) {;}
	virtual void OnStat(BOOL Success, WarFStat &St) {;}
};

class WarFileException
{
private:
	WarFileException();
public:
	~WarFileException();

	enum Errors
	{
		OK,
		ACCESS_DENIED = MED_WAR_FILE,
		FILE_IS_BANNED,
		NO_CREDITS,
		END_OF_FILE,
		NOT_OPEN,
		NOT_OPEN_FOR_READ,
		NOT_OPEN_FOR_WRITE,
		NOT_ENOUGH_SPACE,
		FILE_IS_OPEN,
		PATH_IS_A_DIR,
		NO_SUCH_DIR,
		NO_SUCH_PATH,
		DIR_NOT_OPEN,
		FILE_NOT_OPEN,
		SEEK_FAILED,
		GENERAL_ERROR, // See system error
		GENERAL_ERROR_ON_CALLBACK, // See system error
		INVALID_ARGUMENT,
		CALLBACK_PENDING,
		NOT_A_DRIVE,
		DRIVE_NOT_READY,
		NOT_A_DIRECTORY,
		NOT_A_NAME,
		INVALID_URL,
		UNSUPPORTED_FSYS,
		CACHE_ERROR, // General cache error, detalis not available
		NOT_SAME_TYPES, // Source and dest files are not same types (file/dir)
		NO_SUPPORT, // The class does not support this call
		NOT_CALLBACK, // Not opened with callback flag
		COMM_ERROR // Unspecified communication error, see WarUserHandle errno
	} m_Errno;
	int m_SystemError;

	static void Throw(int Errno, int SystemError = 0);
};

// Base class for all file IO
// The idea is to make one consistent interface to
// all possible file systems/protocols.
// The class can be used by server and client programs,
// and allow copying of files across protocols.
class WarFile
{
public:
	WarFile(DWORD Flags);
	WarFile(WarUserHandle *phUser, user Client, QWORD ConnectionID, DWORD Flags);
	~WarFile();

	enum ProtocolMode
	{
		none,
		file,
		vfsys,
		cdfs,
		nfs,
		http,
		ftp,
		pop3,
		smtp,
		nntp,
		invalid
	} m_Protocol; // Class type
	static LPCSTR m_ProtocolNames[invalid];
	static WarFile * (*m_CreateFunc[invalid])(WarFile& File);

	enum SeekModes
	{
		begin,
		current,
		end
	};

	enum PositionModes
	{
		binary, // Physical offset
		logical, // Logical offset (ascii mapping)
	};

	enum NameModes
	{
		LogicalPath, //  /pub/cdrom1/test
		LogicalNameOnly, // test
		LogicalPathOnly, // /pub/cdrom1
		URL, // vfsys://pub/cdrom/test
		PhysicalPath, // c:\ftp\pub\cdrom1\test
		PhysicalNameOnly, // test
		PhysicalPathOnly, // c:\ftp\pub\cdrom1
		PhysicalDrive, // c:\ ..
		Protocol, // vfsys
	};

	enum // Flags - leftmost 4 bits reserved for overrides
	{
		// General
		callback		= 0x00000001,

		// Permission
		read			= 0x00000002,
		write			= 0x00000004,
		append			= 0x00000008,
		unlink			= 0x00040000, // Check for delete perms, don't delete the file/dir
		execute			= 0x00080000, // Check for execute/chdir perms

		// Sharing
		shread			= 0x00000010,
		shwrite			= 0x00000020,
		exclusive		= 0x00000040,

		// Mapping
		ascii			= 0x00000080,

		// Create mode
		create			= 0x00000100,
		notexist		= 0x00000200,
		mustexist		= 0x00000400,
		truncate		= 0x00000800,
		
		// Optimizing
		sequential  	= 0x00001000,
		temporary		= 0x00002000,
        test_only       = 0x00012000, // temporary | d_message - Del;ete at once if create

		// Type
		t_file			= 0x00004000,
		t_dir			= 0x00008000,

		// Directory
		d_message		= 0x00010000, // Read dir message if it exist

		// Special
		f_ratio			= 0x00020000	// Check ratio (download)
	};

	enum CallbackMode
	{
		c_none, // No callback pending
		c_open, // OnOpen() pending
		c_read, // OnRead() pending
		c_write, // OnWrite pending
		c_scan,
		c_stat,
		c_invalid
	} m_CallbackMode;

	user m_Client; // User to impersonate
	QWORD m_ConnectionID; // Connection to session manager
	WarUserHandle *m_phUser;
	DWORD m_Flags;
	WarString m_LogicalPath;
	static WarString m_DirMessagePattern;
	WarString m_DirMessage;
	BOOL m_bIsFree;
	
	WarBuffer m_Buffer; // Safe buffer for overlapped IO
	WarFileCallback *m_pCallback;
	
	int m_Errno; // File error
	int m_SystemErrno; // Protocol layer error

	void ClearErrors() 
		{m_Errno = m_SystemErrno = 0;}

	// Normal constructor
	WarFile *CreateForURL(LPCSTR URL, WarString& Path);

	// Install non-native file syste,
	static BOOL InstallFs(LPCSTR Name, WarFile * (*CreateFunc)(WarFile& File));

	// Virtual functions
	virtual BOOL Open(LPCSTR Path, DWORD Flags);
	virtual BOOL Close();
	virtual void DoDelete();
	virtual void DoMove(WarFile *pFile);
	virtual void Destroy() {delete this;} // Use in stead of delete!
	virtual BOOL IsOpen();
	virtual BOOL IsFree();
	virtual DWORD Read(LPVOID Buf, DWORD Bytes);
	virtual void Write(LPVOID Buf, DWORD Bytes);
	virtual DWORD Printf(LPSTR Format, ...);
	virtual void Flush();
	virtual void Seek(FLEN Offset, SeekModes Mode = begin);
	virtual void SeekToBegin();
	virtual void SeekToEnd();
	virtual FLEN GetLength(PositionModes Mode = binary);
	virtual void SetLength(FLEN NewLen);
	virtual FLEN GetPositon(PositionModes Mode = logical);
	virtual WarString GetFileName(NameModes Mode = LogicalNameOnly);
	virtual WarString GetFileTitle();
	virtual void Stat(PositionModes Mode, WarFStat &buffer);
	virtual void StatInfo(WarFStat &buffer, WarStatInfo &Info, BOOL bGetAllGroups);
	virtual BOOL MoveNext(); // Applies for directories only
	virtual BOOL Scan(LPCSTR Pattern, DWORD Flags, 
		WarFileList *pFileList = NULL); // Applies for open directories, use before Move*()
	virtual BOOL IsDir() {return m_Flags & t_dir;} // Test if an open file is a directory
	virtual BOOL IsFile() {return m_Flags & t_file;} // Test if an open file is a directory
	virtual BOOL CanRead(); // Test for read access
	virtual BOOL CanWrite(); // Test for write access
	virtual BOOL CanBrowse(); // Applies for directories only
	virtual void SetCallback(WarFileCallback *pCallback);
	virtual void DoDelete(LPCSTR Path);
	virtual BOOL IsSlow(); // True for internet protocols
	virtual BOOL IsCallback() {return m_Flags & callback;}
	virtual void SetLogicalPath(LPCSTR Path) {m_LogicalPath = Path;}
	virtual LPCSTR GetDirMessage();
	virtual BOOL SetDirMessagePattern(LPCSTR Pattern);
	virtual WarTime GetModifyTime();
	virtual BOOL GetFileCopyProgress(int& Progress) {Progress = 0; return FALSE;}
	
#ifdef WARSVRLIB
	static BOOL AccessCheck(user User, WarSocket *pSock, DWORD OpenFlags, DWORD Flags, int& Err, LPCSTR Path);
	virtual BOOL FinalPermsCheck(LPCSTR Path,
		FLEN Length, DWORD Flags, 
		BOOL bCheckBannedList, BOOL bExist);
#endif WARSVRLIB
	
	// Static functions
	static BOOL Delete(LPCSTR URL);
	static BOOL Stat(LPCSTR URL, PositionModes Mode, WarFStat *buffer);
	static BOOL Rename(LPCSTR URL, LPCSTR Name);
	static BOOL Move(LPCSTR SrcURL, LPCSTR DstURL);
	static BOOL Copy(LPCSTR ErcURL, LPCSTR DstURL);
	static BOOL UnixDirListing(WarArgs Args);
	static BOOL DosDirListing(WarArgs Args);
	static BOOL Chdir(LPCSTR Path); // Open the directory
};

#include "WarFileEx.h"


 