/* !!Copyright begin */
/*
War Software series
Copyright (C) 1996 - 1998 by Jarle (jgaa) Aase (www.jgaa.com)
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation version 2
of the License - WITH THE FOLLOWING RESTRICTIONS:

  * GOVERNMENTS, GOVERNMENTAL AGENCIES OR COMMERCIAL 
    COMPANIES OWNED 50% OR MORE BY THE GOVERNMENT ARE
    NOT ALLOWED TO USE THE SOFTWARE OR SORCE CODE FOR
    ANY PURPOSE. THIS RESTRICTION ALSO APPLIES FOR
    PROGRAMS DERIVED FROM THE SOURCE CODE, OR PROGRAMS/
    SOURCE CODE THAT USE SPECIAL TECHNICS/ALOGRITHMS
    INVENTED IN THIS SOFTWARE.

  * THE FULL SOURCE CODE IS WITHELD FROM A PUBLIC RELEASE
    UNTIL THE LINUX VERISON IS RELEASED.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
 
/* !!Copyright end */

// Implementation headers for War files

#ifdef WIN32

#define WarSystemFile WarDosFile

class WarDosFile : public WarFile
{
public:
	WarDosFile(WarFile &wFile);
	WarDosFile(WarFile &wFile, LPCSTR URLtag);
	WarDosFile(WarUserHandle *phUser, user Client, QWORD ConnectionID, DWORD Flags);
	~WarDosFile();

	HANDLE m_hFile;
	WarString m_Path;
	OVERLAPPED m_Overlapped;
	FLEN m_CurrentOffset;
	WIN32_FIND_DATA m_FindData;
	LPCSTR m_URLtag;

	unsigned m_bIsOpen : 1;
	unsigned m_bDeletePending : 1;

	static int m_OverlapCallbackCnt; // Total number of pending requests

	// Overrides
	virtual BOOL Open(LPCSTR Path, DWORD Flags);
	virtual BOOL Close();
	virtual void Destroy(); // Use in stead of delete!
	virtual BOOL IsOpen();
	virtual DWORD Read(LPVOID Buf, DWORD Bytes);
	virtual void Write(LPVOID Buf, DWORD Bytes);
	virtual void Flush();
	virtual void Seek(FLEN Offset, SeekModes Mode = begin);
	virtual void SeekToBegin();
	virtual void SeekToEnd();
	virtual FLEN GetLength(PositionModes Mode = binary);
	virtual void SetLength(FLEN NewLen);
	virtual FLEN GetPositon(PositionModes Mode = logical);
	virtual WarString GetFileName(NameModes Mode = LogicalNameOnly);
	virtual void Stat(PositionModes Mode, WarFStat &buffer);
	virtual void StatInfo(WarFStat &buffer, WarStatInfo &Info, BOOL bGetAllGroups);
	virtual BOOL MoveNext(); // Applies for directories only
	virtual BOOL Scan(LPCSTR Pattern, DWORD Flags, 
		WarFileList *pFileList = NULL); // Applies for open directories, use before Move*()
	virtual BOOL CanBrowse(); // Applies for directories only
	virtual void DoDelete(LPCSTR Path);
	virtual void DoDelete(); // Delete an open file
	virtual void DoMove(WarFile *pFile);
	static BOOL ExpandPath(LPCSTR Path);
	virtual WarTime GetModifyTime();
	static FLEN DoGetFreeDiskSpace(LPCSTR Path);

	static void CALLBACK OnIoCompletion(
		DWORD dwErrorCode, // completion code
		DWORD dwNumberOfBytesTransfered,  // number of bytes transferred
		LPOVERLAPPED lpOverlapped         // pointer to structure with I/O information
		);

protected:
	virtual void Reset();
};

#else
// Native UNIX/LINUX File system

#define WarSystemFile WarUnixFile

class WarUnixFile : public WarFile
{
public:
	WarUnixFile(WarFile &wFile);
	WarUnixFile(WarFile &wFile, LPCSTR URLtag);
	WarUnixFile(WarUserHandle *phUser, user Client, QWORD ConnectionID, DWORD Flags);
	~WarUnixFile();

	int m_fd;
	WarString m_Path;
	FLEN m_CurrentOffset;
	LPCSTR m_URLtag;

	unsigned m_bIsOpen : 1;
	unsigned m_bDeletePending : 1;

	// Overrides
	virtual BOOL Open(LPCSTR Path, DWORD Flags);
	virtual BOOL Close();
	virtual void Destroy(); // Use in stead of delete!
	virtual BOOL IsOpen();
	virtual DWORD Read(LPVOID Buf, DWORD Bytes);
	virtual void Write(LPVOID Buf, DWORD Bytes);
	virtual void Flush();
	virtual void Seek(FLEN Offset, SeekModes Mode = begin);
	virtual void SeekToBegin();
	virtual void SeekToEnd();
	virtual FLEN GetLength(PositionModes Mode = binary);
	virtual void SetLength(FLEN NewLen);
	virtual FLEN GetPositon(PositionModes Mode = logical);
	virtual WarString GetFileName(NameModes Mode = LogicalNameOnly);
	virtual void Stat(PositionModes Mode, WarFStat &buffer);
	virtual void StatInfo(WarFStat &buffer, WarStatInfo &Info, BOOL bGetAllGroups);
	virtual BOOL MoveNext(); // Applies for directories only
	virtual BOOL Scan(LPCSTR Pattern, DWORD Flags, 
		WarFileList *pFileList = NULL); // Applies for open directories, use before Move*()
	virtual BOOL CanBrowse(); // Applies for directories only
	virtual void DoDelete(LPCSTR Path);
	virtual void DoDelete(); // Delete an open file
	virtual void DoMove(WarFile *pFile);
	static BOOL ExpandPath(LPCSTR Path);
	virtual WarTime GetModifyTime();
	static FLEN DoGetFreeDiskSpace(LPCSTR Path);

protected:
	virtual void Reset();
};

#endif // system type