/* !!Copyright begin */
/*
War Software series
Copyright (C) 1996 - 1998 by Jarle (jgaa) Aase (www.jgaa.com)
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation version 2
of the License - WITH THE FOLLOWING RESTRICTIONS:

  * GOVERNMENTS, GOVERNMENTAL AGENCIES OR COMMERCIAL 
    COMPANIES OWNED 50% OR MORE BY THE GOVERNMENT ARE
    NOT ALLOWED TO USE THE SOFTWARE OR SORCE CODE FOR
    ANY PURPOSE. THIS RESTRICTION ALSO APPLIES FOR
    PROGRAMS DERIVED FROM THE SOURCE CODE, OR PROGRAMS/
    SOURCE CODE THAT USE SPECIAL TECHNICS/ALOGRITHMS
    INVENTED IN THIS SOFTWARE.

  * THE FULL SOURCE CODE IS WITHELD FROM A PUBLIC RELEASE
    UNTIL THE LINUX VERISON IS RELEASED.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
 
/* !!Copyright end */
// WarParser.h

#ifndef __PARSERH
#define __PARSERH

///////////////////////////////////////////////////////////////////////
// STokenTab

enum // What we want from lex
{
	LEX_WANT_CMD,
	LEX_WANT_STRING,
	LEX_WANT_BOOL,
	LEX_WANT_INT,
	LEX_WANT_CHAR
};

#define LEX_PARSE_QUOTES 0x1000

enum // General token ID's
{
	// Must be syncronized with LEX_WANT_*
	TOK_STRING = LEX_WANT_STRING, 
	TOK_BOOL, TOK_INT, TOK_CHAR,

	// Used internally
	TOK_INVALID,
	TOK_END_OF_TOKENS,
	// start of derived class enum ID
	TOK_START_TAB
};

typedef struct STokenTabDef
{
	int TokenID; // Token that identifies this node
	LPCSTR Token;
} STokenTab;

///////////////////////////////////////////////////////////////////////
// SCommandTtree

enum // General Command ID's
{
	// Return values
	CMD_PARSE_ERROR, 
	CMD_UNKNOWN_COMMAND,
	CMD_UNKNOWN_PARAMETER, 
	CMD_MISSING_PARAMETER,
	CMD_NOT_IMPLEMENTED,
	CMD_BAD_LEVEL,

	// Used internally
	CMD_INVALID,

	// start of derived class enum ID
	CMD_START_TREE
};

// Flags
#define CMDF_NOT_IMPLEMENTED	0x0001
#define CMDF_PARAMETER				0x0002
#define CMDF_NOPARAM					0x0004
#define CMDF_PARSE_CMD_LINE		0x0008

typedef struct SCommandTreeDef
{
	int TokenID; // Token that identifies this node
	int CommandID; // Command ID to execute
	int AccessLevel; // Access level to process
	int Flags; // Misc flags
	struct SCommandTreeDef *NextLevel;
	int LexTokenDelim;
	int LexWhatWeWant;
	LPCSTR CommandSyntax;
} SCommandTree;

///////////////////////////////////////////////////////////////////////
// CCommandParser
// Override to implement commands
// Call base class to process basic commands like "QUIT" and "HELP"

class WarArgs;

class WarCommandParser
{
public:
	SCommandTree *m_Commands;
	STokenTab *m_Tokens;

public:
	WarCommandParser();
	virtual int ParseCmd(LPCSTR CmdLine, WarArgs& Args, int Level);

public:
	virtual int yyParse(LPCSTR CmdLine, SCommandTree *CmdPtr, STokenTab *LexTab, WarArgs& Args);
	virtual int yyLex(LPCSTR *CmdLine, STokenTab *LexTab, int LexEOT, int Want, WarArgs& Args);
};

#endif __PARSERH

