/* !!Copyright begin */
/*
War Software series
Copyright (C) 1996 - 1998 by Jarle (jgaa) Aase (www.jgaa.com)
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation version 2
of the License - WITH THE FOLLOWING RESTRICTIONS:

  * GOVERNMENTS, GOVERNMENTAL AGENCIES OR COMMERCIAL 
    COMPANIES OWNED 50% OR MORE BY THE GOVERNMENT ARE
    NOT ALLOWED TO USE THE SOFTWARE OR SORCE CODE FOR
    ANY PURPOSE. THIS RESTRICTION ALSO APPLIES FOR
    PROGRAMS DERIVED FROM THE SOURCE CODE, OR PROGRAMS/
    SOURCE CODE THAT USE SPECIAL TECHNICS/ALOGRITHMS
    INVENTED IN THIS SOFTWARE.

  * THE FULL SOURCE CODE IS WITHELD FROM A PUBLIC RELEASE
    UNTIL THE LINUX VERISON IS RELEASED.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
 
/* !!Copyright end */

#define REG_TRYICONNAME "FTPDTrayIcon"
#define REG_TRYICONONLINE "FTPDUsersOnline"

class SessionMgr;
class SessionUserOnline;
class SessionUserSession;
class SessionMgr;
class SessionHostOnline;
class SessionServerOnline;


#include "WarSQLQuery.h"

// One entry for each file
class FileHistoryEntry
{
public:
	FileHistoryEntry();
	~FileHistoryEntry();

	static FileHistoryEntry *Get(LPCSTR URL);
	static FileHistoryEntry *Lookup(LPCSTR URL);
	static FileHistoryEntry *Add(LPCSTR URL);
	static int GetHash(LPCSTR URL);
	static void KillAll();
	
	void Rehash(int OldHashVal, int NewHashVal);
	void Unlink(); // Unlink one reference.

	enum FH_Flags
	{
		DELETED		= 0x00000001 // File is deleted, but still referenced
	};

	LPSTR m_pURL;
	in_addr m_Addr; // Addr of the last user that uploaded the file
	user m_User; // the last user that uploaded the file
	int m_HashVal;
	DWORD m_Flags;
	int m_References; // Can't be deleted while there are references
	DWORD m_DLcnt; // Download counter
	WarTime m_Time;

	static WarLinkedList **m_ppHash;
	static int m_TotalCnt;
	static int m_HashValue;
};

// One entry for each transfer
class SessionFileHistoryEntry
{
public:
	SessionFileHistoryEntry();
	~SessionFileHistoryEntry();

	static SessionFileHistoryEntry *Add(
		SessionUserSession *pSession, DWORD CPS, 
		FLEN Size, LPCSTR URL, BOOL bUpload,
		LPCSTR UserPath);

	enum SFE_flags
	{
		USED			= 0x00000001, // Used in FileHistoryInfoList to indicate that data is valid
		UPLOADED	= 0x00000002	// File is uploaded
	};

	FileHistoryEntry *m_pInfo;
	DWORD m_CPS; // CPS for the transfer
	FLEN m_Size;
	WarTime m_Time; // Time when the files was transferred
	DWORD m_Flags;
};

class CpsSample : public sesmgr_cps_sample
{
public:
	CpsSample();
	~CpsSample();

	void SetCapture(QWORD CPS, DWORD Sessions);
};

class SesCounters : public sesmgr_counters_info
{
public:
	SesCounters();
};

class WarOptions
{
public:
	WarOptions();
	~WarOptions();

	enum OptionDefs
	{
		// Core
		core_RPCPORT,
		core_RESNAME,
		core_PRIORITY,
		core_TRAYICON,
		core_WM_CLOSE,

		// Log
		log_LOGFILE,
		log_NTEVLOG,
		log_SOCKDATA,
		log_MASK,
		log_SYSLOG,
		log_LOGHSTBLEN,
		log_WULOGNAME,

		// Global socket options
		sock_revDNS,
		sock_MASTERTIMER,
		sock_CALLBK,
		sock_DNSserver,
		sock_WINSOCKDLL,

		// CDFS
		cdfs_IMAGEDIR,

		// Session manager
		sesmgr_HELO,
		sesmgr_TIMER,
		sesmgr_TIMER2,
		sesmgr_IPCHK,
		sesmgr_REFRESH,
		sesmgr_SESDELAY,
		sesmgr_HOSTDELAY,
		sesmgr_HACKERDELAY,
		sesmgr_CPSRCNT,
		sesmgr_CPSSAMPLE,
		sesmgr_CPSRES,
		sesmgr_FHASH,

		// ODBC
		odbc_SOURCE,
		odbc_USER,
		odbc_PASSWD,
		odbc_CONNECTRTR,

		// User database
		userdb_AUTOSAVE,
		userdb_EXPIERYSUSP,

		// FTP server
		ftpd_MTHREAD,
		ftpd_BINDDATA,
		ftpd_PASVDATA,
		ftpd_DOPORT,
		ftpd_DOPASV,
		ftpd_PASVWAIT,
		ftpd_HELO,
		ftpd_SVRHELO,
		ftpd_SHTDASAP,
		ftpd_PRELOGINPOLL,
		ftpd_DATA,
		ftpd_PRELOGIN,
		ftpd_NOOP,
		ftpd_CPSSTEST,
		ftpd_CPSSAMPLE,
		ftpd_CPSSUSPEND,
		ftpd_BUFSIZ,
		ftpd_DIRMSG,
		ftpd_MSGNUM,

		// Compatibility issues
		comp_FLEXPATH,

		// End
		invalid
	};

	LPCSTR GetOption(LPCSTR Name);
	LPCSTR GetOption(int Index);
	void SetOption(LPCSTR Name, LPCSTR NewValue);
	void SetOption(int Index, LPCSTR NewValue);
	int Lookup(LPCSTR Name);
	void Load();
	void Save();

	LPCSTR operator [] (int Index) {return GetOption(Index);}

protected:
	WarPtrArray m_Options;
};

class HostDnsLookup : public WarSocket
{
public:
	HostDnsLookup(SessionHostOnline *pOrigin) {m_pOrigin = pOrigin;}
	virtual void OnAsyncGethostbyaddr(hostent *hp, int Errno);
	
	SessionHostOnline *m_pOrigin;
};

// A host that have one or several users logged on
class SessionHostOnline
{
public:
	SessionHostOnline();
	~SessionHostOnline();

	static void DropAll();
	BOOL IsReadyForLogin() {return m_bReadyForLogin;}
	static SessionHostOnline *GetHost(in_addr Host);
	static SessionHostOnline *Lookup(in_addr Host);
	void AddUserSession(SessionUserSession *pUserOnline);
	void RemoveUserSession(SessionUserSession *pUserOnline);

	BOOL m_bHaveDomainName;
	BOOL m_bReadyForLogin;
	HostDnsLookup *m_pDomainLookupSocket;
	in_addr m_Host;
	WarString m_DomainName;
	WarLinkedList m_Sessions; // SessionUserSession *
	DWORD m_Flags; // Flags that apply for all users on this IP
	static WarLinkedList m_Hosts;
	BOOL m_bIsLocal;
	WarString m_InterfaceName;
	in_addr m_SubnetMask;
	SesCounters m_Counters;
	WarTimer m_ExpiryTimer;
	WarTimer m_HackerTimer;
};


// shared handle for one or several user
// sessions from one user
class SessionUserOnline
{
public:
	SessionUserOnline(user User);
	~SessionUserOnline();

	static SessionUserOnline *GetUser(user User);
	static SessionUserOnline *Lookup(user User);
	void AddSession(SessionUserSession *pUS);
	void RemoveSession(SessionUserSession *pUS);
	void Kill();
	static void DropAll();

	user m_User;
	WarLinkedList m_Sessions; // SessionUserSession 
	WarLinkedList m_Files; // FileHistoryEntry

	// Session data cached
	DWORD m_Flags; // Flags that apply for all sessiosn on this user
	WarTimer m_SessionDelay;
	SesCounters m_Counters;
	static WarLinkedList m_List;
};

// Servers that are currently online and using 
// the session manager
class SessionServerOnline
{
private:
	SessionServerOnline();

public:
	~SessionServerOnline();
	in_addr GetServerAddr();
	BOOL Kill();
	static SessionServerOnline *Lookup(user ServerNode, war_svr_type Type);
	static SessionServerOnline *Register(user ServerNode, war_svr_type Type);
	void AddSession(SessionUserSession *psession);
	void RemoveSession(SessionUserSession *psession);

	in_addr m_Addr;
	int m_Port;
	WarString m_Name; // Server name
	WarString m_NodeName;
	war_svr_type m_Type; // Server type
	int m_State; // State of the server
	DWORD m_Flags; // Flags that apply for all users on this server
	user m_User; 
	static WarLinkedList m_List; // *SessionServerOnline
	WarLinkedList m_Sessions; // SessionUserSession
	SesCounters m_Counters;
	CpsSample *m_pCPS;
	
	WarLinkedList m_Files; // FileHistoryEntry
};


// Databuffer used for larger data transfers to
// and from the server
class SessionDataBuffer
{
public:
	enum BufferType
	{
		BT_BINARY,
		BT_TEXT
	} m_Type;

	enum BufferDirection
	{
		BD_TO_SERVER,
		BD_FROM_SERVER
	} m_Direction;

	SessionDataBuffer(BufferType Type, BufferDirection Direction, LPCSTR FileName);
	~SessionDataBuffer();

	BOOL ToFile(LPCSTR FileName); // Save to file
	BOOL FromFile(LPCSTR FileName); // Load from file
	BOOL OpenFile();
	int GetLength() {return m_Buffer.GetLength();}
	operator LPCSTR () {return (LPCSTR)m_Buffer;}
	int GetID() {return m_ID;}
	
	FILE *m_FP;	
	WarString m_Buffer; // Used in text or binary mode
	WarString m_FileName; // File system file name
	int m_ID; // Unique identifier
	static int m_NextID; // Next unique ID
};

// session data for a single session
class SessionUserSession
{
public:
	SessionUserSession();
	~SessionUserSession();

	static SessionUserSession *AddNew(user User,
		wdq_mode AccessTo,
		in_addr Domain, // Login to this IP
		in_addr Host); // Connecting from host)

	void SQLAddSession();
	void SQLCloseSession();
	static void DropAll();
	void SetState(int State, LPCSTR Info);
	user GetUserID() {return m_pUser->m_User;}
	in_addr GetHostAddr();
	int CreateSessionBuffer(WarString& wBuf);
	QWORD GetSessionID();
	static SessionUserSession *LookupID(QWORD ID);
	DWORD GetFlags(); // Collate relevant flags
	void UpadateTransferStats(BOOL bUpload, BOOL bFree, int GiveOrTake, FLEN Bytes);
	void SetCPS(DWORD CPS);
	void FreeQueryList();
	BOOL GetUDRatioData(int Files, FLEN Bytes, sesmgr_ud_restrictions_data *pData);

	BOOL m_bIsOperator; // RPC login user
	BOOL m_IsAnonymous;
	WarTime m_LoggedOn; // Time of logon
	WarTime m_LastAccess; // Last request
	WarTimer m_TimeOut; // Timeout before the connection reset
	SessionHostOnline *m_pHost; // User host
	SessionUserOnline *m_pUser; // User
	SessionServerOnline *m_pServer; // Connected to server
	SessionDataBuffer *m_pBuffer;
	WarLinkedList m_SessionQueryList; // Used by usrproc_sesmgrquery_1_svc
	DWORD m_SessionIndex; // Index to m_Sessions
	QWORD m_SessionID; // Unique number
	QWORD m_SessionKey; // Auth key (random number)

	// General info set by service
	int m_State; // Current state of the connection
	WarString m_LoginName; // Visual login name (set by session manager)
	WarString m_WuLoginName; // Login name ofr wu-log
	WarString m_StateStr; // Current state as a text string
	WarString m_CWD; // Current working directory
	WarString m_CurrentFilePath;
	WarString m_CurrentFileURL;
	FLEN m_CurrentFileLen;
	FLEN m_CurrentFilePos;
	int m_Progress;
	int m_CopyProgress;
	WarTimer m_IdleTime; // Idletime in 1/10 seconds
	int m_IdleTimeLimit; // Idle time limit in seconds
	WarString m_LastCMD; /* Last command given */
	unsigned int m_CPS; /* Bytes / second */
	unsigned int m_PeriodPeekCPS; // Peek CPS between readings
	war_auth_def_100 *m_pAuth;
	DWORD m_Flags;
	SesCounters m_Counters;
	static QWORD m_SessionIDCnt;
	static WarPtrArray m_Sessions; // Index of all allocated SessionUserSession objects
	FLEN m_SessionCredit;

#if USE_SQL
	DWORD m_SQL_ID;
#endif // USE_SQL

	WarLinkedList m_Files; // SessionFileHistoryEntry
};


class SessionMgr
{
public:
	SessionMgr();
	~SessionMgr();

	BOOL Initialize();
	void InitializeODBC();
#if USE_SQL
	void SQLLogError(LPCSTR Module, LPCSTR Msg, SQLSMALLINT HandleType, SQLHANDLE Handle);
#endif // USE_SQL

	war_user_login_ok ClientLogin(war_client_login_req *pReq, int *pErr);
	
	SessionUserSession *Login(wdq_mode m_AccessTo, // To service
		in_addr m_Domain, // Login to this IP
		in_addr m_Host, // Connecting from host
		LPCSTR ID, // User ID
		LPCSTR Passwd, // User password
		user_node_type UserType = UT_USER,
		SessionServerOnline *pServer = NULL,
		BOOL bTestOnly = FALSE); 

	SessionUserSession *RPCLogin(wdq_mode m_AccessTo, // To service
		in_addr m_Domain, // Login to this IP
		in_addr m_Host, // Connecting from host
		LPCSTR ID, // User ID
		LPCSTR Passwd, // User password
		user_node_type UserType,
		war_svr_type ServerLogonType,
		war_auth_mode m_aMode,
		war_auth_crypt_mode m_aEncrLevel);

	BOOL Validate(war_auth_def_100& Auth, 
		in_addr iAddr, user *pUserID, 
		SessionUserSession **ppSession = NULL);

	SessionUserSession *GetSessionFromID(QWORD ID);

	LPCSTR GetOption(WarOptions::OptionDefs OptCode)
		{return m_Options[(int)OptCode];}
	
	// Security functions
	BOOL IsTimeOfDayAllowed(WarUserNode *pUser, WarTime& CurrentTime);
	BOOL ValidateDomain(WarUserNode *pUser, LPCSTR DomainName, in_addr& IPnum);

	// Low level security
	BOOL DomainInList(LPCSTR DomainName, in_addr& IPnum, LPCSTR PatternList);
	BOOL IPVerifyRule(LPCSTR Rule,int **IPmask);
	void IncFailLogin(WarUserNode *pUser, SessionServerOnline *pServer);

	// Misc
	BOOL EnumerateIP();
	void NotifyUpdates();
	static WarString LogSupportGetUserInfo(user User);
	void WuLog(sesmgr_transfer_done_req *Arg, SessionUserSession *pSession);

	static int OnGlobalTimer(LPVOID lParam);
	void OnTimer();
	WarTimer m_MinuteTimer;
	WarTimer m_IPcheckTimer;

	WarTime m_StartupTime;
	WarString m_OS;
	DWORD m_Flags; // Flags that apply for all users
	int m_Errno;
	DWORD m_SessionCnt;
	WarOptions m_Options;
	WarLinkedList m_IPinfo; // Points to HOST list entries, one for each IP used by the machine, including RAS.
	WarTimer m_IPinfoChanged; // Last IP scan where info changed
	WarTimer m_CPStimer;
	SesCounters m_Counters;
	SesCounters m_Operators;
	CpsSample *m_pCPS;
	WarLinkedList m_Sessions; // SessionUserSession - all sessions
	FILE *m_pWuLog;
	WarString m_WuCurrentLogName;

#if USE_SQL
	WarSQLQuery m_SQL;
	static void SQLGetUserHook(WarSQLCol& Col);
#endif

	static void SvrStartShutdown(); // Callback

protected:
#ifdef WIN32
	HANDLE m_hServerSemaphore;
	HKEY m_RegKey;
	HANDLE m_hServerEvent;
	HANDLE m_hGlobalSvrEvent;
#endif
};

EXT SessionMgr *pSesMgr;
#define SesMgr (*pSesMgr)

 