/* !!Copyright begin */
/*
War Software series
Copyright (C) 1996 - 1998 by Jarle (jgaa) Aase (www.jgaa.com)
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation version 2
of the License - WITH THE FOLLOWING RESTRICTIONS:

  * GOVERNMENTS, GOVERNMENTAL AGENCIES OR COMMERCIAL 
    COMPANIES OWNED 50% OR MORE BY THE GOVERNMENT ARE
    NOT ALLOWED TO USE THE SOFTWARE OR SORCE CODE FOR
    ANY PURPOSE. THIS RESTRICTION ALSO APPLIES FOR
    PROGRAMS DERIVED FROM THE SOURCE CODE, OR PROGRAMS/
    SOURCE CODE THAT USE SPECIAL TECHNICS/ALOGRITHMS
    INVENTED IN THIS SOFTWARE.

  * THE FULL SOURCE CODE IS WITHELD FROM A PUBLIC RELEASE
    UNTIL THE LINUX VERISON IS RELEASED.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
 
/* !!Copyright end */

#ifndef ECONNABORTED
#define EWOULDBLOCK	WSAEWOULDBLOCK	
#define EINPROGRESS	WSAEINPROGRESS	
#define EALREADY	WSAEALREADY	
#define ENOTSOCK	WSAENOTSOCK	
#define EDESTADDRREQ	WSAEDESTADDRREQ	
#define EMSGSIZE	WSAEMSGSIZE	
#define EPROTOTYPE	WSAEPROTOTYPE	
#define ENOPROTOOPT	WSAENOPROTOOPT	
#define EPROTONOSUPPORT	WSAEPROTONOSUPPORT	
#define ESOCKTNOSUPPORT	WSAESOCKTNOSUPPORT	
#define EOPNOTSUPP	WSAEOPNOTSUPP	
#define EPFNOSUPPORT	WSAEPFNOSUPPORT	
#define EAFNOSUPPORT	WSAEAFNOSUPPORT	
#define EADDRINUSE	WSAEADDRINUSE	
#define EADDRNOTAVAIL	WSAEADDRNOTAVAIL	
#define ENETDOWN	WSAENETDOWN	
#define ENETUNREACH	WSAENETUNREACH	
#define ENETRESET	WSAENETRESET	
#define ECONNABORTED	WSAECONNABORTED	
#define ECONNRESET	WSAECONNRESET	
#define ENOBUFS	WSAENOBUFS	
#define EISCONN	WSAEISCONN	
#define ENOTCONN	WSAENOTCONN	
#define ESHUTDOWN	WSAESHUTDOWN	
#define ETOOMANYREFS	WSAETOOMANYREFS	
#define ETIMEDOUT	WSAETIMEDOUT	
#define ECONNREFUSED	WSAECONNREFUSED	
#define ELOOP	WSAELOOP	
#define EHOSTDOWN	WSAEHOSTDOWN	
#define EHOSTUNREACH	WSAEHOSTUNREACH	
#define HOST_NOT_FOUND	WSAHOST_NOT_FOUND	
#define TRY_AGAIN	WSATRY_AGAIN	
#define NO_RECOVERY	WSANO_RECOVERY	
#define NO_DATA	WSANO_DATA	
#endif // Error codes

class WarSocket;


// We need thread local storage (TLS) for 'static'
// buffers.

enum cli_callbc_command // RPC UDP client callback
{
	CLICC_ABORT, // Give up
	CLICC_WAIT, // Wait a little longer
	CLICC_TRY_AGAIN // Repeat request
};

class WarCriticalSection
{
public:
	WarCriticalSection();
	~WarCriticalSection();
	void Lock();
	void Unlock();
#ifdef _DEBUG
	int m_NumLocks;
#endif

private:
#ifdef WIN32
	CRITICAL_SECTION m_CriticalSection;
#endif
	BOOL m_bIsInitialized;
};

class WarSocketTLS
{
public:
	~WarSocketTLS();
	static WarSocketTLS *Ptr();
	cli_callbc_command (*m_CallbackTimeout)(timeval& tv, void *lParam); /* Return FALSE to give up */
	void (*m_CallbackAllWell)(void *lParam);
	void *m_cli_callbc_lParam; // User supplied parameter to timeout callback
	WarTimer m_cli_callbc_Timer1;
	WarTimer m_cli_callbc_Timer2;
	static void ResetRPCTimers();

	// WarSocket
	WarLinkedList SocketList;
	fd_set readfds, writefds, excepfds;
	WarSocket **pSckRd, **pSckWr, **pSckEx;
	int NumFdsRd, NumFdsWr, NumFdsEx, MaxFds;
	DWORD TotalSockets;
	DWORD CurrentSockets;
	int NumSets;
	WarTimer Timer;
	BOOL IsInitialized;
	BOOL IsShuttingDown;
	WarLinkedList SuspendList;
	WarLinkedList NotifyTimerList;
#ifdef _WINDOWS
	HWND hWndSocket;
#endif

private:
	static DWORD m_dwTLSIndex;
	static BOOL m_bTLSIsInitialized;
	WarSocketTLS(); // Constructed in WarSocketTLS::Ptr()
};



class WarDNSSocket;

enum ConnectionStates// states m_State
{
		PRELOGIN, 
		GOTNAME, 
		PROCESS, 
		HOLD, 
		GETFILE, // Upload
		PUTFILE, // Download
		QUITTING,
		STATE_INVALID
};

class WarSocket
{
public:
	WarSocket();
	~WarSocket();

	static BOOL Initialize(); // Called by WarSocket()
	static BOOL Shutdown(); // Called when the last socket is deleted
	static void StartShutdown();
	static void PauseServer(BOOL DoPause) {;}
	virtual BOOL Create(int SocketType, LPCSTR Name);
	virtual BOOL Listen(LPCSTR Domain, int Port);
	static BOOL Process_select(DWORD msTimeout, DWORD msCBTimeou);
	static void select_AssignFds();
	static  int OnGlobalTimer();
	virtual BOOL OnAccept(int ErrorCode, WarSocket *pSocket = NULL);
	virtual DWORD Send(LPCSTR Buffer, DWORD Bytes);
	virtual BOOL OnSend(int Errno);
	virtual int Receive(LPSTR Buffer, DWORD Bytes);
	virtual BOOL OnReceive(int Errno);
	virtual BOOL Connect(LPCSTR Domain, int Port, long BindToHost = INADDR_ANY, int BindToPort = 0);
	BOOL Connect(sockaddr& Addr, long BindToHost, int BindToPort);
	virtual BOOL OnConnect(int Errno);
	virtual int SendFile(FILE *fp);
	virtual int OnSendFile(int Type);
	virtual BOOL OnError(int ErrorCode);
	virtual BOOL Close();
	virtual void Kill();
	virtual BOOL SetBlockingMode(BOOL Mode);
	virtual BOOL EnableNagleDelay(BOOL Mode); // TCP_NODELAY	
	virtual BOOL AsyncGethostbyaddr(LPCSTR addr, int len, int type, int TimeOut);
	virtual void OnAsyncGethostbyaddr(hostent *hp, int Errno);
	virtual int sendto (LPCSTR buf, int len, int flags, const sockaddr *to, int tolen);
	virtual int recvfrom (LPSTR buf, int len, int flags, struct sockaddr * from, int *fromlen);
	virtual void OnCallback(int CallbackID, BOOL Success, LPVOID Param) {;}
	virtual void IgnoreErrors(BOOL State);
	virtual BOOL IsConnected() 
		{return ((CurrentState >= CONNECTED) && (CurrentState < DEAD));}
	virtual void OnTimer() {;}
	static LPCSTR GetNetworkInfo() {return m_NetworkInfo;}
	void SetTimerInterval(int ms) {TimerInterval = ms;}
#ifdef WIN32
	static LRESULT CALLBACK WarSocketWndProc(HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam);
#endif
	virtual user GetUserID() {return 0;}
	virtual QWORD GetSesID() {return 0;}

	BOOL LogToFile(BOOL bEnable);
	BOOL IsLoggingToFile();

	QWORD GetBytesSent() {return m_BytesSent;}
	QWORD GetBytesReceived() {return m_BytesReceived;}
	QWORD GetBytesTransfered() {return m_BytesSent + m_BytesReceived;}

	in_addr GetPeerAddr();
	in_addr GetHostAddr();
#if defined(UNIX) || defined(GNUCPP)
	void UpdateMaxFds() {WarSocketTLS::Ptr()->MaxFds = max(WarSocketTLS::Ptr()->MaxFds -1, Socket) +1;}
#elif defined(WIN32) || defined(MSVC)
	void UpdateMaxFds() {WarSocketTLS::Ptr()->MaxFds = 1;}
#elif
#error "not handled"
#endif
	static void ResetMaxFds() {WarSocketTLS::Ptr()->MaxFds = 0;}

	LPCSTR GetName() {return Name;}
	int GetMyFamily();
	LPCSTR GetMyFamilyName();
	int GetMyProtocol();
	LPCSTR GetMyProtocolName();
	float GetElapsedTime(BOOL bForce = FALSE);
	float GetCPS();
	float GetRelativeCPS();
	
	static BOOL GetHostAddr(sockaddr& Addr, LPCSTR Host, int Port);
	static DWORD GetAddr (LPCSTR Host);


	enum SocketStates
	{
		// States
		NOSTATE = 0,
		CREATED,
		LISTEN,
		CONNECTING,
		CONNECTED,
		UDP_WAITING,
		DEAD
	} CurrentState;

	enum SoketModes
	{
		// Modes
		DATAGRAM = 0,
		STREAM,
		ASSIGNONCONNECT
	} SocketMode;

	enum SocketProcessMethods
	{
		// Process methods
		SELECT_METHOD
	} ;

	enum SpecialErrorCodes
	{
		SCKERR_SUSPEND = -10
	};

	// Suspend support
	enum SocketSuspendMode
	{
		SUSPEND_NOT,
		SUSPEND_CPS,
		SUSPEND_SMG,
		SUSPEND_CPY,
	} m_SuspendMode;
	WarTimer m_SuspendTimeout;
	BOOL Suspend(SocketSuspendMode Reason, DWORD msTimeout,...);
	virtual BOOL OnSuspendTimeout();
	virtual void ActivateAfterSuspend();
	unsigned OpenForReadSave : 1;
	unsigned OpenForWriteSave : 1;
	unsigned ClearToSendSave : 1;
	unsigned ClearToReceiveSave : 1;

	sockaddr m_PeerSck;
	sockaddr m_LocalSck;

	// CPS control
	float m_CPS;
	float m_CPSTotal;
	DWORD m_CPSlimit;
	FLEN m_CPSsampleBytes; // Bytes at last sample
	FLEN m_CPSxmitBytes; // Total bytes for socket + children
	WarSocket *m_pCPSSocket; // Use this socket for CPS
	WarTimer m_CPSSampleTime;
	int m_CPSSampleTimeout; // Milliseconds
	int m_CPSSuspendTimeout; // Milliseconds
	int m_CPSTestTimeout; // Milliseconds

	int xp_addrlen; /* length of remote address */
	struct sockaddr_in xp_raddr;	 /* remote address */

	WarString Name;
	QSWORD m_SessionID; // Used to identify sessiosn in the log 
	DWORD m_SesChldCnt; // Counter for child sockets, used for naming

	// Server shutdown support
	static void (*m_SvrStartShutdown)();

protected:
	SOCKET Socket;

	unsigned OpenForRead : 1;
	unsigned OpenForWrite : 1;
	unsigned ClearToSend : 1;
	unsigned ClearToReceive : 1;
	unsigned CloseIsPending : 1; // receive() has returned 0
	unsigned DestroyWhenDone : 1; // Delete when send buffer is empty
	unsigned SocketIsBound : 1; // Socket is bound
	unsigned AsyncMode : 1;
	int m_bIgnoreErrors;
	

	QSWORD m_BytesSent;
	QSWORD m_BytesReceived;

	DWORD ConnectionID;

	WarTimer CreatedTime;
	WarTimer ConnectedTime;
	WarTimer LastAccessTime;
	WarTimer Timer;
	int	TimerInterval; // Set to -1 to execute OnTimer() at next check

	WarDNSSocket *m_pDNSSock;
	WarString m_DNSname;

	static int DoLoop;

	static void SelectMethod(SocketProcessMethods Method = SELECT_METHOD);
	static BOOL (*Process)(DWORD msTimeout, DWORD msCBTimeout);
	static void (*ProcessOnDestroy)();
	BOOL CheckCPS(BOOL bDoThrow, WarSocket *pOrigin = NULL);
	FILE *m_fpLog;

#ifdef _WINDOWS
	HANDLE m_LookupHandle;
#endif

public:
	static WarString m_NetworkInfo;
	static unsigned int m_MaxSockets;
	SOCKET GetSocket() {return Socket;}
	int DoProcess(DWORD msTimeout, DWORD msCBTimeout) {return Process(msTimeout, msCBTimeout);}

	static WarCriticalSection *m_pLock;
	static void Lock();
	static void Unlock();
};

// Used to notify some event between worker threads/processes
// and a server thread. This creates a little overhead,
// but makes a clean and portable interface.
// Only one WarSocketNotification object is needed for each
// thread that has it's own adress space
// 
// The server execute a worker thread, and add's an event
// to the threads list of pending events. When the worker
// thread has finished the task, it sends a notification back,
// referring to the event ID. It then awaits an ack from the
// server thread. If a timeout occures, the event is re-sent.
// The event ID is a sequence number and a table index.
// The server also set a timeout on the task. If the timeout
// is triggered, the event is destroyed, the dispatch routine
// called with an error code, and the thread eventually
// killed.
// The same WarSocketNotification class is used on both sides.
// The server side object can handle any number of pending 
// events, while the client side only can handle one.

class WarSocketNotificationEvent
{
public:
	DWORD	m_Seq; // Unique sequence number
	WarTimer m_Timeout;
	void (*m_Dispatch)(LPVOID Param, LPVOID Arg, int Errno);
	LPVOID m_Param; // Param to send to dispatch routine
	DWORD m_Flags;
	DWORD m_Auth[4]; // Random auth number to autenticate sender
};

// Message sent to notify about an event
struct WSEEventMsg
{
	DWORD m_IndexID;
	DWORD	m_Seq; // Unique sequence number
	DWORD m_Flags;
	DWORD m_Length;
	int	m_Error;
	LPSTR AuthKey[16]; // Random auth number to autenticate sender
	char buf[512]; // Buffer for arguments
};

class WarSocketNotification : WarSocket
{
public:
	WarSocketNotificationEvent *m_Events; // Any number of events
	DWORD m_NumAllocated;
	DWORD m_NumUsed;
	static DWORD g_NextSeqNumber;

	int AddEvent(LPVOID Param,
		void (*Dispatch)(LPVOID, LPVOID, int),
		DWORD Flags,
		DWORD msTimeout = 5 * 60 * 1000); // Server side creation of event
															// Returns Index number

	BOOL Notify(WSEEventMsg& Msg); // Client --> Server

	WarSocketNotification();
	~WarSocketNotification();
	virtual BOOL OnReceive(int Errno);	// Called when we receive events
	virtual void OnSocketTimer();					// Called now and then...

	// Low level
	void DestroyEvent(DWORD Index, DWORD Seq);
	BOOL AllocateEvent(DWORD& Index, DWORD& Seq);
	void CreateAuthKey(char **Auth, int Len); // Create a random key
};

class WarSocketException
{
public:
	static void _Throw(
		LPCSTR _File, 
		int _Line, 
		WarSocket *pSck, 
		int ErrID,
		LPCSTR pMsg);

	void Tell();

	WarSocket *pSocket;
	int ErrorCode;
	LPCSTR ErrorMessage;
	int Line;
	LPCSTR File;

};

#define ThrowSchExc(Sck, _ErrID, Msg) WarSocketException::_Throw(__FILE__, __LINE__, Sck, _ErrID, Msg)


#if TESTCODE
class WarTestSvrSocket : public WarSocket
{
public:
	WarTestSvrSocket();
	~WarTestSvrSocket();
	BOOL ServerTest(DWORD NumBytes, LPCSTR Domain, int Port);
	virtual BOOL OnAccept(int ErrorCode, WarSocket *pSocket);
	virtual BOOL OnSend(int Errno);

	static QSWORD BytesToSend;
};

class WarTestClientSocket : public WarSocket
{
public:
	~WarTestClientSocket();
	static BOOL ClientTest(DWORD NumConnections, LPCSTR Domain, int Port);
	virtual BOOL OnConnect(int Errno);
	virtual BOOL OnReceive(int Errno);
	virtual void Kill();
};
#endif // TESTCODE


// Timer to be called from WarSocket processing engine
class WarNotifyTimer
{
public:
	static WarNotifyTimer *Add(BOOL (*Callback)(LPVOID Param), LPVOID Param, QWORD msTimeout = 1000);
	static void Remove(BOOL (*Callback)(LPVOID Param));
	~WarNotifyTimer();
	void SetTimerInterval(QWORD ms);

	static void OnTimer(); // Called by WarSocket

protected:
	WarNotifyTimer();
	
	WarTimer m_Timer;
	QWORD m_msTimeout;
	LPVOID m_Param;
	BOOL (*m_Callback)(LPVOID Param);
};


 