/* !!Copyright begin */
/*
War Software series
Copyright (C) 1996 - 1998 by Jarle (jgaa) Aase (www.jgaa.com)
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation version 2
of the License - WITH THE FOLLOWING RESTRICTIONS:

  * GOVERNMENTS, GOVERNMENTAL AGENCIES OR COMMERCIAL 
    COMPANIES OWNED 50% OR MORE BY THE GOVERNMENT ARE
    NOT ALLOWED TO USE THE SOFTWARE OR SORCE CODE FOR
    ANY PURPOSE. THIS RESTRICTION ALSO APPLIES FOR
    PROGRAMS DERIVED FROM THE SOURCE CODE, OR PROGRAMS/
    SOURCE CODE THAT USE SPECIAL TECHNICS/ALOGRITHMS
    INVENTED IN THIS SOFTWARE.

  * THE FULL SOURCE CODE IS WITHELD FROM A PUBLIC RELEASE
    UNTIL THE LINUX VERISON IS RELEASED.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
 
/* !!Copyright end */

#ifdef PtrToInt
#undef PtrToInt
#endif

#ifdef IntToPtr
#undef IntToPtr
#endif

inline BOOL IsSlash(char ch)\
{ return (ch == '/') || (ch == '\\');}


QSWORD atoq(LPCSTR Str, int base = 0);
class WarString;

class WarBuffer
{
public:
	WarBuffer();
	WarBuffer(LPCSTR val);
	WarBuffer(WarBuffer& Val);
	~WarBuffer();

protected:
	LPSTR buffer;
	size_t buf_len;
	size_t buf_used;

	enum
	{
		DEFAULT_SIZE = 32,
	};

public:

	int buf_free() {return (buf_len - buf_used) - 1;}
	int buf_free(int len);
	inline void terminate() {buffer[buf_used] = 0;}
    int GetLength() const {return buf_used;}
    int GetBufferLength() const {return buf_len;} 
    BOOL IsEmpty() const {return GetLength() == 0;} 
	LPSTR GetBuffer(int MinLen);
	void Expand(size_t Len, BOOL Exact = FALSE);
	void AddData(LPCVOID Data, const size_t Len, const BOOL Exact = FALSE);
	LPVOID AllocBuffer(size_t Len, BOOL Exact = FALSE);
	int PtrToInt(LPVOID Ptr);
	LPVOID IntToPtr(int Offset);
	void AddChar(char ch);
	void AddStr(LPCSTR str);
	void Clear();
#ifdef _DEBUG
#define ReleaseOwnership() __ReleaseOwnership(THIS_FILE,__LINE__)
	LPSTR __ReleaseOwnership(LPCSTR File, int Line);
#else
#define ReleaseOwnership() __ReleaseOwnership()
	LPSTR __ReleaseOwnership();
#endif
	int do_divq(QWORD& n,int base);
	int do_divi(int& n,int base);
	int do_divi(DWORD& n,int base);
	LPCSTR itoa(int val, LPSTR buf, int base, BOOL LargeDigits = TRUE);
	LPCSTR itoa(DWORD val, LPSTR buf, int base, BOOL LargeDigits = TRUE);
	LPCSTR itoa(QWORD val, LPSTR buf, int base, BOOL LargeDigits = TRUE);
	LPSTR Reverse(LPSTR Buf, int len);
	char operator [] (int Index);

	operator LPCSTR () const {return buffer ? buffer : NULL;}

protected:
	void AddIt(int Val);
	void AddIt(DWORD Val);
	void AddIt(QWORD& Val);
	void AddIt(LPCSTR Val);
	void AddIt(WarString& Val);
	void SetIt(DWORD Val);
	void SetIt(QWORD& Val);
	void SetIt(LPCSTR Val);
	void SetIt(const WarString& Val);

};

class WarPtrArray : public WarBuffer
{
public:
	WarPtrArray();
	LPVOID operator [] (int Index);
	int AddNew(LPVOID lParam);
	void SetAt(int Index, LPVOID lParam);
	void Release(int Index);
	int GetItemCount();

private:
	size_t m_Used;
};

class WarString : public WarBuffer
{
public:
	WarString();
	WarString(LPCSTR Text);
	WarString(const WarString& String);
	void operator = (char ch) {Clear(); AddChar(ch);}
	void operator += (char ch) {AddChar(ch);}
	void operator = (int val) {SetIt(val);}
	void operator += (int val) {AddIt(val);}
	void operator = (DWORD val) {SetIt(val);} 
	void operator += (DWORD val) {AddIt(val);}
	void operator = (QWORD val) {SetIt(val);}
	void operator += (QWORD val) {AddIt(val);}
	void operator = (LPCSTR val) {SetIt(val);}
	void operator += (LPCSTR val) {AddIt(val);}
	void operator = (const WarString& val) {SetIt(val);}
	void operator += (WarString& val) {AddIt(val);}
	BOOL operator == (LPCSTR Val) 
		{return buf_used ? (strcmp(buffer, Val) == 0) : FALSE;}
	BOOL operator != (LPCSTR Val) 
		{return buf_used ? (strcmp(buffer, Val) != 0) : FALSE;}

	void operator << (char ch) {AddChar(ch);}
	void operator << (int val) {AddIt(val);}
	void operator << (DWORD val) {AddIt(val);}
	void operator << (QWORD val) {AddIt(val);}
	void operator << (LPCSTR val) {AddIt(val);}
	void operator << (WarString& val) {AddIt(val);}
	void operator << (in_addr& addr) {MakeInAddr(addr);}
	void operator << (sockaddr *addr) {MakeSockAddr(addr);}
	void operator << (sockaddr& addr) {MakeSockAddr(&addr);}

	char operator [] (int Index) {return WarBuffer::operator [] (Index);}
	operator LPCSTR () const {return buffer ? buffer : "";}

	BOOL InPattern(LPCSTR Pattern);
	int Stricmp(LPCSTR Tag);

	int Printf(LPCSTR Format, ...);
	int Format(LPCSTR fmt, ...);
	void ReleaseBuffer(int NewLength = -1);
	void SetAt(int Index, char ch);
	int vprintf (LPCSTR fmt, va_list ap);
	LPCSTR GetToken(GCHAR **tok);
	LPCSTR MakeInAddr(in_addr Domain);
	LPCSTR MakeSockAddr(sockaddr *psa);
	char LastChr() {return buf_used ? buffer[buf_used -1] : 0;}
	LPCSTR MkFsysPath(LPCSTR Path, ...);
	LPCSTR AddFsysPath(LPCSTR Path, ...);
	void MkFsysPathV(LPCSTR Path, va_list ap);
	BOOL CutAtLastCh(GCHAR ch);
	BOOL ParsePath();
	BOOL CopyToClipboard();
	BOOL PasteFromClipboard();
	void TrimRight();
	WarString ExpandFileName(BOOL bExpandPath = FALSE);
	char GetLastSlash();
};


// Some useful string functions...
int strcasecmp(LPCSTR s1, LPCSTR s2);
BOOL PatternMatchesName(LPCSTR OrgName, LPCSTR Pattern);
LPSTR StrDup(LPCSTR Str);
LPSTR SepStrDup(LPCSTR Buf);
WarString GetLastErrorText(int Errno, BOOL bShowAlways = TRUE);

// Safe string array
LPCSTR SafeStringIndexExp(LPCSTR File, int Line, LPCSTR *StrIndex, int Index, int Limit);
#define SafeStringIndex(StrIndex, Index, Limit)\
	SafeStringIndexExp(__FILE__, __LINE__, (LPCSTR *)StrIndex, Index, Limit)
 