/* !!Copyright begin */
/*
War Software series
Copyright (C) 1996 - 1998 by Jarle (jgaa) Aase (www.jgaa.com)
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation version 2
of the License - WITH THE FOLLOWING RESTRICTIONS:

  * GOVERNMENTS, GOVERNMENTAL AGENCIES OR COMMERCIAL 
    COMPANIES OWNED 50% OR MORE BY THE GOVERNMENT ARE
    NOT ALLOWED TO USE THE SOFTWARE OR SORCE CODE FOR
    ANY PURPOSE. THIS RESTRICTION ALSO APPLIES FOR
    PROGRAMS DERIVED FROM THE SOURCE CODE, OR PROGRAMS/
    SOURCE CODE THAT USE SPECIAL TECHNICS/ALOGRITHMS
    INVENTED IN THIS SOFTWARE.

  * THE FULL SOURCE CODE IS WITHELD FROM A PUBLIC RELEASE
    UNTIL THE LINUX VERISON IS RELEASED.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
 
/* !!Copyright end */
// WarTimer
// Used to test for millisecond based timers

/*c++Doc ===================================================
group Time;
Abstract: Miscellaneous timer functions and classes;
Arguments: ;
Note: ;
Group:;
Project: ;
Return: ;
Sample: ;
See also: ;
==========================================================*/


/*c++Doc ===================================================
class WarTimer;
Abstract: millisecond based timer for timeout handling.
This class is not event driven, it is only used to test
if a timer has expiered;
Arguments: ;
Note: ;
Group:Time;
Project: ;
Return: ;
Sample: ;
See also: ;
==========================================================*/


// File time offset from January 1, 1601 to January 1, 1970
//#define FILETIME_OFFSET ((QSWORD)5347526345495640)

#ifdef GetCurrentTime
#undef GetCurrentTime
#endif

class WarTimer
{
public:
	QSWORD m_TimerVal;

public:
	WarTimer(); // Initializes to current system tick
	void Reset(DWORD msTimeout = 0); // Set the internal value to current system tick
	BOOL TimeOut(QSWORD Milliseconds = 0) const; // Check for timeout
	QSWORD TimeDiff() const; // Difference in ms between the stored and the current time
	QSWORD TimeDiff(QSWORD Milliseconds) const; // // Difference in ms between the stored and the given time
	QSWORD TimeLeft(QSWORD Milliseconds) const; // Time left before timeout in ms
	static QSWORD GetCurrentTime(BOOL bForce = FALSE); // Get the current time as ms
	static BOOL SetCurrentTime(QSWORD Milliseconds);
	static void Sleep(DWORD msTime); // Sleep
	operator QSWORD() {return m_TimerVal;}

	QSWORD &operator = (const QSWORD t) {m_TimerVal = t; return m_TimerVal;}
	QWORD operator = (const QWORD t) {m_TimerVal = (QSWORD)t; return (QWORD)m_TimerVal;}

	static int m_Optimized; // Don't get system time unless forced to
private:
	static QSWORD m_CurrentTime;
};

#ifdef MSVC
int gettimeofday(struct timeval *tv, struct timezone *tz);
#endif

enum DaysInWeekDef
{
	Sun,
	Mon,
	Tue,
	Wed,
	Thu,
	Fri,
	Sat
};

enum MonthsInYear
{
	Jan,
	Feb,
	Mar,
	Apr,
	May,
	Jun,
	Jul,
	Aug,
	Sep,
	Oct,
	Nov,
	Dec
};

class WarTime
{
public:
	WarTime();
	WarTime(time_t t);
	WarTime(WarTime& t);
	WarTime(WarTimer& t);
	WarTime(timeval& t);
	WarTime(QSWORD &t);
#ifdef WIN32
	WarTime(FILETIME& ft);
	WarTime(SYSTEMTIME& st);
#endif // WIN32

	QSWORD m_Time; // time in milliseconds since 00:00:00 GMT, January 1, 1970

#ifdef WIN32
	static QWORD FILETIME_OFFSET;
#endif

	void SetToNow() {m_Time = WarTimer::GetCurrentTime();}
	WarString FormatGMT(LPCSTR Fmt);
	WarString FormatLT(LPCSTR Fmt);
	static void Initialize();
	void SetToNull() {m_Time = 0;}
	BOOL HasExpiered();

#ifdef WIN32
	WarTime &operator = (FILETIME& ft);
	WarTime &operator = (SYSTEMTIME& st);
#endif // Win32
	WarTime &operator = (const time_t t);
	WarTime &operator = (const WarTime& t);
	WarTime &operator = (const WarTimer& t);
	WarTime &operator = (const timeval& t);
	QSWORD &operator = (const QSWORD t) {m_Time = t; return m_Time;}
	QWORD operator = (const QWORD t) {m_Time = (QSWORD)t; return (QWORD)m_Time;}

	BOOL operator == (const WarTime& t) const {return m_Time == t.m_Time;}
	BOOL operator != (const WarTime& t) const {return m_Time != t.m_Time;}
	BOOL operator >= (const WarTime& t) const {return m_Time >= t.m_Time;}
	BOOL operator <= (const  WarTime& t) const {return m_Time <= t.m_Time;}
	BOOL operator < (const WarTime& t) const {return m_Time < t.m_Time;}
	BOOL operator > (const WarTime& t) const {return m_Time > t.m_Time;}

	operator QSWORD() {return m_Time;}
	operator WarTimer();
	operator timeval();
	operator time_t();
	operator tm();

	DaysInWeekDef GetDayOfWeek();
	MonthsInYear GetMonth();
	int GeyYear(); // 1900 style
	int GetDayInMonth(); // 0 - 31
	int GetHour(); // 00 - 23
	int GetMin(); // 0 - 59
	int GetSec(); // 0 - 59

#ifdef WIN32
	operator FILETIME();
	operator SYSTEMTIME();
#endif // WIN32
};

int WarCompareFileTime(WarTime& w1, WarTime& w2);


