/* !!Copyright begin */
/*
War Software series
Copyright (C) 1996 - 1998 by Jarle (jgaa) Aase (www.jgaa.com)
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation version 2
of the License - WITH THE FOLLOWING RESTRICTIONS:

  * GOVERNMENTS, GOVERNMENTAL AGENCIES OR COMMERCIAL 
    COMPANIES OWNED 50% OR MORE BY THE GOVERNMENT ARE
    NOT ALLOWED TO USE THE SOFTWARE OR SORCE CODE FOR
    ANY PURPOSE. THIS RESTRICTION ALSO APPLIES FOR
    PROGRAMS DERIVED FROM THE SOURCE CODE, OR PROGRAMS/
    SOURCE CODE THAT USE SPECIAL TECHNICS/ALOGRITHMS
    INVENTED IN THIS SOFTWARE.

  * THE FULL SOURCE CODE IS WITHELD FROM A PUBLIC RELEASE
    UNTIL THE LINUX VERISON IS RELEASED.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
 
/* !!Copyright end */


class WarTree;
class WarTreeNode;
class WarTreeInode;

struct WarInodeArrayNode
{
	LPCVOID m_ptr;
	int m_NextFree;
	int m_PrevFree;
};

class WarInodeArray
{
public:
	WarInodeArray();
	~WarInodeArray();

	void *operator [] (DWORD Inode);
	void SetAt(QWORD Inode, LPCVOID ptr);
	void SetAt(DWORD Inode, LPCVOID ptr);
	DWORD SetNew(LPCVOID ptr);
	void Remove(QWORD Inode);
	void Remove(user Inode);
	void RemoveFromFree(int Index);
	WarInodeArrayNode *GetNode(DWORD Index);
	void AddToFree(int Index, int Num);
	DWORD GetFree();

	WarInodeArrayNode *m_pArray;

	//DWORD num_used;
	DWORD num_alloced;
	int m_FirstFreeNode;
};


// Referenced trough a dynamic pointer array.
class WarTreeInode
{
public:
	WarTreeInode();
	~WarTreeInode();
	virtual void Kill(); // Use in stead of delete !!

	DWORD GetInode() {return m_Inode;}
	DWORD SetInode(DWORD Inode) {return m_Inode = Inode;}
	virtual BOOL Serialize(WarArchive& ar);
	int GetLinks() {return m_Links;}

	virtual LPCSTR GetName() {return m_Name;}
	LPCSTR SetName(LPCSTR Name) 
		{m_Name = Name; return m_Name;} 

	BOOL Link(WarTreeNode *pTreeNode);
	BOOL Unlink(WarTreeNode *pTreeNode);

protected:
	DWORD m_Links;
	WarString m_Name; // Name
	DWORD m_Inode;
	union
	{
		WarTreeNode *m_pNode; // Points back to single reference (m_Links == 1)
		WarLinkedList *m_pList;
	};
};


/*
 * This class can be used to reference a Inode structure (to allow
 * links), or it can be used as a tree + 1 inode.
 * The actual data should be stored in a class derived from WarTreeNode
 * or WarTreeInode. If the WarTreeInode method is used, each data item
 * can be referenced by several nodes - allowing complex linking of objects
 * (like UNIX hard links). This will however give some more memory usage, and
 * a little CPU overhead.
 * The WarTreeInode method require a lookup table. the WarTreeNode method can
 * (optionally) use a lookup table.
 * 
 */
class WarTreeNode
{
public:
	~WarTreeNode();
	WarTreeNode();

	virtual void Kill(); // Use in stead of delete!
	
	virtual WarTreeNode *Lookup(LPCSTR Name, DWORD Flags = 0);
	virtual WarTreeNode *LookupSon(LPCSTR Name, DWORD Flags = 0);

	virtual LPCSTR GetName() {return m_Name;}
	LPCSTR SetName(LPCSTR Name) 
		{m_Name = Name; return m_Name;} 

	// Inode is the unique identifier of the data item.
	DWORD GetInode() 
		{return m_DataIndex;}
	DWORD SetInode(DWORD Inode) 
		{return m_DataIndex = Inode;}

	// Index is the unique identifier to a tree node
	// Used as file/user identifier, together with the
	// couter.
	user GetUserID() // Get the unique user instance identifier
		{return m_NodeIndex;}
	DWORD GetIndex() 
		{return (DWORD)(m_NodeIndex & 0xffffffff);}
	DWORD GetCounter() 
		{return (DWORD)((m_NodeIndex >> 32) & 0xffffffff);}
	DWORD SetIndex(DWORD Index) 
#ifdef GNUCPP
		{m_NodeIndex &= 0xffffffff00000000ULL;  
#else
		{m_NodeIndex &= 0xffffffff00000000;  
#endif
			return (DWORD)(m_NodeIndex |= Index);}
	DWORD SetCounter(DWORD Counter)
#ifdef GNUCPP
		{m_NodeIndex &= 0x00000000ffffffffULL;  
#else 
		{m_NodeIndex &= 0x00000000ffffffff;  
#endif
			return (DWORD)(m_NodeIndex |= ((QWORD)Counter << 32));}

	BOOL VerifyPerms(WarUserNode *pUser, DWORD Mode);
	BOOL VerifyPerms(DWORD Perms, DWORD PermWanted);

	virtual BOOL Serialize(WarArchive& ar);

	WarTreeNode *GetNext() {return m_pNext;}
	WarTreeNode *GetPrev() {return m_pPrev;}
	WarTreeNode *GetSon() {return m_pSon;}
	WarTreeNode *GetFather() {return m_pFather;}

	WarTreeNode *SetNext(WarTreeNode *p) {return m_pNext = p;}
	WarTreeNode *SetPrev(WarTreeNode *p) {return m_pPrev = p;}
	WarTreeNode *SetSon(WarTreeNode *p) {return m_pSon = p;}
	WarTreeNode *SetFather(WarTreeNode *p) {return m_pFather = p;}

	BOOL Link(WarTreeInode *pDataNode);
	BOOL Unlink(WarTreeInode *pDataNode);
	BOOL ValidateUserNum(user Num)
		{return Num == m_NodeIndex;}
	void BldFullList(WarLinkedList& List, BOOL bAddBrothers = FALSE);
	
	void SetUid(user uid) {m_uid = uid;}
	void SetGid(user gid) {m_gid = gid;}
	void SetMode(DWORD mode) {m_Mode = mode;}

	user GetUid() {return m_uid;}
	user GetGid() {return m_gid;}
	DWORD GetMode() {return m_Mode;}

	virtual WarTreeInode *AllocNew();

	enum
	{
		MAX_LEVELS = 64, // Depth of recursion
	};

protected:
	WarString m_Name; // Name. If empty, use the inode's name

private:

	WarTreeNode *m_pSon; // Subdirs
	WarTreeNode *m_pFather; // Parent dir
	WarTreeNode *m_pNext;
	WarTreeNode *m_pPrev;

	DWORD m_DataIndex; // Index to WarTree::m_InodeArray
	user m_NodeIndex; // Index to WarTree::m_TreeNodeArray
	
	unsigned m_bDeleted : 1; // Deleted

	user m_uid; // Owner
	user m_gid; // Group
	DWORD m_Mode; // Mode/attributes

	WarTime m_Created;
	WarTime m_Modified;
	WarTime m_Accessed;
	WarTimer m_Timer; // Refresh timer/close timer
};


class WarTree
{
public:
	WarTree(DWORD Flags);
	~WarTree();

	enum 
	{
		USE_INODE_TABLE =			0x0001, // Use an inode lookup array
		USE_INODES =				0x0002, // Use base -> Tree -> InodeData 
		CMP_CASE_SENSITIVE =	0x0004,

		// Link mode
		LINK_FIRST_SON = 0,
		LINK_LAST_SON,
		LINK_BEFORE,
		LINK_AFTER
	};

	// API Calls
	LPCSTR GetName(WarTreeNode *pNode); // the nodes name
	LPCSTR GetNativeName(WarTreeNode *pNode); // The object's real name
	LPCSTR GetPath(WarTreeNode *pCurr, WarString& Path);
	LPCSTR GetNativePath(WarTreeNode *pCurr, WarString& Path);
	
	WarTreeNode *GetNode(LPCSTR Path);
	WarTreeNode *GetNode(user User);
	WarTreeNode *GetNode(DWORD dwIndex);
	WarTreeInode *GetDataNode(DWORD iNode)
		{ return (WarTreeInode *)m_InodeArray[iNode];}
	WarTreeInode *GetDataNode(user User)
		{ WarTreeNode *pNode = GetNode(User);
		  return pNode ? GetDataNode(pNode->GetInode()) : NULL;}


	WarTreeNode *GetNext(WarTreeNode& Node) 
		{return Node.GetNext();}
	WarTreeNode *GetPrev(WarTreeNode& Node)
		{return Node.GetPrev();}
	WarTreeNode *GetFather(WarTreeNode& Node)
		{return Node.GetFather();}
	WarTreeNode *GetSon(WarTreeNode& Node)
		{return Node.GetSon();}
	WarTreeNode *GetRoot()
		{return m_pRoot;}
	WarTreeNode *SetRoot(WarTreeNode *pNode)
		{return m_pRoot = pNode;}

	WarTreeNode *Find(WarTreeNode& Node, LPCSTR Pattern, BOOL Recurse = FALSE); // Start search at node.

	// Create a new entry. The 'user' number returned
	// will be unique for all time.
	user WarTree::CreateNew(
	WarTreeNode *pTreeNode, // If null, a node is created automatically
		WarTreeInode *pDataNode, // If NULL and USE_INODES, node is created automatically
		DWORD Flags, // Link mode
		WarTreeNode *pRelative, // Link to
		LPCSTR Name, 
		user Owner_uid,
		user Owner_gid,
		DWORD Mode);
	BOOL DeleteNode(user Node, user Caller);
	
	// low level
	void LinkFirst(WarTreeNode *pFather, WarTreeNode *pNode); // Null to link at root level
	void LinkLast(WarTreeNode *pFather, WarTreeNode *pNode); // Null to link at root level
	void LinkBefore(WarTreeNode *pBrother, WarTreeNode *pNode); 
	void LinkAfter(WarTreeNode *pBrother, WarTreeNode *pNode); 
	void Unlink(WarTreeNode *pNode);
	void DeleteSubtree(WarTreeNode *pNode, BOOL bDeleteBrothers = TRUE);
	
	virtual BOOL Serialize(WarArchive& ar);
	virtual WarTreeNode *AllocNew();
	virtual WarTreeInode *AllocNewInode();

	// low-level Add and remove inodes
	virtual QWORD LinkInode(WarTreeInode *pData);
	virtual void RemoveInode(WarTreeInode *pData);

	// Release all memory
	virtual void KillAll();

protected:
	void Clear();

	void Link(WarTreeNode *pNode); // Called by all link* calls
	int m_NumNodes;
	WarTreeNode *m_pRoot;
	DWORD m_Flags;
	WarInodeArray m_InodeArray; // Data node index
	WarInodeArray m_TreeNodeArray; // Tree node index
	DWORD m_NodeCnt; // Unique node counter
};


