/* !!Copyright begin */
/*
War Software series
Copyright (C) 1996 - 1998 by Jarle (jgaa) Aase (www.jgaa.com)
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation version 2
of the License - WITH THE FOLLOWING RESTRICTIONS:

  * GOVERNMENTS, GOVERNMENTAL AGENCIES OR COMMERCIAL 
    COMPANIES OWNED 50% OR MORE BY THE GOVERNMENT ARE
    NOT ALLOWED TO USE THE SOFTWARE OR SORCE CODE FOR
    ANY PURPOSE. THIS RESTRICTION ALSO APPLIES FOR
    PROGRAMS DERIVED FROM THE SOURCE CODE, OR PROGRAMS/
    SOURCE CODE THAT USE SPECIAL TECHNICS/ALOGRITHMS
    INVENTED IN THIS SOFTWARE.

  * THE FULL SOURCE CODE IS WITHELD FROM A PUBLIC RELEASE
    UNTIL THE LINUX VERISON IS RELEASED.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
 
/* !!Copyright end */


#define USER_DATABASE_NAME "WarUser.dat"


class WarUserException
{
public:
	static void _Throw(
		LPCSTR _File, 
		int _Line, 
		user User, 
		int ErrID,
		LPCSTR pMsg);

	void Tell();

	user User;
	int ErrorCode;
	LPCSTR ErrorMessage;
	int Line;
	LPCSTR File;

};

#define ThrowUserExc(User, _ErrID, Msg) \
	WarUserException::_Throw(__FILE__, __LINE__, User, _ErrID, Msg)

// NB: Only one WarUserDB object pr. cluster!!
class WarUserDB : public WarTree
{
public:
	WarUserDB();
	~WarUserDB();

	BOOL Setup(LPCSTR Name);
	BOOL Load(LPCSTR Name);
	BOOL LoadOld();
	BOOL Save(LPCSTR Name);
	static WarUserDB *GetUserDB() {return m_pMe;}
	static WarUserNode *GetUser(user User);
	WarUserNode *FindUser(LPCSTR Name, user InDomain, user_node_type Type = UT_USER);
	static user GetVDomain(in_addr Domain);
	static WarUserNode *GetDomainRoot(user InDomain);
	BOOL ExchangeNTUsers(user StartNode);
	BOOL CreateNewDatabase();

	virtual BOOL Serialize(WarArchive& ar);
	virtual WarTreeNode *AllocNew();
	virtual WarTreeInode *AllocNewInode();

	user CreateNode(
								user Origin, // The user that call the function
								user_node_type Type, 
								LPCSTR Name, 
								user Father, 
								user Owner_uid,
								user Owner_gid,
								DWORD Mode,
								user_gids Groups);


	BOOL CreateAnonymous(user InDomain, BOOL bEnable);

	static usrstat GetError() {return m_Errno;}
	static usrstat SetError(usrstat Err) 
		{return m_Errno = Err;}

protected:

	static usrstat m_Errno;
	static WarUserDB *m_pMe;
	Validate();
};

class WarUserData : public WarTreeInode
{
public:
	WarUserData();
	~WarUserData();
	virtual void Kill(); // Use in stead of delete !!

	user_node_type GetType();
	user_node_type SetType(user_node_type Type);
	void SetHomePath(LPCSTR Path);
	void SetRootPath(LPCSTR Path);
	void AddFsAccess(user_fs_access& ufa);
	void SetFTP_MaxIdleTime(user_login_perms_mode Mode, long Num);
	void SetFTP_MaxSimLogin(user_login_perms_mode Mode, long Num);
	void SetFTP_MaxSimLoginIP(user_login_perms_mode Mode, long Num);
	void SetFTP_MaxLoginTime(user_login_perms_mode Mode, long Num);
	void SetFTP_MaxCPS(user_login_perms_mode Mode, long Num);
	void SetFTP_LoginAccess(user_login_access Access);
	virtual BOOL Serialize(WarArchive& ar);
	BOOL CreateRootFileSystem();

	static int FsCmp(const void *elem1, const void *elem2);
	void VerifyFileAttrs();

	// Low level
	void SetMaxIdleTime(
		user_login_perms& lpr, 
		user_login_perms_mode Mode, 
		long Num);
	void SetMaxSimLogin(user_login_perms& lpr, 
		user_login_perms_mode Mode, 
		long Num);
	void SetMaxSimLoginIP(user_login_perms& lpr, 
		user_login_perms_mode Mode, 
		long Num);
	void SetMaxLoginTime(user_login_perms& lpr, 
		user_login_perms_mode Mode, 
		long Num);
	void SetMaxCPS(user_login_perms& lpr, 
		user_login_perms_mode Mode, 
		long Num);

	war_user_data m_Data; // All user properties

	void ClearValues();

public:
	// WarTree support
};


class WarUserNode : public WarTreeNode
{
public:
	~WarUserNode();
	virtual void Kill(); // Use in stead of delete!!

	// Server, Native API
#ifdef _DEBUG
	WarUserData *p_data();
#else
	WarUserData *p_data() {
		return (WarUserData *)WarUserDB::GetUserDB()->GetDataNode(GetInode());}
#endif
	WarUserNode *FindUser(LPCSTR Name, user_node_type Type = UT_USER);
	user GetVDomain(in_addr Domain);
	virtual BOOL Serialize(WarArchive& ar);;
	
	const int IsAdministrator();
	const user GetUID();
	const user_perm_id &GetPerms();
	const user_node_subclass GetSubclass();
	const user_node_type GetType();
	void  SetType(user_node_type Type);
	const user GetOwnerUID();
	const user GetOwnerGID();
	const user_priv_type GetUserType();
	const DWORD GetModeBits();
	const user_pwd &GetPwdInfo(user_pwd& Pwd);
	const war_daemon_info *GetDaemonInfo();
	const user_attrs &GetUserAttrs(user_attrs& a);
	const user_expiery_res *GetExperyData(wdq_qmode Mode);
	const ip_access_list *GetIPAccessList(wdq_mode DataMode, wdq_qmode Mode);
	const user_fs_access_list *GetFSAccessList(wdq_qmode Mode);
	const user_info *GetUserInfo(wdq_qmode Mode); // Ignore parameter
	const user_greeting *GetUserGreeting(wdq_qmode Mode); // Ignore parameter
	const user_stat *GetUserFTPStat(wdq_qmode Mode); // Ignore parameter
	const ud_restrictions_res *GetUserFTPUDR(wdq_qmode Mode); // Ignore parameter
	const user_login_perms_result *GetUserFTPLoginPerm(wdq_mode DataMode, wdq_qmode Mode);
	const fs_dup_res *GetUserFTPDup(wdq_qmode Mode);
	const generic_level_int *GetUserPtype(wdq_qmode Mode);
	const generic_level_int *GetUserFTPLoginAccess(wdq_qmode Mode);
	const generic_level_int *GetUserLoginAccess(wdq_mode DataMode, wdq_qmode Mode);
	const user_login_time_result *GetUserLoginTimeMask(
		wdq_mode DataMode, wdq_qmode Mode);
	const user_subclass_result *GetUserSubclass(wdq_qmode Mode); // Mode is ignored
	const user_fsys_paths *GetUserFsysPaths(wdq_qmode Mode);

	BOOL InPattern(LPCSTR Pattern)
		{return m_Name.InPattern(Pattern);}

	void AllowAll(); // Give all privilegies
	void Resynch(BOOL bDoDestructiveTests = TRUE); // Check everything....
	void ResynchBeforeLogin();
	void ResynchAfterLogout();
	void VerifyFileAttrs(); // Full (logical) checking of paths and aliases
	
protected:
	void UserHasExpiered(const user_expiery *pe);

	static usrstat m_Errno;
	static user_expiery_res m_Expery;
	static ip_access_list *m_pIpAccessList;
	static user_fs_access_list *m_pFSAccessList;
	static fs_banned_files_list *m_pBannedFilesList;
	static user_login_perms_result m_LoginPermResult;
	static fs_dup_res m_Dup;
	static generic_level_int m_GenericInt;
	static user_login_time_result m_TimeMask;
	static ud_restrictions_res m_UDR;
	static user_info m_Info;
	static user_greeting m_Greeting;
	static WarString m_wBuffer;
	static user_subclass_result m_Subclass;
	static war_daemon_info m_DaemonInfo;
	static union UserSubclassU
		{
			war_nt_domain m_SubclassNTDomainData;
		} m_Subclass_u;
	static user_fsys_paths m_Paths;
};


class WarUserFindHandle
{
public:
	WarUserFindHandle();
	~WarUserFindHandle();

protected:
	user_attrs_result m_Result;
	user_find_req m_Request;
};

EXT BOOL UseLocalUserdb;

 