/* !!Copyright begin */
/*
War Software series
Copyright (C) 1996 - 1998 by Jarle (jgaa) Aase (www.jgaa.com)
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation version 2
of the License - WITH THE FOLLOWING RESTRICTIONS:

  * GOVERNMENTS, GOVERNMENTAL AGENCIES OR COMMERCIAL 
    COMPANIES OWNED 50% OR MORE BY THE GOVERNMENT ARE
    NOT ALLOWED TO USE THE SOFTWARE OR SORCE CODE FOR
    ANY PURPOSE. THIS RESTRICTION ALSO APPLIES FOR
    PROGRAMS DERIVED FROM THE SOURCE CODE, OR PROGRAMS/
    SOURCE CODE THAT USE SPECIAL TECHNICS/ALOGRITHMS
    INVENTED IN THIS SOFTWARE.

  * THE FULL SOURCE CODE IS WITHELD FROM A PUBLIC RELEASE
    UNTIL THE LINUX VERISON IS RELEASED.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
 
/* !!Copyright end */

/*
 * API wrapper for RPC and direct access to the
 * user database.
 * This is the function _all_ programs should
 * use to access the user database
 * One WarUserHandle object will communicate with 
 * one server.
 */
class WarUserHandle
{
public:
	WarUserHandle();
	~WarUserHandle();
	BOOL Create(WarSocket *pCaller, LPCSTR Host); // Host == NULL = Localhost 
	static usrstat GetLastError() {return m_Errno;}
	BOOL IsConnected() {return m_RPCclnt || m_pSVC;}
	
	// User database access API
	BOOL RPCLogin(LPCSTR Name, LPCSTR Passwd, 
		int *pErr = NULL, // Extedned error information
		user_node_type UserType = UT_USER, 
		war_svr_type ServerLogonType = WST_INVALID);
	void RPCLogout();
	BOOL ClientLogin(war_client_login_req &Req, 
		war_user_login_ok &Handle, int &Err);
	BOOL GetAttr(user User, user_attrs& Attr);
	BOOL CreateNew(user_create_req& Arg, user& Node);
	BOOL DeleteNode(user Node);
	BOOL FindUser(user Origin, user_node_type Type, 
		LPCSTR Pattern, 
		user_find_direction Direction,
		user_attrs& Attr);
	BOOL FetchDomains(WarLinkedList& Domains);
	LPCVOID Query(user User, wdq_qmode QueryMode, 
		wdq_mode DataMode);
	BOOL SetPwdMode(user User, user_pwd_type Mode);
	BOOL SetPwd(user User, LPCSTR Pwd);
	BOOL SetAccess(user User, user_access_type Mode, user_login_access Access);
	BOOL SetPrivMode(user User, user_priv_type Mode);
	BOOL MoveUser(user SrcUser, user Dst, int Flags = 0);
	BOOL SetLoginPerm(user User, wdq_mode Where, 
		long Data, user_login_perms_mode Mode);
	BOOL GenericSetList(user User, 
		ip_restrictions_mode Mode,
		LPCSTR AllowList,
		LPCSTR DenyList,
		BOOL bModeIsValid,
		BOOL bAllowListIsValid,
		BOOL bDenyListIsValid,
		ip_access_list **ppAccessList,
		wdq_mode DataMode);
	BOOL SetIPAccessList(user User, 
		ip_restrictions_mode Mode,
		LPCSTR AllowList,
		LPCSTR DenyList,
		BOOL bModeIsValid,
		BOOL bAllowListIsValid,
		BOOL bDenyListIsValid,
		ip_access_list **ppAccessList);
	BOOL SetFTPBannedFiles(user User, 
		ip_restrictions_mode Mode,
		LPCSTR AllowList,
		LPCSTR DenyList,
		BOOL bModeIsValid,
		BOOL bAllowListIsValid,
		BOOL bDenyListIsValid,
		fs_banned_files_list **ppAccessList);
	BOOL SetExpiry(user User, 
		user_expiery_type Mode,
		user_expiery_method Method, 
		WarTime& Date, // Can be ex. date or when ex. countdown was activated
		long ExpireCnt,
		long RelativeDays,
		int FlagsMask,
		int Flags);
	BOOL SetLoginTimeMask(user User,
		wdq_mode ReqMode, /* Normally WDO_FTP_LOGIN_TIMEM */
		user_login_perms_mode Mode,
		long DayMask,
		long SaturdayMask,
		long SundayMask,
		user_login_time_result **ppNewEffective);
	BOOL SetStat(user User, wdq_mode ReqMode,
		user_stat_update UpdateMode, user_stat& Stat);
	BOOL SetFTPUDR(user User, const ud_restrictions& UDR);
	BOOL SetFSDup(user User, wdq_mode ReqMode, fs_dup_mode Mode);
	BOOL SetUserInfo(user User, const user_info& Info, int Mask);
	BOOL SetUserGreeting(user User,
		LPCSTR Message,
		BOOL bDelWhenShown,
		time_t ExpiryDate /*0 to disable timeout */);
	BOOL SetUserSubclass(user User, user_subclass_data& Subclass);
	BOOL SetServerInfo(user User, war_daemon_info &Info);
	BOOL QueryPath(user User, LPCSTR Path, 
		user_get_user_URL_query_mode Mode,
		WarString& EffectiveURL, WarString& EffectivePath,
		DWORD& EffectiveFlags, FLEN& Quota);
	BOOL SetFileAttrs(user User, LPCSTR Path, 
		LPCSTR Alias, FLEN Quota, 
		DWORD PermMask, DWORD Perms);
	BOOL DeleteFileAttrs(user User, LPCSTR Path);
	BOOL ResynchUser(user User);
	BOOL VerifyFileOpenAccess(user User, 
			QWORD SesID,
			LPCSTR URL, LPCSTR Path, LPCSTR Name,
			FLEN Size,
			DWORD WarFileFlags,
			BOOL bCheckBannedList, 
			BOOL bFileExist);
	BOOL ChangeSessionPwd(LPCSTR OldPwd, LPCSTR NewPwd);
	
	// Session manager API
	BOOL QueryHost(in_addr& Host);
	const sesmgr_query_session_list *SesMgrQuerySessions(sesmgr_query_session_req& Arg);
	const sesmgr_query_session_list *SesMgrQuerySessions(int NextIndex);
	BOOL SesMgrSet(QWORD SessionKey, 
		QSWORD NewIdleTime, sesmgr_sdt Mode, ...);
	BOOL SesMgrHelo(QWORD SessionKey, DWORD& Flags);
	WarOption *SesMgrGetOption(LPCSTR Name, int Index);
	BOOL SesMgrSetOption(LPCSTR Name, int Index, LPCSTR NewValue);
	BOOL SesMgrSetFlag(DWORD Mask, DWORD Flag, 
		sesmgr_set_flag_mode Mode, ...);
	sesmgr_query_system_info_reply *GetSystemInfo();
	sesmgr_query_system_current_reply *GetSystemStats();
	sesmgr_query_domain_reply *GetDomainInfo(int Index); // Index start at 1
	sesmgr_query_usersonline_reply *GetUsersOnline(int StartIndex); // Index start at 0
	sesmgr_query_host_reply *GetHostsOnline(int StartIndex); // Index start at 0
	svrmgr_sysmsg *SesMgrGetSysMsgs(int &NumReturned);
	BOOL SesMgrSetSysMsg(LPCSTR FileName, LPCSTR Text);
	BOOL SesMgrGetSysMsg(LPCSTR FileName, WarString& Text);
	svrmgr_macro *SesMgrGetMacros(int& NumReturned);
	sesmgr_file_info_reply *SesMgrQueryFiles(sesmgr_file_info_req& Arg);
	sesmgr_log_event *SesMgrQueryLog(QSWORD EventID, DWORD Mask);
	BOOL SesMgrSaveUserDB();
	sesmgr_ud_restrictions_data *SesMgrGetCredits(QSWORD SessionKey);
	cdfs_unit_list *SesMgrQueryCdFsData();
	BOOL SesMgrCngCdFsData(cdfs_chg_mode Mode, ...);
	int SesMgrQueryOnline(sesmgr_qyery_online_mode Mode);

	// Prorietary session manager calls - do not use!
	BOOL SesMgrPostTransfer(hyper ID, BOOL IsTmp,
		BOOL IsFree, 
		BOOL IsSending, LPCSTR URL, LPCSTR FsysPath, 
		LPCSTR UserPath, hyper Bytes, 
		int TransferSeconds, unsigned int CPS, 
		int FTPType, int& ReturnStatus);

	// Wrapper API for NT network calls via server
	BOOL WarNTAddGroup(LPCSTR Host, LPCSTR Name, LPCSTR Comment, BOOL bMakeGlobal);
	WarString WarNTGetDC(LPCSTR Machine = NULL, LPCSTR Domain = NULL);
	WarLinkedList *WarNTGetDomains(LPCSTR RunOnMachine = NULL);
	WarLinkedList *WarNTGetServers(LPCSTR RunOnMachine, LPCSTR Domain);
	WarLinkedList *WarNTGetGroups(LPCSTR Host);
	BOOL WarNTExchangeUsers(user StartNode);

	// General server API
	sesmgr_svr_startup_options *SvrGetStartupOptions();
	sesmgr_svr_startup_options *SvrSetStartupOptions(sesmgr_svr_startup_req& Arg);

	// Low level
	BOOL ReadBuffer(int ID, int From, int Bytes, LPVOID *ppBuffer, int& BytesRead, BOOL& IsEOF);

protected:
	BOOL CreateRPCClient(LPCSTR Host);
	void DestroyRPCClient();
	CLIENT *m_RPCclnt; // Remote acess
	svc_req *m_pSVC; // Direct access
	static usrstat m_Errno;
};


 