/* !!Copyright begin */
/*
War Software series
Copyright (C) 1996 - 1998 by Jarle (jgaa) Aase (www.jgaa.com)
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation version 2
of the License - WITH THE FOLLOWING RESTRICTIONS:

  * GOVERNMENTS, GOVERNMENTAL AGENCIES OR COMMERCIAL 
    COMPANIES OWNED 50% OR MORE BY THE GOVERNMENT ARE
    NOT ALLOWED TO USE THE SOFTWARE OR SORCE CODE FOR
    ANY PURPOSE. THIS RESTRICTION ALSO APPLIES FOR
    PROGRAMS DERIVED FROM THE SOURCE CODE, OR PROGRAMS/
    SOURCE CODE THAT USE SPECIAL TECHNICS/ALOGRITHMS
    INVENTED IN THIS SOFTWARE.

  * THE FULL SOURCE CODE IS WITHELD FROM A PUBLIC RELEASE
    UNTIL THE LINUX VERISON IS RELEASED.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
 
/* !!Copyright end */

/*
 * Code to handle async DNS lookup. This code replace
 * gethostbyname and gethostbyaddr. Unlike the standard
 * implementation, these functions will return control
 * to the program, and call On* callback functions when
 * the request is served.
 * This is implemented in Winsock. My implementation does
 * however not depend on callback messages. That means that
 * both UNIX and Win32 console applications can use them.
 */

// This object is created, initialized, and lives only
// until the request is served.

#define	MAXALIASES	35
#define	MAXADDRS	35
#define MAXTRIMDOMAINS  4

#ifndef NETDB_INTERNAL
#define	NETDB_INTERNAL -1 /* see errno */
#endif

#ifndef NETDB_SUCCESS
#define	NETDB_SUCCESS   0 /* no problem */
#endif 

#ifndef HOST_NOT_FOUND
#define	HOST_NOT_FOUND	1 /* Authoritative Answer Host not found */
#endif 

#ifndef TRY_AGAIN
#define	TRY_AGAIN	2 /* Non-Authoritive Host not found, or SERVERFAIL */
#endif

#ifndef NO_RECOVERY
#define	NO_RECOVERY	3 /* Non recoverable errors, FORMERR, REFUSED, NOTIMP */
#endif

#ifndef NO_DATA
#define	NO_DATA		4 /* Valid name, no data record of requested type */
#endif 

#ifndef NO_ADDRESS
#define	NO_ADDRESS	NO_DATA		/* no address, look for MX record */
#endif

#include "nameser.h"

#if PACKETSZ > 1024
#define	MAXPACKET	PACKETSZ
#else
#define	MAXPACKET	1024
#endif

union querybuf
{
    HEADER hdr;
    GCHAR buf[MAXPACKET];
};

union align 
{
    long al;
    char ac;
};

#ifdef MSVC
#pragma pack(push,1)  
#endif 
typedef struct 
{
	WORD len WAR_PACKED;
	querybuf query WAR_PACKED;
} dns_quety_t;
#ifdef MSVC
#pragma pack(pop)
#endif 

class WarDNSSocket : public WarSocket
{
public:
	WarDNSSocket(WarSocket *pSocket /*caller*/,
		 int TimeOut);
	~WarDNSSocket();
	// Interface functions
	BOOL AsyncGethostbyaddr(LPCSTR addr, int len, int type);
	void OnAsyncGethostbyaddr(hostent *hp, int Errno);
	BOOL ConnectToNS();
	BOOL OnConnect(int Errno);
	BOOL OnSend(int Errno);
	BOOL DoSend(int Errno);
	BOOL OnError(int ErrorCode);
	BOOL OnReceive(int Errno);
	void OnTimer();
	void OnProcessReply();
	int  GetLastError() {return hh_errno;}

	// Low level
	int res_mkquery(int op, LPCSTR dname, int qclass, int type, 
		LPCSTR data, int datalen, LPCSTR newrr_in, LPSTR buf, 
		int buflen);
	int dn_comp( const char *exp_dn, GCHAR *comp_dn, int length, 
		GCHAR **dnptrs, GCHAR **lastdnptr );
	int dn_find( GCHAR *exp_dn, GCHAR *msg, GCHAR **dnptrs, 
		GCHAR **lastdnptr );
	hostent *getanswer( const querybuf *answer, int anslen, 
		const char *qname, int qclass, int qtype );
	int dn_expand( LPCSTR msg, LPCSTR eomorig, 
		LPCSTR comp_dn, LPSTR exp_dn, int length );
	u_int16_t _getshort(LPCSTR msgp );

	GCHAR mklower(GCHAR ch)	{
		if (isascii((int)ch) && isupper((int)ch))
			return (tolower((int)ch));
		return (ch);}
	hostent *trim_domains(hostent *h );
	void dotrimdomain( char *c );

private:
	WarSocket *m_pOrigin;
	dns_quety_t dns_query;
	static unsigned short resid;
	unsigned short MyID;
	BOOL m_CallbackMode;
	WORD m_Req_len;
	DWORD m_ReceivedTotal;
	dns_quety_t m_answer;
	hostent host;
	char hostbuf[BUFSIZ+1];
	int hh_errno;
	GCHAR *host_aliases[MAXALIASES];
	GCHAR *h_addr_ptrs[MAXADDRS + 1];
	GCHAR namebuf[64];
	char *trimdomain[MAXTRIMDOMAINS];
	char trimdomainbuf[BUFSIZ];
	int numtrimdomains;
	in_addr host_addr;
	int a_type, a_len;
	in_addr a_addr;
	static WarString m_NameServers;
	int m_NameServerCnt;
#ifdef _WINDOWS
public:
	char m_hostbuf[MAXGETHOSTSTRUCT];
#endif
}; 

