/* !!Copyright begin */
/*
War Software series
Copyright (C) 1996 - 1998 by Jarle (jgaa) Aase (www.jgaa.com)
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation version 2
of the License - WITH THE FOLLOWING RESTRICTIONS:

  * GOVERNMENTS, GOVERNMENTAL AGENCIES OR COMMERCIAL 
    COMPANIES OWNED 50% OR MORE BY THE GOVERNMENT ARE
    NOT ALLOWED TO USE THE SOFTWARE OR SORCE CODE FOR
    ANY PURPOSE. THIS RESTRICTION ALSO APPLIES FOR
    PROGRAMS DERIVED FROM THE SOURCE CODE, OR PROGRAMS/
    SOURCE CODE THAT USE SPECIAL TECHNICS/ALOGRITHMS
    INVENTED IN THIS SOFTWARE.

  * THE FULL SOURCE CODE IS WITHELD FROM A PUBLIC RELEASE
    UNTIL THE LINUX VERISON IS RELEASED.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
 
/* !!Copyright end */
%/*
% * Sun RPC is a product of Sun Microsystems, Inc. and is provided for
% * unrestricted use provided that this legend is included on all tape
% * media and as a part of the software program in whole or part.  Users
% * may copy or modify Sun RPC without charge, but are not authorized
% * to license or distribute it to anyone else except as part of a product or
% * program developed by the user or with the express written consent of
% * Sun Microsystems, Inc.
% *
% * SUN RPC IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
% * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
% * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
% *
% * Sun RPC is provided with no support and without any obligation on the
% * part of Sun Microsystems, Inc. to assist in its use, correction,
% * modification or enhancement.
% *
% * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
% * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY SUN RPC
% * OR ANY PART THEREOF.
% *
% * In no event will Sun Microsystems, Inc. be liable for any lost revenue
% * or profits or other special, indirect and consequential damages, even if
% * Sun has been advised of the possibility of such damages.
% *
% * Sun Microsystems, Inc.
% * 2550 Garcia Avenue
% * Mountain View, California  94043
% */

%/*
% * RPC protocol spesification for the user database in the
% * War servers
% * Copyright 1997, 1998 Jarle (jgaa) Aase
% */

const WAR_MAXIDLEN		= 32;	/* Max ID length */
const WAR_MAXPWDLEN		= 32; /* Max password length */
const WAR_MAXDATA			= 8192; /* UDP packet data */
const WAR_MAXNAMELEN	= 256; /* Filename */
const WAR_MAXPATHLEN	= 1024; /* Path */
const USER_MAX_GIDS		= 16; /* Max groups a user can belong to */

const USER_INVALID		= -1; /* Invalid user */
const USER_NONE				= 0; /* No user */
const USER_SYSTEM			= -2; /* Maps to the root node */

typedef hyper					user;

typedef string				RPC_string<>;

#ifdef WARLIB
#define RPC_time hyper
#define RPC_timer hyper
#else
#define RPC_time WarTime
#define RPC_timer WarTimer
#endif

typedef string				RPC_userid<WAR_MAXIDLEN>;
typedef string				RPC_password<WAR_MAXPWDLEN>;
typedef string				RPC_path<WAR_MAXPATHLEN>;
typedef string				RPC_filename<WAR_MAXNAMELEN>;

typedef hyper					RPC_userinode;

/* Server types */
enum war_svr_type
{
	WST_USER,			/* User database server */
	WST_VFSYS,		/* VfSys server */
	WST_LOGD,			/* Log daemon */
	WST_FTPD,			/* FTP server */
	WST_INVALID
};

/* War Daemon query modes */
enum wdq_mode
{
	WDQ_INVALID,
	WDQ_DAEMON_INFO,
	WDQ_EXPIERY,
	WDQ_IP_ACCESS_LIST,
	WDQ_USER_FS_ACCESS_LIST,
	WDQ_USER_INFO,
	WDQ_GREETING,
	WDQ_FTP_STAT,
	WDQ_FTP_UDR,
	WDO_FTP_BANNED_FILES,
	WDO_FTP_LOGIN_ACCESS, /* Login access */
	WDO_FTP_LOGIN_IDLE,	/* Idle time */
	WDO_FTP_LOGIN_SIML, /* Simultaineous logins */
	WDO_FTP_LOGIN_SIMIP, /* Simultaineous logins on IP */
	WDO_FTP_LOGIN_MAXT, /* Max login time */
	WDO_FTP_LOGIN_MAXCPS, /* max CPS */
	WDO_FTP_LOGIN_TIMEM, /* Time mask */
	WDO_USER_PTYPE, /* User privilegie type */
	WDO_FTP_ADMIN_ACCESS,
	WDO_USERDB_ADMIN_ACCESS,
	WDO_LOG_ADMIN_ACCESS,
	WDO_VFSYS_ADMIN_ACCESS,
	WDO_FTP_DUP,
	WDQ_FSYS_PATHS,
	WDQ_USER_SUBCLASS /* Get subclass data */
};


enum user_node_type
{
	UT_ROOT, /* default values only. */
	UT_DOMAIN,
	UT_CLASS,
	UT_GROUP,
	UT_USER,
	UT_ALIAS,

	/* Used by file access flag */
	UT_NODEDEFAULT /* set by this' nodes [default] permissions */
};

enum usrstat
{
	USR_OK = 0,
	USRERR_PERM=1,		/* Not owner */
	USRERR_NOENT=2,		/* No such file or directory */
	USRERR_IO=5,		/* I/O error */
	USRERR_NXIO=6,		/* No such device or address */
	USRERR_ACCES=13,	/* Permission denied */
	USRERR_EXIST=17,	/* File exists */
	USRERR_NODEV=19,	/* No such device */
	USRERR_NOTDIR=20,	/* Not a directory*/
	USRERR_ISDIR=21,	/* Is a directory */
	USRERR_INVAL=22,	/* invalid argument */
	USRERR_FBIG=27,		/* File too large */
	USRERR_NOSPC=28,	/* No space left on device */
	USRERR_ROFS=30,		/* Read-only file system */
	USRERR_NAMETOOLONG=63,	/* File name too long */
	USRERR_NOTEMPTY=66,	/* Directory not empty */
	USRERR_DQUOT=69,	/* Disc quota exceeded */
	USRERR_STALE=70,	/* Stale user handle */
	USRERR_NOPARENT, /* No parent node */
	USRERR_NODATA,	/* No data is available */
	USRERR_NOOPT, /* Requested option not implemented */
	USRERR_SYSERR, /* System call failure. */
	USRERR_BANNED, /* Object is banned */
	USRERR_NOCALLER, /* Caller ID is invalid */

	/* Errors from higher levels */
	USRERR_COMMFAIL = 1000,	/* RPC communication failure */
	USRERR_FAIL	= 9999	/* Unknown error */
};

enum user_node_subclass
{
	UN_DEFAULT, /* Use default */
	UN_NORMAL, /* Handled by the user manager alone */
	UN_NTDOMAIN, /* NT Domain user */
	UN_UNIX, /* Local UNIX user */
	UN_NIS /* NIS network user */
};


struct war_nt_domain
{
	RPC_string m_Domain; /* Domain or machine name */
	RPC_string m_LinkToNTGroup; /* NT user group to link to */
	int m_bIsLocalGroup; /* specifies that the group is a local group */
};

union user_subclass_data switch (user_node_subclass m_Subclass)
{
	case UN_NTDOMAIN:
		war_nt_domain *m_pNTDomain;
	default:
		void;
};

struct user_subclass_result
{
	user_node_type m_Level;
	user_subclass_data m_Data;
};

struct user_subclass_req
{
	user m_User;
	user_subclass_data m_Data;
};

struct user_stat
{
	hyper	m_dlAcnt; /* Failed downloads */
	hyper	m_dlBcnt; /* Kbytes downloaded */
	hyper	m_dlFcnt; /* Files downloaded */
	hyper	m_dlFreeBcnt; /* Free Kbytes downloaded */
	hyper	m_dlFreeFcnt; /* Free Files downloaded */
	hyper	m_lfScnt; /* Sequently failed logins  */
	hyper	m_lfTcnt; /* Total failed logins */
	hyper	m_LoginCnt; /* Total successful logins */
	hyper	m_ulAcnt; /* Failed uploads */
	hyper	m_ulBcnt; /* KBytes uploaded */
	hyper	m_ulFcnt; /* Files uploaded */
};



enum user_stat_update
{
	USU_REPLACE, /* Replace the value */
	USU_ADD /* Add the value */
};

struct user_stat_req
{
	user m_User;
	user_stat_update m_UpdateMode;
	wdq_mode m_DataMode;
	user_stat m_Stat;
};

enum user_login_perms_mode
{
	ULP_DEFAULT, /* Use default */
	ULP_NOLIMIT, /* No limits */
	ULP_USETHIS  /* Use this info */
};

enum user_login_access
{
	ULA_ALLOW,
	ULA_DENY,
	ULA_DEFAULT
};

struct user_login_t
{
	long	m_Data;
	user_login_perms_mode m_Mode;
};

/* One bit represent each hour of the day */
/* #define IsAllowed(Mask, Hour) ((Mask & (1 << Hour)) != 0) */
struct user_login_time_t
{
	long	m_DayMask;
	long	m_SaturdayMask;
	long	m_SundayMask;
	user_login_perms_mode m_Mode;
};

struct user_login_perms
{
	user_login_access m_LoginAccess;
	user_login_t m_MaxIdleTime;
	user_login_t m_MaxSimLogin;
	user_login_t m_MaxSimLoginIP;
	user_login_t m_MaxLoginTime;
	user_login_t m_MaxCPS;
	user_login_time_t m_LoginTimeMask;
};

/* Result on query */
struct user_login_perms_result
{
	user_login_t m_Data;
	user_node_type m_Level;
};

/* Result on query */
struct user_login_time_result
{
	user_login_time_t m_Data;
	user_node_type m_Level;
};

/* Update */
struct login_perms_req
{
	user m_User;
	wdq_mode m_Option; /* Same as generic request */
	user_login_t m_Data;
};

/* Update */
struct user_login_time_req
{
	user m_User;
	wdq_mode m_Option; /* Same as generic request */
	user_login_time_t m_Data;
};

enum user_pwd_encoding /* PWD encoding */
{
	PWDE_NONE,		/* No encoding */
	PWDE_DES13,		/* Unix DES 13 encoded */
	PWDE_INVALID
};

enum user_pwd_type
{
	PWDT_NONE, /* No password */
	PWDT_EMAIL, /* Use email */
	PWDT_SECURE_EMAIL, /* Use vailidated email */
	PWDT_NORMAL
};

struct user_pwd
{
	RPC_password m_Passwd;
	user_pwd_encoding m_PwdEncoding;
	user_pwd_type m_PwdType;
};

enum ud_restriction_types
{
	UC_DEFAULTU, /* Fallback to default */
	UC_NORESTR,	/* No u/d restrictions */
	UC_SESSON,  /* Session only */
	UC_NORMAL		/* Normal */
};

enum ud_restriction_mode
{
	UM_DEFAULT, /* Fallback to default */
	UM_FILE,	/* Based on file */
	UM_BYTE	/* based on bytes */
};

const UDR_INIT_SET		= 0x0001; /* Initial credit is set for user - start countdown */
const UDR_EXTIMER_SET = 0x0002; /* The extra timer is set, start countdown for the next credit */

enum ex_credit_mode
{
	ecm_Day,
	ecm_2Days,
	ecm_3Days,
	ecm_4Days,
	ecm_5Days,
	ecm_6Days,
	ecm_Week,
	ecm_2Weeks,
	ecm_3Weeks,
	ecm_4Weeks,
	ecm_1Month,
	ecm_2Months,
	ecm_Quaterly,
	ecm_Halfyear,
	ecm_Year
};

/* Raw UD ratio data */
struct ud_restrictions
{
	ud_restriction_types	m_UDclass;
	ud_restriction_mode	m_FileOrByte;
	int	m_DlGet;
	int	m_DlPut;
	int m_InitialCredit; /* Initial credit. Can be in Kbytes or files */
	int m_SessionCredit; /* Initial credit. Can be in Kbytes or files */
	int m_ExCredit; /* Extra credit */
	ex_credit_mode m_exCreditMode;
	long m_ExCreditTimer; /* (read-only) Timer for Ex. Credit in seconds from initialization */
	long m_ExCreditLeft; /* (read-only) Ex credit left (countdown and reset by timer) */
	int m_Flags; /* (read-only) */
};

/* Calculated UD ratio */
struct sesmgr_ud_restrictions_data
{
	hyper m_CurrentCredits; /* Current credits for the user */
	int	m_DlGet; /* Info */
	int	m_DlPut; /* Info */
	hyper m_UlCnt; /* Upload count in the cuttent context */
	hyper m_DlCnt; /* Download count in the current context */
	ud_restriction_types	m_UDclass; /* Info */
	ud_restriction_mode	m_FileOrByte; /* Info */
};

struct ud_restrictions_res
{
	user_node_type m_DataLevel;
	user_node_type m_ModeLevel;
	ud_restrictions m_Data;
	int m_InitialCredit; /* Effective credit left */
	int m_SessionCredit; /* Effective session credit */
	int m_CurrentCredit; /* Current credit from the servers point of view */
};


struct ud_restrictions_req
{
	user m_User;
	ud_restrictions m_Data;
};

enum user_expiery_type
{
	UE_DEFAULT,
	UE_NEVER,	/* Account will never expire */
	UE_DATE,	/* Account will expire on the date */
	UE_RELATIVE, /* Account wil expire in # milliseconds after the first login */
	UE_LOGINS /* Account will expire after # logins */
};

enum user_expiery_method
{
	UEM_DEFAULT,
	UEM_KILL,	/* User is deleted */
	UEM_DISABLE, /* User is denied access */
	UEM_ANON	/* User is mapped to another account */
};

const UEX_RESET_AT_LOGIN		=	0x0001; /* Reset relative limit at first login */
const UEX_SET_RELATIVE_NOW	=	0x0002; /* Reset relative limit now (data in m_Date) */

struct user_expiery
{
	/* Recursive properties */
	user_expiery_method m_Method;
	user_expiery_type m_Mode;
	RPC_time m_Date; /* Expiery date */
	long m_ExpireCnt; /* Login counter used for UE_LOGINS */
	int m_Flags; 

	/* Dual use properties */
	RPC_time m_ActivatedDate; /* The date the properties was set. */
							  /* If this is newer in the inherited properties */
							  /* than the abvsolute, the absolute vaules are */
							  /* reset to match the inherited. */
	                          
	/* Absolute properties */
	RPC_timer m_Relative; /* Timer relative to the first successful login */
	RPC_timer m_SuspendDelay; /* Delay set when the info is being updated */
	long m_StartCnt; /* Current login counter when expiration is set */
	int m_RelativeDays; /* Expeiery after n days. Used to set the timer */
};

struct user_expiery_res
{
	user_node_type m_DataLevel;
	user_node_type m_MethodLevel;
	user_expiery m_Data;
};

struct user_expiery_req
{
	user m_User;
	user_expiery_method m_Method;
	user_expiery_type m_Mode;
	long m_Date; /* Expiery date in time_t format */
	long m_ExpireCnt; /* Login counter used for UE_LOGINS */
	int m_RelativeDays; /* Expeiery after n days. Used to set the timer */
	int m_Flags;
	int m_FlagsMask;
};

enum fs_dup_mode
{
	FDM_DEFAULT,	/* Use default */
	FDM_DENY,			/* Deny existant files */
	FDM_ALLOW			/* Allow existant files */
};

struct fs_dup
{
	fs_dup_mode m_Mode;
};

struct fs_dup_res
{
	user_node_type m_Level; /* effective level */
	fs_dup_mode m_Mode;
};

struct fs_dup_req
{
	user m_User;
	wdq_mode m_DataMode;
	fs_dup m_Data;
};

enum user_priv_type
{
	USERPT_ANONYMOUS,
	USERPT_USER,
	USERPT_ADMIN,
	USERPT_INHERIT /* get from higher level */
};

enum ip_restrictions_mode
{
	IR_DISABLE,		/* Disable for this user */
	IR_NORMAL,		/* IP restrictions checked after userid is given */
	IR_THIS,			/* Same as IR_NORMAL, but no recursive effect */
	IR_PRELOGIN	/* Check before login (applies for domain level only) */
};

struct ip_access
{
	RPC_string m_AllowList;
	RPC_string m_DenyList;
	ip_restrictions_mode m_Mode;
};

/* Used for several list types */
/* m_DataMode identifies the actual settings */
struct generic_list_req
{
	user m_User;
	ip_access m_Data; /* for simplicity, we use the ip_access struct. */
	int m_ModeIsValid;
	int m_DenyListIsValid;
	int m_AllowListIsValid;
	wdq_mode m_DataMode;
};

struct ip_access_list
{	
	ip_access m_Data;
	user_node_type m_Level;
	ip_access_list *m_pNext;
};

typedef ip_access fs_banned_files;
typedef ip_access_list fs_banned_files_list;

/* FTP spesific data */

struct war_ftp_user_data
{
	user_stat m_Stat; /* Statistics/History */ 
	ud_restrictions m_UD; /* Up/Download restrictions */
	fs_banned_files m_BannedFiles; /* Banned files */
	user_login_perms m_LoginPerms; /* Login permissions */
	fs_dup m_DupMode; 
};

struct user_greeting
{
	RPC_time m_Expire; /* Greetings expire at this date */
	bool m_bDelete; /* Delete when displayed */
	RPC_string m_Message;
};

struct user_greeting_req
{
	user m_User;
	long m_Expire; /* Greetings expire at this date, or 0 to disable this option. time_t format */
	bool m_bDelete; /* Delete when displayed */
	RPC_string m_Message; /* Message. Up to 2048 bytes */
};

struct user_info
{
	RPC_string m_RealName;
	RPC_string m_Email;
	RPC_string m_Http;
	RPC_string m_Ftp;
	RPC_string m_ICQ;
	RPC_string m_IRCnick;
	RPC_string m_Memo;
};


const UIR_NAME				= 0x0001; /* Name is valid */
const UIR_EMAIL				= 0x0002; /* Email is valid */
const UIR_HTTP				= 0x0004; /* Http valid */
const UIR_FTP					= 0x0008; /* FTP is valid */
const UIR_ICQ					= 0x0010; /* ICQ is valid */
const UIR_IRC					= 0x0020; /* IRC is valid */
const UIR_MEMO				= 0x0040; /* Memo is valid */

struct user_info_req
{
	user m_User;
	int m_Mask;
	user_info m_Data;
};

enum fs_access_flag_value
{
	AF_CREATE			= 0x0001,	/* Create directory */
	AF_DELETE			= 0x0002, /* Delete files */
	AF_DENY				= 0x0004, /* Absolute no access */
	AF_EXECUTE		= 0x0008, /* Execute programs */
	AF_LIST				= 0x0010, /* List directory */
	AF_MAP				= 0x0020, /* Map to path */
	AF_READ				= 0x0040, /* Read files */
	AF_RECURSIVE	= 0x0080, /* Applies for subdirectories */
	AF_REMOVE			= 0x0100, /* Remove directories */
	AF_WRITE			= 0x0200, /* Write files */
	AF_FREE				= 0x0400, /* Files are free (no UD restrictions) */
	AF_HIDE				= 0x0800  /* Hide files/dirs */
};

struct fs_access_flag
{
	fs_access_flag_value m_Value; /* The setting at this node */
	bool m_Effective; /* The effective setting */
	user_node_type m_EffectiveAtLevel; /* What level has set the value? */
};

struct user_fs_access
{
	unsigned m_Flags; /* Permissions */
	RPC_filename	m_Path; /* URL */
	RPC_filename	m_Alias; /* Logical path */
	hyper m_Quota; /* Disk quota limit */
};

struct user_fs_access_list
{
	user_fs_access m_Data;
	user_node_type m_Level;
	user_fs_access_list *m_pNext;
};

/* When updated, quita can be -1 to identify no change */
/* FlagMask is set to the flags that are valid */
/* Path and alias are always valid */
/* If the entry exist, it is changed. If not, it is created */
/* The path name is the unique identifier */

enum user_set_fs_access_actions
{
	SFA_UPDATE, /* Add or update */
	SFA_DELETE /* Delete */
};

struct user_set_fs_access_req
{
	user m_User; /* User node to set on */
	user_fs_access m_Data; /* Data to set */
	unsigned m_FlagMask; /* Flags that are valid */
	user_set_fs_access_actions m_Action; /* What to do */
};

/* Query the server for a WarFile URL to a path, and for the */
/* permissions to that path */

enum user_get_user_URL_query_mode
{
	queryURL_logical, /* Lookup in ALIAS */
	queryURL_physical /* Lookup in paths */
};

struct user_get_user_URL_req
{
	user m_User;
	RPC_path m_Path; /* Path to query for. Can be logical or URL */
	user_get_user_URL_query_mode m_Mode;
};

union  user_get_user_URL_result switch (usrstat status) {
case USR_OK:
	user_fs_access m_FileAccess; /* Effective file access data */
default:
	void;
};

struct user_verify_file_open_access_req
{
	user m_User;
	hyper m_SesID; /* Used for up/dnl ratio */
	RPC_path m_URL; /* Physical (expanded) URL. */
	RPC_path m_Path; /* User path */
	RPC_path m_Name; /* Physical name, no path */
	hyper m_Size; /* If set, and the WarFile::Read is set, download is assumed and credits checked */
	unsigned m_WarFileFlags;
	int m_bCheckBannedList;
	int m_bFileExist;
	int m_reserved1;
	int m_reserved2;
};

/* Extra properties for plugins */

enum user_param_datatype
{
	UP_BOOL,
	UP_INT,
	UP_UNSIGNED,
	UP_HYPER,
	UP_DOUBLE,
	UP_STRING
};

union user_param_data switch (user_param_datatype m_Type)
{
	case UP_BOOL:
		bool m_bool_val;
	case UP_INT:
		int m_int_val;
	case UP_UNSIGNED:
		unsigned m_unsigned_val;
	case UP_HYPER:
		hyper m_hyper_val;
	case UP_DOUBLE:
		double m_double_val;
	case UP_STRING:
		RPC_string m_string_val;
	default:
		void;
};

enum recursive_mode
{
	RM_NONE, /* Not recursive */
	RM_DEFAULT, /* use this or default value */
	RM_ALL /* Use all levels together */
};

struct user_param
{
	user_param_data m_Data;
	int m_Len; /* Length of string field */
	bool m_Unique; /* Data value must be unique */
	recursive_mode m_Recursive;
	RPC_string m_Name; /* Name of option */
};

/* All extra options */
struct user_param_list
{
	recursive_mode m_Data;
	recursive_mode *m_pNext;
};

/* Groups a user belongs to */
/* Affetcs raw permissions opnly */
typedef user user_gids[USER_MAX_GIDS];

/* 
 * The actual permission id for the user.
 * Note: The user also have uid and gid
 *       properties. These defines the
 *       owner. Only the root node owns
 *       itself. All other users are
 *       owend by someone else
 */

struct user_perm_id
{
	user m_uid;
	user_gids m_gids;
};


enum war_svr_ports
{
	WSP_FTPD, /* FTP daemon port index */
	WSP_INVALID, /* Last known service type */
	WSP_MAX = 16 /* Frozen - max 16 service types */
};

struct war_daemon_info
{
	in_addr m_Addr; /* Actual host */
	RPC_string m_HostName; /* Virtual domain address, IP or name */
	unsigned short m_Port[WSP_MAX]; /* Port to use 0 to disable a service */
	int m_GoOnline; /* bitfield - Go online automatially st startup */
	int m_Enabled; /* bitfield - Service is enabled */
};

struct generic_level_int
{
	user_node_type m_Level;
	int m_Data;	
};

struct war_daemon_info_req
{
	user m_User;
	war_daemon_info m_Data;
};

struct user_fsys_paths
{
	RPC_path m_Root;
	RPC_path m_Home;
};



/* General query reply */
union wdq_result switch (wdq_mode mode) 
{
	case WDQ_DAEMON_INFO:
	  war_daemon_info *m_pDaemonInfo;
	case WDQ_EXPIERY:
		user_expiery_res *m_pExpiery;
	case WDQ_IP_ACCESS_LIST:
		ip_access_list *m_pIpAccessList;
	case WDQ_USER_FS_ACCESS_LIST:
		user_fs_access_list *m_pUserFsAccessList;
	case WDQ_USER_INFO:
		user_info *m_pUserInfo;
	case WDQ_GREETING:
		user_greeting *m_pUserGreeting;
	case WDQ_FTP_STAT:
		user_stat *m_pFTPStat;
	case WDQ_FTP_UDR:
		ud_restrictions_res *m_pFTPUDR;
	case WDO_FTP_BANNED_FILES:
		fs_banned_files_list *m_pFTPBannedFiles;
	case WDO_FTP_LOGIN_IDLE:
	case WDO_FTP_LOGIN_SIML:
	case WDO_FTP_LOGIN_SIMIP:
	case WDO_FTP_LOGIN_MAXT:
	case WDO_FTP_LOGIN_MAXCPS:
		user_login_perms_result *m_pFTPLoginPerm;
	case WDO_FTP_LOGIN_TIMEM:
		user_login_time_result *m_pLoginTimeMask;
	case WDO_FTP_LOGIN_ACCESS:
		generic_level_int *m_pFTPLoginAccess;
	case WDO_FTP_DUP:
		fs_dup_res *m_pDup;
	case WDO_USER_PTYPE:
		generic_level_int *m_pData;
	case WDO_FTP_ADMIN_ACCESS:
	case WDO_USERDB_ADMIN_ACCESS:
	case WDO_VFSYS_ADMIN_ACCESS:
	case WDO_LOG_ADMIN_ACCESS:
		generic_level_int *m_pLoginAccess;
	case WDQ_USER_SUBCLASS:
		user_subclass_result *m_pSubclass;
	case WDQ_FSYS_PATHS:
		user_fsys_paths *m_Paths;

		
default:
	void;
};


union user_query_result switch (usrstat status) {
case USR_OK:
	wdq_result m_Data;
default:
	void;
};


enum wdq_qmode
{
	WDQM_ABSOLUTE, /* Do not recurse */
	WDQM_EFFECTIVE, /* Get the effective value */
	WDMQ_ALL, /* Get the whole (recursive) list */
	WDQM_EFFECTIVE_RESET /* Get effective value, reset property */
};

struct war_user_query
{
	wdq_qmode m_QueryMode; /* What kind of result sets we want */
	wdq_mode m_Mode; /* Data (information) we want */
	user m_User; /* User/node we want information about */
};


/* 
 * Complete user record (except ID)
 * Not used for RPC. The client must
 * ask for the different properties
 */
struct war_user_data
{
	user_subclass_data m_Subclass;
	user_node_type m_Type;
	user_pwd m_Pwd; /* Password */ 
	user_info m_Info; 
	user_expiery m_Expiery; /* Expiery info */ 
	war_ftp_user_data m_FTP; /* FTP data */
	ip_access m_IPAccess;
	user_fs_access_list *m_FSAccessList;
	user_perm_id m_Perm;
	war_daemon_info *m_pDaemon; /* Only used for vdomain nodes */
	user_greeting m_Greeting;
	user_fsys_paths m_Paths;
	user_priv_type m_UserType; /* admin, user, anon */ 
	user_login_access m_FTPAdminAccess;
	user_login_access m_UserDBAdminAccess;
	user_login_access m_LogAdminAccess;
	user_login_access m_VfSysAdminAccess;
};

struct user_attrs
{
	RPC_userid m_Name;
	user m_User;
	user_node_subclass m_Subclass;
	user_node_type m_Type;
	user_perm_id m_Perm;
	user m_Owner_uid;
	user m_Owner_gid;
	unsigned m_Mode;
	user_pwd m_Pwd; /* Password is read only - not sent to clients */
	user_priv_type m_UserType;
	int m_HasChildren;
};


/*
 * Reply status with general user info
 */
union user_attrs_result switch (usrstat status) {
case USR_OK:
	user_attrs m_Data;
default:
	void;
};

/*
 * Reply status for create
 */
union user_create_result switch (usrstat status) {
case USR_OK:
	user m_User; /* User Inode */
default:
	void;
};

union user_generic_result switch (usrstat status) {
case USR_OK:
	int m_Status; /* User Inode */
default:
	void;
};

struct user_create_req
{
	user_node_type m_Type;
	RPC_userid m_Name; 
	user m_Father;
	user m_Owner_uid;
	user m_Owner_gid;
	unsigned m_Mode;
	user_gids m_Groups;
};

enum user_find_direction
{
	UF_ROOT, /* Root node, origin is USER_SYSTEM, pattern is ignored */
	UF_FIRST, /* Origin is a 'father' node, start search at first son */
	UF_NEXT, /* Search down */
	UF_PREV, /* Search up */
	UF_PARENT /* Get parent, pattern is ignored */
};

struct user_find_req
{
	user m_Origin;
	RPC_userid m_Pattern;
	user_find_direction m_Direction;
	user_node_type m_Type;
};

/*
 * Reply status for fetch domains
 */
union user_fetcth_domains_result switch (usrstat status) {
case USR_OK:
	user m_Domains<>;
default:
	void;
};

struct user_generic_set_int
{
	user m_User;
	int m_OptClass; /* Not always used */
	int m_Val; 
};

struct user_generic_set_str
{
	user m_User;
	int m_OptClass; /* Not always used */
	RPC_string m_Val;
};

struct user_generic_user_req
{
	user m_User;
	unsigned m_Flags; /* If not used, reserved as 0 */
};

enum user_access_type
{
	UAT_FTP,
	UAT_FTP_ADMIN,
	UAT_USERDB_ADMIN,
	UAT_LOG_ADMIN,
	UAT_VFSYS,
	UAT_VFSYS_ADMIN
};

struct move_user_req
{
	user m_SrcUser;
	user m_DstUser;
	int m_Flags;
};

const WAR_AUTH_LOGIN_DATALEN = 256;

/* Login request for server */
struct war_rpclogin_req
{
	int m_AuthMode; /* autentication mode we want to use */
	int m_AuthEncrLevel; /* Encryption method */
	user_node_type m_UserType; /* Normally UT_USER */
	war_svr_type m_ServerLogonType; /* Only used for service logins. Identify the service type */
	RPC_userid m_ID;
	opaque m_Data[WAR_AUTH_LOGIN_DATALEN]; /* Login data */
};

/* Login reply data */


/* Login reply */
union war_user_rpclogin_result switch (usrstat status) {
case USR_OK:
	opaque m_Data[MAX_AUTH_BYTES]; /* Raw data to put in new calls */
case USRERR_PERM: 
	int m_ReasonID;
default:
	void;
};

/* Client login request - sent from a server on behalf of a client */
/* If successful, the client will be enlisted as 'logged on' until the */
/* server logs the client out, or the client times out */

/* Reasons why a login fails */
enum war_login_fail_reasons
{
	LF_CALLER_NOT_AUTHENTICATED, /* The calling server is not permitted to request for logins */
	LF_UNKNOWN_SVRTYPE, 
	LF_SERVER_NOT_KNOWN,
	LF_ILLEGAL_NAME,
	LF_USER_NOT_FOUND,
	LF_USER_NOT_READY, /* Unstable user condition. user might be deleted */
	LF_USER_NOT_PERMITTED_TO_SERVICE, /* The user is not allowed on this type of server */
	LF_TOO_LONG_PASSWORD,
	LF_BAD_PASSWORD,
	LF_NEED_EMAIL_AS_PWD,
	LF_NEED_PASSWORD,
	LF_TOO_MANY_CONNECTIONS,
	LF_TOO_MANY_CONNECTIONS_FROM_IP,
	LF_INTERNAL_ERROR,
	LF_THROW_USER_OFF, /* Returned when the user have tried a little too much... */
	LF_WAIT_FOR_DNS_LOOKUP, /* Try again later */
	LF_TIME_OF_DAY_DENIAL,
	LF_IP_SHITLISTED,
	LF_IP_MARKED_AS_HACKER,
	LF_SYSTEM_SHUTDOWN,
	LF_NO_ANONYMOUS_USERS,
	LF_INVALID
};

struct war_client_login_req
{
	war_svr_type m_ServerLogonType; /* Identifies what server to log on to */
	RPC_userid m_ID; /* User ID */
	RPC_password m_Pwd; /* User password */
	in_addr m_ClientIP; /* The IP number of the client */
	unsigned m_ClientPort; /* The client-side port number */
	in_addr m_ServerIP; /* The servers IP number that the client connected to */
	unsigned m_ServerPort; /* The server-side port number */
	user m_Server; /* The servers user ID - can be USER_SYSTEM */
	int m_bDoNotLogin; /* Used to test if a user can log on */
};

struct war_user_login_ok
{
	hyper m_ConnectionID; /* Used to update or close the connection */
	user m_User; /* User identifier */
	hyper m_SessionID; /* Unique sequence counter for the session */
};

/* Client login reply */
union war_user_client_login_result switch (usrstat status) {
case USR_OK:
	war_user_login_ok m_Data;
case USRERR_ACCES:
	int m_ReasonID; /* Why the user can't log on */
default:
	void;
};

/* Data buffer */
struct warsvr_read_buffer_req
{
	int m_Handle; /* Identifier for buffer. Currently only one buffer is allowed pr. session */
	int m_ReadFrom;
	int m_ReadBytes;
};

struct warsvr_read_buffer_res
{
	int m_BytesFrom;
	int m_BytesRead;
	int m_IsEOF;
	opaque m_Data<>;
};

union  warsvr_read_buffer_result switch (usrstat status) {
case USR_OK:
	warsvr_read_buffer_res m_Data;
default:
	void;
};

union  warsvr_buffer_result switch (usrstat status) {
case USR_OK:
	int m_Handle; /* Handle of the buffer, used in subsequent warsvr_read_buffer_req calls */
default:
	void;
};

union  warsvr_generic_str_result switch (usrstat status) {
case USR_OK:
	RPC_string m_Data; 
default:
	void;
};

/* NT Network wrapper */

struct warusr_NT_get_DC_req
{
	RPC_string m_Machine; 
	RPC_string m_Domain; 
};

struct warusr_NT_get_domains_req
{
	RPC_string m_RunOnMachine; /* Empty buffer cause default to be used */
};

struct warusr_NT_get_servers_req
{
	RPC_string m_RunOnMachine; /* Empty buffer cause default to be used */
	RPC_string m_Domain; /* Return servers on this NT domain */
};

struct warusr_NT_get_groups_req
{
	RPC_string m_RunOnMachine; /* Empty buffer cause default to be used */
};

struct warusr_NT_add_group_req
{
	RPC_string m_Host; 
	RPC_string m_Name; 
	RPC_string m_Comment; 
	int m_bMakeGlobal;
};

struct warusr_NT_exchange_req
{
	user m_StartNode; /* Start exchange at this node */
};

struct warusr_delete_req
{
	user m_User; /* user to delete */
};

struct warusr_rename_req
{
	user m_User; /* User tp rename */
	RPC_userid m_NewName; /* New user name */
};

struct warusr_setpwd_req
{
	RPC_password m_NewPwd;
	RPC_password m_OldPwd;
};

struct sesmgr_counters_info
{
	int m_OnlineNow; /* Users online now counter */
	int m_OnlineTotal; /* Users that has been online since the server started */
	int m_UplFilesNow; /* Files being uploaded now */
	int m_UplFilesTotal; /* Totel number of uploaded files */
	int m_DownlFilesNow; /* Files being downloaded now */
	int m_DownlFilesTotal; /* Total number of downloaded files */
	int m_FreeDownlFilesTotal; /* Total number of free downloaded files */
	hyper m_UplBytesTotal; /* Total number of uploaded bytes */
	hyper m_DownlBytesTotal; /* Total number of downloaded bytes */
	hyper m_FreeDownlBytesTotal; /* Total number of downloaded bytes */
};

struct sesmgr_query_session_req
{
	/* Session filtering (return only the requested session) */
	hyper m_ID; /* Use USER_NONE for all */
	
	/* Server filtering */
	war_svr_type m_SvrType; /* WST_INVALID if m_SvrNode is USER_NONE */
	hyper m_SvrNode; /* USER_NONE for all */

	/* User filtering */
	hyper m_UserNode; /* USER_NONE for all */

	/* Host filtering */
	in_addr m_Host; /* INADDR_ANY for all */
};

struct sesmgr_query_session_reply
{
	hyper m_SessionID; /* Unique sequence counter */
	hyper m_QID; /* Session query ID */
	in_addr m_UserHost;
	in_addr m_ServerHost;
	RPC_string m_DomainName;
	hyper m_TimeLoggedOn;
	hyper m_TimeLastReq;
	hyper m_TimeoutBeforeReset;
	user m_User;
	RPC_string m_Nickname;
	RPC_string m_UserName;
	war_svr_type m_ServerType;
	RPC_string m_ServerName;
	int m_ProgressMeter; /* Used for progress on transfers */
	int m_CopyProgressMeter; /* Used for progress on file systems that cache files */
	int m_State;
	RPC_string m_StateInfo;
	int m_IdleTime; /* Idle time in seconds */
	int m_IdleTimeLimit; /* Max idle time in seconds */
	RPC_string m_CWD; /* Current working directory */
	RPC_string m_CurrentFilePath;
	RPC_string m_CurrentFileURL;
	hyper m_FileLen;
	hyper m_CurrentFileOfs;
	int m_CPS; /* Bytes / second */
	sesmgr_counters_info m_Counters;
	hyper m_TotalFilesUploaded;
	hyper m_TotalFilesDownloaded;
	hyper m_TotalBytesUploaded;
	hyper m_TotalBytesDownloaded;
	RPC_string m_LastCMD; /* Last command given */
	int m_Flags;
};

struct sesmgr_query_session_list
{
	sesmgr_query_session_reply m_Data;
	sesmgr_query_session_list *m_pNext;
	int m_NextIndex; /* non-zero if there is more data. Use for follow-up call */
	int m_NodeSize; /* Size of this structure including data */
};


/* sesmgr_query_session_req reply */
union sesmgr_query_session_result switch (usrstat status) {
case USR_OK:
	sesmgr_query_session_list m_Data;
default:
	void;
};


struct sesmgr_ip_info
{
	RPC_string m_Tag; /* Comment */
	in_addr m_Addr; /* primary IP of system */
	RPC_string m_Domain; /* Domain name for system */
};

struct sesmgr_cps_sample
{
	unsigned int m_CurrentIndex; /* Index to current reading */
	unsigned int m_PeekCPS; /* Max CPS at any reading */
	unsigned int m_PeekOnline; /* Max users online at any reading */
	int m_Resolution; /* Numbers are CPS / m_Resolution */
	int m_msSampleRate; /* Sample rate in milliseconds */
	unsigned int m_CPSreadings<>; /* Array of readings */
	unsigned int m_SessionReadings<>; /* Users online at the reading */
};

/* Get all kinds of static information */
struct sesmgr_query_system_info_reply
{
	RPC_string m_Name; /* Name of root domain */
	unsigned int m_Flags; /* SesMgr.m_Flags; */
	in_addr m_Addr; /* primary IP of system */
	RPC_string m_Domain; /* Domain name for system */
	int m_Servers; /* Number of domains that are registered */
	hyper m_StartupTime; /* Local time for startup */
	RPC_string m_OS; /* Operating system */
	sesmgr_ip_info m_IP<>; /* Full list of IP numbers */
	RPC_path m_StartupPath; /* The servers current directory */
	RPC_string m_NetworkInfo; /* Information about the network */
};

/* Get current info */
struct sesmgr_query_system_current_reply
{
	sesmgr_counters_info m_Counters<>;
	hyper m_CurrentTime; /* Local time for server */
	sesmgr_cps_sample m_CPSdata;
};

/* sesmgr_query_system_info_reply reply */
union sesmgr_query_system_info_result switch (usrstat status) {
case USR_OK:
	sesmgr_query_system_info_reply m_Data;
default:
	void;
};

/* sesmgr_query_system_current_reply reply */
union sesmgr_query_system_current_result switch (usrstat status) {
case USR_OK:
	sesmgr_query_system_current_reply m_Data;
default:
	void;
};

/* Message and macro support */
struct svrmgr_sysmsg
{
	RPC_string m_MessageName;
	RPC_string m_MessageFileName;
};


struct svr_sysmsg_req
{
	RPC_string m_MessageFileName; /* sysmsg#[@nodename] */
	RPC_string m_Text; /* Limit to 7 KB at this time */
};

union sesmgr_query_sysmsg_result switch (usrstat status) {
case USR_OK:
	svrmgr_sysmsg m_Data<>;
default:
	void;
};

union sesmgr_get_sysmsg_result switch (usrstat status) {
case USR_OK:
	svr_sysmsg_req m_Data;
default:
	void;
};

struct svrmgr_macro
{
	RPC_string m_Name;
	int m_eNum;
	RPC_string m_Explained;
};

union sesmgr_query_macro_result switch (usrstat status) {
case USR_OK:
	svrmgr_macro m_Data<>;
default:
	void;
};

/* Virtual domain/server information */
struct sesmgr_query_domain_reply
{
	int m_Port;
	in_addr m_Addr;
	RPC_string m_DomainName;
	RPC_string m_Name; /* Server name */
	RPC_string m_NodeName; /* Name of the user node object (m_User) */
	war_svr_type m_Type; /* Server type */
	user_node_type m_NodeType;
	int m_State; /* State of the server */
	unsigned int m_Flags; /* Flags that apply for all users on this server and the server */
	user m_User;  /* User node for the server object */
	int m_Index; /* Index that match the array of domains in sesmgr_query_system_current_result */
	sesmgr_cps_sample m_CPSdata; /* CPS history for the server */
};

/* sesmgr_query_doiain_reply reply */
union sesmgr_query_doiain_result switch (usrstat status) {
case USR_OK:
	sesmgr_query_domain_reply m_Data;
default:
	void;
};

struct sesmgr_query_domaun_req
{
	int m_DomainIndex; /* Index that match the array of domains in sesmgr_query_system_current_result */
};


/* Users online information */
struct sesmgr_query_usersonline
{
	user m_User;
	user_node_type m_NodeType;
	RPC_string m_Name;
	RPC_string m_FullName;
	RPC_string m_Domain; /* The domain this user belongs to */
	user m_DomainNode;
	user_node_type m_DomainNodeType;
	int m_Connections;
	unsigned int m_Flags;
	sesmgr_counters_info m_Counters;
	user_node_subclass m_SubclassMode;
	RPC_string m_SubclassInfo;
};

struct sesmgr_query_usersonline_list
{
	sesmgr_query_usersonline m_Data;
	sesmgr_query_usersonline_list *m_pNext;
};

struct sesmgr_query_usersonline_reply
{
	int m_NextIndex; /* Index to follow-up query or 0 for end of list */
	sesmgr_query_usersonline_list *m_pList;
};

/* sesmgr_query_usersonline_reply reply */
union sesmgr_query_usersonline_result switch (usrstat status) {
case USR_OK:
	sesmgr_query_usersonline_reply m_Data;
default:
	void;
};

struct sesmgr_query_usersonline_req
{
	int m_NextIndex; /* sesmgr_query_usersonline_reply.m_NextIndex or 0 to start new query */
};

/* Query hosts */
/* Note: These structures will change without further notice */
struct sesmgr_query_host
{
	in_addr m_Addr;
	int m_bReadyForLogin;
	int m_bIsLocal;
	RPC_string m_DomainName;
	RPC_string m_InterfaceName;
	int m_NumSessions;
	unsigned int m_Flags;
	sesmgr_counters_info m_Counters;
};

struct sesmgr_query_host_list
{
	sesmgr_query_host m_Data;
	sesmgr_query_host_list *m_pNext;
};

struct sesmgr_query_host_reply
{
	int m_NextIndex; /* Index to follow-up query or 0 for end of list */
	sesmgr_query_host_list *m_pList;
};

/* sesmgr_query_host_reply reply */
union sesmgr_query_host_result switch (usrstat status) {
case USR_OK:
	sesmgr_query_host_reply m_Data;
default:
	void;
};

struct sesmgr_query_hostonline_req
{
	int m_NextIndex; /* sesmgr_query_host_reply.m_NextIndex or 0 to start new query */
};

enum sesmgr_sdt
{
	SDT_LoginName,
	SDT_Info,
	SDT_CWD,
	SDT_CurrentFileInfo,
	SDT_CurrentFileProgress,
	SDT_State,
	SDT_Progress,
	SDT_IdleTime,
	SDT_Logout,
	SDT_CPS,
	SDT_LastCMD,
	SDT_invalid
};

struct sesmgr_set_file_progress
{
	hyper m_FileLength;
	hyper m_CurPos;
	int m_Progress;
	int m_CPS;
	int m_CopyProgress;
};

struct sesmgt_set_file_info
{
	RPC_string m_Path;
	RPC_string m_URL;
	sesmgr_set_file_progress m_Progress;
};

struct sesmgt_set_lastcmd_info
{
	RPC_string m_CMD;
	int m_IdleTime;
};

union sesmgr_set_data switch (sesmgr_sdt mode) {
case SDT_LoginName:
case SDT_CWD:
case SDT_Info:
	RPC_string m_Str;
case SDT_LastCMD:
	sesmgt_set_lastcmd_info *m_pLastCmdInfo;
case SDT_CurrentFileInfo:
	sesmgt_set_file_info *m_pFileInfo;
case SDT_CurrentFileProgress:
	sesmgr_set_file_progress *m_pProgress;
case SDT_State:
case SDT_Progress:
case SDT_CPS:
	int m_iVal;
default:
	void;
};

struct sesmgr_set_data_req
{
	hyper m_ID; /* SessionUserSession index ID */
	hyper m_NewIdleTime; /* -1 to keep the old one */
	sesmgr_set_data m_Data;
};


/* Post-transfer notification */

enum sesmgr_transfer_done_status
{
	std_ok, /* File transfer os OK */
	std_rejected /* File transfer is rejected */
};

struct sesmgr_transfer_done_req
{
	hyper m_ID; /* SessionUserSession index ID */
	int m_bIsTmp; /* Tmp file - dir listing */
	int m_bIsFree; /* Free file */
	int m_bIsSending; /* download ? */
	int m_FTPtype;
	RPC_path m_URL; /* URL for file */
	RPC_path m_FsysPath; /* File system path */
	RPC_path m_UserPath; /* The path as seen by the user */
	hyper m_FileBytes;
	int m_TransferSeconds;
	unsigned int m_CPS;
};

/* File history messages */
struct FileHistoryInfoList
{
	/* from FileHistoryEntry */
	RPC_path m_URL;
	unsigned int m_InfoFlags;
	unsigned int m_DLcnt;

	/* from SessionFileHistoryEntry */
	unsigned int m_CPS;
	hyper m_Size;
	hyper m_Time;
	unsigned int m_Flags;
	FileHistoryInfoList *m_pNext;
};

struct sesmgr_file_info_reply
{
	int m_NextHash; /* -1 on end of list */
	int m_NextEntry;
	hyper m_LastTime; /* Last change time for any of the returned items */

	FileHistoryInfoList *m_pList;
};

enum sesmgr_file_info_req_mode
{
	frm_all, /* Request all entries - */
	frm_session, /* Request entries from session */
	frm_user, /* Request entries from SessionUserOnline */
	frm_server /* Request entries from server */
};

struct sesmgr_file_info_req
{
	sesmgr_file_info_req_mode m_Mode;
	hyper m_Req; /* User, session ID or server's user id. 0 when all is requested */
	war_svr_type m_SvrType; /* used for server list query only, set to 0 for other requests */
	int m_NextHash; /* 0 unless all is requested, then use 0 for first query and then value returned by sesmgr_file_info_reply */
	int m_NextEntry; /* 0 for first query, then value returned by sesmgr_file_info_reply */
	hyper m_FromTime; /* Only show entries changed since the given time (WarTime) */
};

union sesmgr_file_info_result switch (usrstat status) {
case USR_OK:
	sesmgr_file_info_reply m_Data;
default:
	void;
};


/* Query to see if a DNS lookup to a host is ready */
/* Require no authentication */
struct sesmgr_query_host_req
{
	in_addr m_Host;
};

struct sesmgr_getsvrlist_reply
{
	user m_User;
	war_svr_type m_Type;
	int m_State;
	int m_Flags;
	int m_NumUsersOnline;
	int m_Port;
};

/* sesmgr_getsvrlist_result reply */
union sesmgr_getsvrlist_result switch (usrstat status) {
case USR_OK:
	sesmgr_getsvrlist_reply m_Servers<>;
default:
	void;
};

enum sesmgr_set_flag_mode
{
	SSFM_USER,
	SSFM_SERVER,
	SSFM_SESSION,
	SSFM_ADDR,
	SSFM_GLOBAL
};

struct sesmgr_server_id
{
	user m_ServerNode;
	war_svr_type m_Type;
};

union sesmgr_setflag_data switch (sesmgr_set_flag_mode Mode) 
{
	case SSFM_USER:
		user m_ClientUser;
	case SSFM_SERVER:
		sesmgr_server_id m_Server;
	case SSFM_SESSION:
		hyper m_SessionID;
	case SSFM_ADDR:
		in_addr m_ClientAddr;
	default:
		void;
};


struct smgt_set_flag_req
{
	int m_Mask; /* Flags to touch */
	int m_Flag; /* Flags to set/clear */
	sesmgr_setflag_data m_Where;
};

/* Do only set flags that are defined. New flags will be defined for the */
/* current version of the protocol! */
enum SesMgrFlagsDef
{
	SMF_LogoffPendingMask =			0x00000fff,
	SMF_LogoffPendingTOD =			0x00000001, /* Logoff due to time of day */
	SMF_LogoffPendingTimeout =	0x00000002, /* Logoff pending due to client timeout */
	SMF_LogoffPendingTime =			0x00000004, /* Logoff pending due to session time limit */
	SMF_LogoffPendingShutdown =	0x00000008, /* Logoff pending due to system shutdown */
	SMF_LogoffPendingKick =			0x00000010,	/* Logoff pending due to operator kick */
	SMF_LogoffASAP =						0x00000100, /* Logoff instantly. Other flag(s) describe why... */
	SMF_AbortTransfer =					0x00001000, /* Abort current transfer */
	SMF_ChatMessageWaiting =		0x00010000,	/* Chat message is waiting */
	SMF_IPchangeNotify =				0x00100000,	/* Servers only - IP list has chagned. Time to call listen() again... */
	SMF_IPHackerFlag =					0x00200000, /* Host only - the host is marked as 'hacker' and login disabled. When set, a deactivation timer is set */
	SMF_ModeSetOnSessions =			0x10000000, /* If set, the flags are applied to the affected sessions, and not on to the shared objects themself */
	SMF_ModeMask =							0xf0000000  /* Mode bits. Not set as flags */
};

struct WarOption
{
	RPC_string m_Name; /* Option name */
	RPC_string m_Value; /* Current value */
	RPC_string m_Default; /* Default value */
	RPC_string m_Description; /* Descriptive text */
};

/* Can search on index or name. If name is a string, index is ignored */
struct sesmgr_option_req
{
	int m_Index;
	RPC_string m_Name;
};

struct sesmgr_option_set
{
	int m_Index;
	RPC_string m_Name;
	RPC_string m_NewValue;
};

union sesmgr_option_result switch (usrstat status) {
case USR_OK:
	WarOption m_Data;
default:
	void;
};

/* Startup options */
struct sesmgr_svr_startup_req
{
	int m_RunAsDaemon; /* boolean - run as daemon/Service */
	int m_StartAtBoot; /* Automatically start the service at system startup ? */
	int m_Priority; /* Priority class */
	RPC_string m_UseName; /* User account to use - owner/system if empty*/
	RPC_string m_UsePasswd; /* Use this password - ignored if UseName is empty */
};

/* Startup options */
struct sesmgr_svr_startup_options
{
	int m_bCanSetStartupOptions; /* bolean - if the server running as a NT service, the options can not be set */
	int m_bCanBeDaemon; /* boolean */
	int m_RuningAsDaemon; /* boolean - running as daemon/Service */
	int m_bCanStartAtBoot; /* boolean */
	int m_StartingAtBoot; /* Automatically start the service at system startup ? */
	RPC_string m_SupportedPrioritoes; /* String in format Pri Name; .. where pri is a number */
	int m_CurrentPriority; /* Priority class */	
	int m_bCanUseSystemAccount; /* boolean */
	int m_bCanUseUserAccount; /* Boolean */
	RPC_string m_UseName; /* User account in use - owner/system if empty*/
	RPC_string m_VisualOSInfo; /* Information string */
};

union sesmgr_svr_startup_options_result switch (usrstat status) {
case USR_OK:
	sesmgr_svr_startup_options m_Data;
default:
	void;
};

/* TMP query log feature */

struct sesmgr_log_event
{
	hyper m_Time;
	RPC_string m_ProgramName;
	unsigned int m_Type;
	hyper m_UserNo;
	RPC_string m_SckName;
	int m_SystemErrno;
	int m_ProgramErrno;
	RPC_string m_Module;
	RPC_string m_SystemErrText;
	RPC_string m_Text;
	hyper m_SessionID;
	int m_BufLen;
	hyper m_SeqCnt; /* Ask for next number on subsequent queries */
};

struct sesmgr_query_log_event_req
{
	hyper m_Event; /* Get an event equal or bigger */
	unsigned int m_Mask; /* Filter type */
};

union sesmgr_query_log_result switch (usrstat status) {
case USR_OK:
	sesmgr_log_event m_Data;
default:
	void;
};

/* Query current credits data for a session */
union sesmgr_ud_restrictions_data_result switch (usrstat status) {
case USR_OK:
	sesmgr_ud_restrictions_data m_Data;
default:
	void;
};

/* CDROM file system support (CDFS) */

enum cdfs_dev_states
{
	CDDEV_IDLE,
	CDDEV_SCANNING,
	CDDEV_READ,
	CDDEV_LOCKED
};

const CML_FIXED_LABEL = 0x00000001; /* Fixed label in a device */

struct cdfs_mount_list
{
	RPC_string m_Name; /* Logical name used to access the drive from the server */
	RPC_string m_PhysPath; /* System dependent path to the CD-ROM player */
	RPC_string m_DirMessageFile; /* Pattern for directory message files */
	RPC_string m_Label; /* Volume label */
	unsigned int m_Flags;
	cdfs_dev_states m_CurrentState;
	cdfs_mount_list *m_pNext;
	int m_pHandle; /* Not used by the RPC interface (ptr to server internal data) */
};

struct cdfs_unit_list
{
	RPC_string m_Name; /* Logical name of the device, used by War */
	RPC_string m_CacheDir; /* Temporary storage for cached files */
	int m_MaxSimAccess; /* Max concurrent IO requests to the unit */
	int m_CacheTimeout; /* Time to keep a file in the cache after all IO is finished (minutes)*/
	int m_LRUnum; /* Number of MB to keep in the cache. */
	int m_NumPendingRequests;
	unsigned int m_Flags;
	cdfs_mount_list *m_pMountList;
	cdfs_unit_list *m_pNext;
	int m_pHandle; /* Not used by the RPC interface (ptr to server internal data) */
};

enum cdfs_chg_mode 
{
	CFDSM_ADDUNIT,
	CDFSM_DELUNIT,
	CDFSM_CHGUNIT,
	CDFSM_ADDDEV,
	CDFSM_DELDEV,
	CDFSM_CHGDEV,
	CDFSM_RESCAN,
	CDFSM_INIT
};

struct cdfs_cng_unit_req
{
	RPC_string m_UnitName;
	cdfs_unit_list m_UnitData;
};

struct cdfs_cng_dev_req
{
	RPC_string m_UnitName; /* CDFSM_ADDDEV only */
	RPC_string m_DeviceName; /* CDFSM_CHGDEV only */
	cdfs_mount_list m_DeviceData;
};

union sesmgr_cdfs_change_req switch(cdfs_chg_mode mode) {
	case CFDSM_ADDUNIT:
	case CDFSM_CHGUNIT:
		cdfs_cng_unit_req m_Unit;
	case CDFSM_DELUNIT:
		RPC_string m_UnitName;
	case CDFSM_ADDDEV:
	case CDFSM_CHGDEV:
		cdfs_cng_dev_req m_Device;
	case CDFSM_DELDEV:
	case CDFSM_RESCAN:
	case CDFSM_INIT:
		RPC_string m_DeviceName;
};

union sesmgr_cdfs_unit_result switch (usrstat status) {
case USR_OK:
	cdfs_unit_list m_Data;
default:
	void;
};


enum sesmgr_qyery_online_mode
{
	sqo_domusers, /* Users online on domain */
	sqo_domausres, /* Anonymous users online on domain */
	sqo_dmaxusres, /* Max users on domain */
	sqo_dmaxausers /* Max anonymous users on domain */
};

/*
 * Remote user service routines
 */
program WARUSER_PROGRAM 
{
	version WARUSER_VERSION 
	{
		void 
		USERPROC_NULL(void) = 0;

		user_attrs_result 
		USRPROC_GETATTR(user) =	1;

		user_create_result 
		USRPROC_CREATENEW(user_create_req) =	2;

		user_attrs_result 
		USRPROC_FIND(user_find_req) =	3;

		user_query_result
		USRPROC_QUERY(war_user_query) = 4;

		user_fetcth_domains_result
		USRPROC_FETCHDOMAINS(void) = 5;

		user_generic_result 
		USRPROC_SETPWDMODE(user_generic_set_int) = 6;

		user_generic_result 
		USRPROC_SETPWD(user_generic_set_str) = 7;

		user_generic_result 
		USRPROC_SETACCESS(user_generic_set_int) = 8;

		user_generic_result 
		USRPROC_SETPTYPE(user_generic_set_int) = 9;

		user_generic_result 
		USRPROC_MOVEUSER(move_user_req) = 10;

		user_generic_result 
		USRPROC_SETLOGINPERMS(login_perms_req) = 11;

		user_query_result /* EFFECTIVE settings are returned */ 
		USRPROC_GENERICSETLIST(generic_list_req) = 12;

		user_generic_result
		USRPROC_SETEXPIERY(user_expiery_req) = 13;

		user_query_result /* EFFECTIVE settings are returned */ 
		USRPROC_SETLOGINTMASK(user_login_time_req) = 14;

		user_query_result /* EFFECTIVE settings are returned */ 
		USRPROC_SETSTAT(user_stat_req) = 15;

		user_generic_result
		USRPROC_SETFTPUDR(ud_restrictions_req) = 16;

		user_generic_result
		USRPROC_SETFSDUP(fs_dup_req) = 17;

		user_generic_result
		USRPROC_SETUINFO(user_info_req) = 18;

		user_generic_result
		USRPROC_SETUGREETING(user_greeting_req) = 19;

		user_generic_result
		USRPROC_SETUSUBCLASS(user_subclass_req) = 20;

		war_user_rpclogin_result
		USRPROC_RPCLOGIN(war_rpclogin_req) = 21;

		warsvr_read_buffer_result
		USRPROC_READBUFFER(warsvr_read_buffer_req) = 22;

		warsvr_generic_str_result
		USRPROC_NT_GET_DC(warusr_NT_get_DC_req) = 23;

		warsvr_buffer_result
		USRPROC_NT_GET_DOMAINS(warusr_NT_get_domains_req) = 24;

		warsvr_buffer_result
		USRPROC_NT_GET_SERVERS(warusr_NT_get_servers_req) = 25;

		warsvr_buffer_result
		USRPROC_NT_GET_GROUPS(warusr_NT_get_groups_req) = 26;

		user_generic_result
		USRPROC_NT_ADD_GROUP(warusr_NT_add_group_req) = 27;

		user_generic_result
		USRPROC_NT_EXCHANGE(warusr_NT_exchange_req) = 28;

		user_generic_result
		USRPROC_DELETE(warusr_delete_req) = 29;

		user_generic_result
		USRPROC_RENAME(warusr_rename_req) = 30;

		user_generic_result
		USRPROC_SET_DAEMON_INFO(war_daemon_info_req) = 31;

		war_user_client_login_result
		USRPROC_CLIENTLOGIN(war_client_login_req) = 32;

		user_get_user_URL_result
		USRPROC_QUERYPATH(user_get_user_URL_req) = 33;

		user_generic_result
		USRPROC_SETFILEATTR(user_set_fs_access_req) = 34;

		user_generic_result
		USRPROC_RESYNCHUSER(user_generic_user_req) = 35;

		sesmgr_query_session_result
		USRPROC_SESMGRQUERY(sesmgr_query_session_req) = 36;

		sesmgr_query_session_result
		USRPROC_SESMGRQUERYMORE(int NextIndex) = 37;

		user_generic_result
		USRPROC_SESMGRUPDATE(sesmgr_set_data_req) = 38;

		user_generic_result
		USRPROC_SESMGRQUERYHOST(sesmgr_query_host_req) = 39;

		void 
		USERPROC_RPCLOGOUT(void) = 40;

		user_generic_result
		USRPROC_SESMGRHELO(hyper) = 41;

		sesmgr_option_result
		USRPROC_SESMGRGETOPTION(sesmgr_option_req) = 42;

		user_generic_result
		USRPROC_SESMGRSETOPTION(sesmgr_option_set) = 43;

		sesmgr_getsvrlist_result
		USRPROC_SESMGRGETSERVERLIST(void) = 44;

		user_generic_result
		USRPROC_SESMGRSETFLAG(smgt_set_flag_req) = 45;
	
		user_generic_result
		USRPROC_CONFIRMFILEOPENACCESS(user_verify_file_open_access_req) = 46;

		user_generic_result
		USRPROC_SESMGRSETCONNPWD(warusr_setpwd_req) = 47;
		
		sesmgr_query_system_info_result
		USRPROC_SESMGRGETSYSINFO(void) = 48;

		sesmgr_query_system_current_result
		USRPROC_SESMGGETCURRINFO(void) = 49;

		sesmgr_query_doiain_result
		USRPROC_SESMGRQUERYDOMAIN(sesmgr_query_domaun_req) = 50;

		sesmgr_query_usersonline_result
		USRPROC_SESMGRQUERYUSERONLINE(sesmgr_query_usersonline_req) = 51;

		sesmgr_query_host_result
		USRPROC_SESMGRQUERYHOSTONLINE(sesmgr_query_hostonline_req) = 52;

		user_generic_result
		USRPROC_SESMGRPOSTTRANSFER(sesmgr_transfer_done_req) = 53;

		sesmgr_query_sysmsg_result
		USRPROC_SESMGRQUERYSYSMSGS(void) = 54;

		sesmgr_get_sysmsg_result
		USRPROC_SESMGRGETSYSMSG(RPC_string) = 55;

		user_generic_result
		USRPROC_SESMGRSETSYSMSG(svr_sysmsg_req) = 56;

		sesmgr_query_macro_result
		USRPROC_SESMGRQUERYMACROS(void) = 57;

		sesmgr_file_info_result
		USRPROC_SESMGRQUERYFILEINFO(sesmgr_file_info_req) = 58;

		sesmgr_svr_startup_options_result
		USRPROC_SVR_QUERY_STARTUPOPTIONS(void) = 59;

		sesmgr_svr_startup_options_result
		USRPROC_SVR_SET_STARTUPOPTIONS(sesmgr_svr_startup_req) = 60;

		sesmgr_query_log_result
		USRPROC_QUERYLOG(sesmgr_query_log_event_req) = 61;

		user_generic_result
		USRPROC_SAVEUSERDB(void) = 62;

		sesmgr_ud_restrictions_data_result
		USRPROC_SESMGRQUERYCREDITS(hyper) = 63;

		sesmgr_cdfs_unit_result
		USRPROC_SESMGRQUERYCDFSDATA(void) = 64;

		user_generic_result
		USRPROC_SESMGRCNGCDFSDATA(sesmgr_cdfs_change_req) = 65;

		user_generic_result
		USRPROC_QUERYONLINE(sesmgr_qyery_online_mode) = 66;

	} = 1;
} = 352250;
 