/* !!Copyright begin */
/*
War Software series
Copyright (C) 1996 - 1998 by Jarle (jgaa) Aase (www.jgaa.com)
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation version 2
of the License - WITH THE FOLLOWING RESTRICTIONS:

  * GOVERNMENTS, GOVERNMENTAL AGENCIES OR COMMERCIAL 
    COMPANIES OWNED 50% OR MORE BY THE GOVERNMENT ARE
    NOT ALLOWED TO USE THE SOFTWARE OR SORCE CODE FOR
    ANY PURPOSE. THIS RESTRICTION ALSO APPLIES FOR
    PROGRAMS DERIVED FROM THE SOURCE CODE, OR PROGRAMS/
    SOURCE CODE THAT USE SPECIAL TECHNICS/ALOGRITHMS
    INVENTED IN THIS SOFTWARE.

  * THE FULL SOURCE CODE IS WITHELD FROM A PUBLIC RELEASE
    UNTIL THE LINUX VERISON IS RELEASED.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
 
/* !!Copyright end */
// WarDaemonManagerDoc.cpp : implementation of the CWarDaemonManagerDoc class
//

#include "stdafx.h"
#include "WarWin32Support.h"
#include "WarDaemonManager.h"
#include "AdvancedServerOptionsDlg.h"
#include "SpyInfo.h"
#include "WaitingOnSvrDlg.h"
#include "ConnectionPropDlg.h"
#include "PasswdSetDlg.h"
#include "UsersOnlineView.h"
#include "SystemMsgsEditDlg.h"
#include "SvrOptionsPropertySheet.h"
#include "SvrOptStartupDlg.h"
#include "SvrOptLogDlg.h"
#include "SvrOptCdFsys.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define m_ServerFlags m_SystemData.m_Flags

/////////////////////////////////////////////////////////////////////////////
// CWarDaemonManagerDoc

IMPLEMENT_DYNCREATE(CWarDaemonManagerDoc, CDocument)

BEGIN_MESSAGE_MAP(CWarDaemonManagerDoc, CDocument)
	//{{AFX_MSG_MAP(CWarDaemonManagerDoc)
	ON_COMMAND(ID_USERMGR, OnUsermgr)
	ON_UPDATE_COMMAND_UI(ID_USERMGR, OnUpdateUsermgr)
	ON_COMMAND(ID_OPTIONS_ADVANCEDSERVERPROPERTIES, OnOptionsAdvancedserverproperties)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_ADVANCEDSERVERPROPERTIES, OnUpdateOptionsAdvancedserverproperties)
	ON_COMMAND(ID_SHUTDOWN, OnShutdown)
	ON_UPDATE_COMMAND_UI(ID_SHUTDOWN, OnUpdateShutdown)
	ON_COMMAND(ID_SHUTDOWNWHENREADY, OnShutdownwhenready)
	ON_UPDATE_COMMAND_UI(ID_SHUTDOWNWHENREADY, OnUpdateShutdownwhenready)
	ON_COMMAND(ID_STARTSERVER, OnStartserver)
	ON_UPDATE_COMMAND_UI(ID_STARTSERVER, OnUpdateStartserver)
	ON_COMMAND(ID_FILE_PROPERTIES, OnFileProperties)
	ON_COMMAND(ID_FILE_REFRESHNOW, OnFileRefresh)
	ON_UPDATE_COMMAND_UI(ID_FILE_REFRESHNOW, OnUpdateFileRefresh)
	ON_COMMAND(ID_CHANGEPWD, OnChangepwd)
	ON_UPDATE_COMMAND_UI(ID_CHANGEPWD, OnUpdateChangepwd)
	ON_COMMAND(ID_CONNECT, OnConnect)
	ON_UPDATE_COMMAND_UI(ID_CONNECT, OnUpdateConnect)
	ON_COMMAND(ID_EDIT_SYSMSGS, OnEditSysmsgs)
	ON_UPDATE_COMMAND_UI(ID_EDIT_SYSMSGS, OnUpdateEditSysmsgs)
	ON_COMMAND(ID_SERVER_PROPERTIES, OnServerProperties)
	ON_UPDATE_COMMAND_UI(ID_SERVER_PROPERTIES, OnUpdateServerProperties)
	ON_COMMAND(ID_SERVER_SAVEUSERDATABASENOW, OnServerSaveuserdatabasenow)
	ON_UPDATE_COMMAND_UI(ID_SERVER_SAVEUSERDATABASENOW, OnUpdateServerSaveuserdatabasenow)
	ON_COMMAND(ID_SERVER_STARTASPROGRAM, OnServerStartasprogram)
	ON_UPDATE_COMMAND_UI(ID_SERVER_STARTASPROGRAM, OnUpdateServerStartasprogram)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWarDaemonManagerDoc construction/destruction

CWarDaemonManagerDoc::CWarDaemonManagerDoc()
{
	m_pUserH = NULL;

	m_Host = "127.0.0.1";
	m_User = DEF_USER;
	m_Password = DEF_PASSWORD;
	m_msRefresh = 2000; // Default 2 seconds refresh delay
	m_pTimer = NULL;
	m_Port = 0; // Auto - use port mapper
	m_bAutoConnect = TRUE; // Go online at once
	m_bRememberPassword = TRUE;
	m_bAbortConnection = FALSE;
	m_pWaitDlgThread = NULL;
	m_pDomainList = NULL;
	m_DomainListLen = 0;
	m_Tag = "WARSVR";
}

CWarDaemonManagerDoc::~CWarDaemonManagerDoc()
{
	if (m_pTimer)
		delete m_pTimer;

	KillUserList();
	KillLogList();

	if (m_pDomainList)
	{
		for(int i = 0; i < m_DomainListLen; i++)
			delete m_pDomainList[i];
		delete m_pDomainList;
	}

	if (m_pUserH)
	{
		delete m_pUserH;
		m_pUserH = NULL;
	}
}

void CWarDaemonManagerDoc::Refresh()
{
	if (!m_pUserH)
		return;

	if (ServerIsLocal() && !ServerIsRunning())
	{
		BrutalDisconnect();
		return;
	}

	// Check what we have to refresh..
	POSITION Pos = GetFirstViewPosition();
	BOOL bDoServers = FALSE;
	BOOL bDoUsers = FALSE;
	BOOL bDoHosts = FALSE;
	BOOL bDoLog = FALSE;
	while (Pos != NULL)
	{
		CView* pView = GetNextView(Pos);
		if (IsWindow(pView->m_hWnd)
			&& pView->IsKindOf(RUNTIME_CLASS(UsersOnlineView)))
		{
			UsersOnlineView *p = (UsersOnlineView *)pView;
			switch(p->m_CurrentMode)
			{
				case UsersOnlineView::ModeServerList:
					bDoServers = TRUE;
					break;
				case UsersOnlineView::ModeUserList:
					bDoUsers = TRUE;
					break;
				case UsersOnlineView::ModeHost:
					bDoHosts = TRUE;
					break;
				case UsersOnlineView::ModeLog:
					bDoLog = TRUE;
			}
		}
	}   
	RefreshUserList();
	RefreshServerStats();

	if (bDoServers)
		RefreshDomainData();

	if (bDoUsers)
		RefreshUsersOnline();

	if (bDoHosts)
		RefreshHostsOnline();

	if (bDoLog)
		RefreshLogList();

	m_RefreshUserList.Reset();
}


BOOL CWarDaemonManagerDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	ConnectionPropDlg Dlg(this);
	if (Dlg.DoModal() != IDOK)
		return FALSE;

	if (m_bAutoConnect)
		ConnectToHost();

	if (m_pTimer)
		delete m_pTimer;
	m_pTimer = WarNotifyTimer::Add(OnGlobalTimer, this, (QWORD)m_pTimer);

	return TRUE;
}

// Unexpected disconnect. (Timeout or user command)
// Shut down everything and clear the views.
void CWarDaemonManagerDoc::BrutalDisconnect()
{
	ClearAllLists();

	if (m_pWaitDlgThread)
	{
		m_pWaitDlgThread->PostThreadMessage(WM_QUIT, 0, 0);
		m_pWaitDlgThread = NULL;
	}

	if (m_pUserH)
	{
		m_pUserH->RPCLogout(); // Safe call - will not block.

		delete m_pUserH;
		m_pUserH = NULL;
	}
}


void CWarDaemonManagerDoc::Serialize(CArchive& ar)
{
	int Version = 1; // Document version
	int Ver;

	if (ar.IsStoring())
	{
		ar << Version;
		ar << m_Host;
		ar << m_User;
		ar << m_Tag;
		ar << m_bRememberPassword;
		if (m_bRememberPassword)
			ar << m_Password;
		ar << m_Port;
		ar << m_bAutoConnect;
		ar << m_msRefresh;
	}
	else
	{
		ar >> Ver;
		if (Ver > Version)
		{
			AfxThrowArchiveException(CArchiveException::badSchema, NULL);
		}
		ar >> m_Host;
		ar >> m_User;
		ar >> m_Tag;
		ar >> m_bRememberPassword;
		if (m_bRememberPassword)
			ar >> m_Password;
		else
			m_Password = "";
		ar >> m_Port;
		ar >> m_bAutoConnect;
		ar >> m_msRefresh;
	}
}

/////////////////////////////////////////////////////////////////////////////
// CWarDaemonManagerDoc diagnostics

#ifdef _DEBUG
void CWarDaemonManagerDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CWarDaemonManagerDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CWarDaemonManagerDoc commands

void CWarDaemonManagerDoc::OnUsermgr() 
{
	if (m_pUserH)
	{
		try
		{
			UserMgrDlg Dlg(this);
			Dlg.DoModal();
		}
		catch(CWarDaemonManagerDoc *e)
		{
			e = e; // Compiler food
			BrutalDisconnect();
		}
	}
}

void CWarDaemonManagerDoc::OnUpdateUsermgr(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pUserH != NULL);
}

BOOL CWarDaemonManagerDoc::CanCloseFrame(CFrameWnd* pFrame) 
{
	return CDocument::CanCloseFrame(pFrame);
}


BOOL CWarDaemonManagerDoc::ConnectToHost()
{
	if (m_pUserH)
		return TRUE; // Alredy connected...

	if (ServerIsLocal() && !CanConnectToServer())
	{
		AfxMessageBox("Server is not running.");
		return FALSE;
	}

	// Initialize callback on RPC requests so that we can update the window..
	WarSocketTLS *pTLS = WarSocketTLS::Ptr();
	pTLS->m_CallbackTimeout = RPCcallbackTimeout;
	pTLS->m_CallbackAllWell = RPCcallbackOK;
	pTLS->m_cli_callbc_lParam = (LPVOID)this;

	CDialog pDlg;
	pDlg.Create(IDD_WAIT_CONNSVR);
	pDlg.ShowWindow(SW_SHOW);
	pDlg.UpdateWindow();

	if (m_pUserH)
		delete m_pUserH;
	m_pUserH = new WarUserHandle;

	WarString wHost = m_Host;
	if (m_Port)
	{
		wHost << ':';
		wHost << m_Port;
	}

	int Err;
	try
	{
		if (!m_pUserH->Create(NULL, wHost))
		{
			if (m_pUserH->GetLastError() == USRERR_COMMFAIL)
				AfxMessageBox("Failed to connect to server. Communication link error.");
			else if (m_pUserH->GetLastError() == 1)
			{
				AfxMessageBox("Failed to connect to server. Invalid password.");
			}
			else
			{
				CString cBuf;
				cBuf.Format("Failed to connect to server. Error code %d", 
					m_pUserH->GetLastError());
				AfxMessageBox(cBuf);
			}
			delete m_pUserH;
			m_pUserH = NULL;
			return FALSE;
		}

		while(!m_pUserH->RPCLogin(m_User, m_Password, &Err))
		{
			if ((m_pUserH->GetLastError() == USRERR_PERM) 
				&& (Err == LF_WAIT_FOR_DNS_LOOKUP))
			{
				Sleep(500);
				continue;
			}	

			CString cBuf;
			cBuf.Format("Failed to connect to server. Error code %d", 
				m_pUserH->GetLastError());
			AfxMessageBox(cBuf);
			delete m_pUserH;
			m_pUserH = NULL;
			return FALSE;
		}
	}
	catch(CWarDaemonManagerDoc *pDoc)
	{
		// User pressed abort, or connection timed out
		ASSERT(pDoc == this);
		pDoc = pDoc; // Compiler food
		delete m_pUserH;
		m_pUserH = NULL;
		return FALSE;
	}

	LoadSystemInfo();
	RefreshServerStats();
	RefreshDomainData();
	RefreshServerFlags();

	UpdateAllViews(NULL, UH_UPDATE_ALL, NULL);

	return TRUE;
}

BOOL CWarDaemonManagerDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
	if (!CDocument::OnOpenDocument(lpszPathName))
		return FALSE;

	if (!m_bRememberPassword)
	{
		ConnectionPropDlg Dlg(this);
		if (Dlg.DoModal() != IDOK)
			return FALSE;
	}
	
	if (m_bAutoConnect)
		ConnectToHost();

	if (m_pTimer)
		delete m_pTimer;
	m_pTimer = WarNotifyTimer::Add(OnGlobalTimer, this, 250);
	
	return TRUE;
}

void CWarDaemonManagerDoc::OnConnect() 
{
	if (!m_pUserH)
	{
		ConnectToHost();
	}
	else
		BrutalDisconnect();
}

void CWarDaemonManagerDoc::OnUpdateConnect(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_pUserH  ? 1 : 0);

	if (ServerIsLocal())
		pCmdUI->Enable(m_pUserH || CanConnectToServer());
	else
		pCmdUI->Enable(TRUE);
}


BOOL CWarDaemonManagerDoc::ServerIsLocal()
{
	return m_Host == "127.0.0.1";
}

BOOL CWarDaemonManagerDoc::ServerIsRunning()
{
	WarString wSName = m_Tag;
	wSName << "smph";

	// See if we are local
	// Only loopback is considered local...
	if (ServerIsLocal())
	{
		// See if the server is running on this machine
		HANDLE h = OpenSemaphore(SEMAPHORE_MODIFY_STATE, FALSE, wSName);
		if (h)
		{
			CloseHandle(h);
			return TRUE;
		}
		int Err = GetLastError();
	}

	return FALSE;
}

BOOL CWarDaemonManagerDoc::CanConnectToServer()
{
	return ServerIsRunning();
}

BOOL CWarDaemonManagerDoc::CanStartServer()
{ 
	WarString wSName = m_Tag;
	wSName << "smph";

	// See if we are local
	// Only loopback is considered local...
	if (ServerIsLocal())
	{
		// See if the server is running on this machine
		HANDLE h = OpenSemaphore(SEMAPHORE_MODIFY_STATE, FALSE, wSName);
		if (h)
		{
			CloseHandle(h);
			return FALSE;
		}

		int Err = GetLastError();
		if ((Err == ERROR_FILE_NOT_FOUND) // NT return value
			|| (Err == ERROR_INVALID_NAME)) // Win95 return value
			return TRUE; // Server not running
	}

	return FALSE;
}

void CWarDaemonManagerDoc::OnOptionsAdvancedserverproperties() 
{
	if (!m_pUserH)
		return;

	try
	{
		AdvancedServerOptionsDlg Dlg(m_pUserH);
		Dlg.DoModal();
	}
	catch(CWarDaemonManagerDoc *e)
	{
		e = e; // Compiler food
		BrutalDisconnect();
	}
}

void CWarDaemonManagerDoc::OnUpdateOptionsAdvancedserverproperties(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pUserH != NULL);
}

void CWarDaemonManagerDoc::OnShutdown() 
{
	if (m_pUserH)
	{
		if (AfxMessageBox(
			"This will shut down the server. Current transfers will be aborted.",
			MB_OKCANCEL) != IDOK)
			return;

		try
		{
			if (m_pUserH->SesMgrSetFlag(SMF_LogoffPendingShutdown | SMF_LogoffASAP, 
				SMF_LogoffPendingShutdown | SMF_LogoffASAP,
				SSFM_GLOBAL))
				m_ServerFlags = (DWORD)m_pUserH->GetLastError();
		}
		catch(CWarDaemonManagerDoc *e)
		{
			e = e; // Compiler food
			BrutalDisconnect();
		}
	}
}

void CWarDaemonManagerDoc::OnUpdateShutdown(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pUserH != NULL);
	pCmdUI->SetCheck((m_ServerFlags & (SMF_LogoffPendingShutdown | SMF_LogoffASAP)) 
		== (SMF_LogoffPendingShutdown | SMF_LogoffASAP));
}

void CWarDaemonManagerDoc::OnShutdownwhenready() 
{
	if (m_pUserH)
	{
		if (AfxMessageBox(
			"This will shut down the server when the current transfers have completed.",
			MB_OKCANCEL) != IDOK)
			return;
		try
		{
			if (m_pUserH->SesMgrSetFlag(SMF_LogoffPendingShutdown, 
				SMF_LogoffPendingShutdown,
				SSFM_GLOBAL))
				m_ServerFlags = (DWORD)m_pUserH->GetLastError();
		}
		catch(CWarDaemonManagerDoc *e)
		{
			e = e; // Compiler food
			BrutalDisconnect();
		}
	}
}

void CWarDaemonManagerDoc::OnUpdateShutdownwhenready(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pUserH != NULL);
	pCmdUI->SetCheck((m_ServerFlags & SMF_LogoffPendingShutdown) != 0);
}

void CWarDaemonManagerDoc::OnStartserver() 
{
	CString Name = "";
	if (!m_pUserH && ServerIsLocal())
	{
		if (IsNT() && WarNTServiceMgr::IsInstalled(m_Tag))
		{
			// Try to start as NT service
			if (WarNTServiceMgr::MyStartService(m_Tag))
			{
				AfxMessageBox("The server was successfully started as a NT service");
				ConnectToHost();
				return;
			}
			
			CString cBuf;
			cBuf.Format("Failed to start the server as a NT service. (%s)",
				(LPCSTR)GetLastErrorText(GetLastError(), TRUE));
			AfxMessageBox(cBuf);
		}

		// Not NT service. Try normal startup
		Startftpserver();
	}
}

void CWarDaemonManagerDoc::Startftpserver() 
{
	if (!m_pUserH && ServerIsLocal())
	{
		extern char far *_pgmptr;
		LPSTR p = strdup(_pgmptr);
		if (p && *p)
		{
			LPSTR pp = strrchr(p,'\\');
			if (pp && *pp)
			{
				*pp = 0;
				CString Pgm;
				// Find out what exe name we have here...
				if (GetFileAttributes("war-ftpd.exe") != 0xffffffff)
					Pgm.Format("%s\\war-ftpd.exe -notservice", p);
				else
					Pgm.Format("%s\\warftpdv1.exe -notservice", p);

				STARTUPINFO si;
				memset(&si,0,sizeof(STARTUPINFO));
				si.cb = sizeof(STARTUPINFO);
				PROCESS_INFORMATION pi;
				memset(&pi,0,sizeof(PROCESS_INFORMATION));

				if (CreateProcess(
					NULL,
					(LPSTR)(LPCSTR)Pgm,
					NULL,
					NULL,
					FALSE,
					CREATE_NEW_PROCESS_GROUP | CREATE_NEW_CONSOLE,
					NULL,
					p,
					&si,
					&pi))
				{
					Sleep(2000);
					ConnectToHost();
				}
				else
				{
					CString cBuf;
					cBuf.Format("Failed to start \"%s\".", Pgm);
					AfxMessageBox(cBuf);
				}

			}
			free(p);
		}
	}
}

void CWarDaemonManagerDoc::OnUpdateStartserver(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable((m_pUserH == NULL) && CanStartServer());
}

void CWarDaemonManagerDoc::OnServerSaveuserdatabasenow() 
{
	CWaitCursor Cursor;
	if (!m_pUserH->SesMgrSaveUserDB())
	{
		CString Msg;
		Msg.Format("Failed to save the user database. Error was %d.", 
			m_pUserH ? m_pUserH->GetLastError() : -1);
		AfxMessageBox(Msg);
	}
}

void CWarDaemonManagerDoc::OnUpdateServerSaveuserdatabasenow(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pUserH != NULL);
}

void CWarDaemonManagerDoc::OnServerStartasprogram() 
{
	if ((m_pUserH == NULL) && CanStartServer())
	{
		Startftpserver();
	}
}

void CWarDaemonManagerDoc::OnUpdateServerStartasprogram(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable((m_pUserH == NULL) && CanStartServer());
}


void CWarDaemonManagerDoc::OnFileRefresh() 
{
	try
	{
		Refresh();
	}
	catch(CWarDaemonManagerDoc *e)
	{
		e = e; // Compiler food
		BrutalDisconnect();
	}
}

void CWarDaemonManagerDoc::OnUpdateFileRefresh(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pUserH != NULL);
}


void CWarDaemonManagerDoc::OnChangepwd() 
{
	if (!m_pUserH)
		return;

	PasswdSetDlg Dlg;
	if (Dlg.DoModal() == IDOK)
	{
		try
		{
			// Set the password for the current (logged in) account
			if (m_pUserH->ChangeSessionPwd(m_Password, Dlg.m_Pwd1))
			{
				m_Password = Dlg.m_Pwd1;
				SetModifiedFlag(TRUE);
			}
		}
		catch(CWarDaemonManagerDoc *e)
		{
			e = e; // Compiler food
			BrutalDisconnect();
		}
	}
}

void CWarDaemonManagerDoc::OnUpdateChangepwd(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pUserH != NULL);
}

void CWarDaemonManagerDoc::OnServerProperties() 
{
	SvrOptionsPropertySheet Sheet("Server Properties");
	SvrOptStartupDlg StartupDlg(this);
	SvrOptLogDlg LogDlg(this);
	SvrOptCdFsys CdFsDlg(this);

	Sheet.AddPage(&StartupDlg);
	Sheet.AddPage(&LogDlg);
	Sheet.AddPage(&CdFsDlg);
	Sheet.DoModal();
}

void CWarDaemonManagerDoc::OnUpdateServerProperties(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pUserH != NULL);
}


void CWarDaemonManagerDoc::OnEditSysmsgs() 
{
	SystemMsgsEditDlg Dlg(this);
	Dlg.DoModal();
}

void CWarDaemonManagerDoc::OnUpdateEditSysmsgs(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pUserH != NULL);
}


void CWarDaemonManagerDoc::RefreshServerFlags()
{
	if (m_pUserH)
	{
		try
		{
			if (m_pUserH->SesMgrSetFlag(0,0,SSFM_GLOBAL))
				m_ServerFlags = (DWORD)m_pUserH->GetLastError();
			m_HeloTimer.Reset();
		}
		catch(CWarDaemonManagerDoc *e)
		{
			e = e; // Compiler food
			BrutalDisconnect();
		}
	}
}

void CWarDaemonManagerDoc::RefreshLogList()
{
	if (!m_pUserH)
	{
		// No connection..
		KillLogList();
		return;
	}


	WarLinkedListItem *pFirstNewItem = NULL;

	QSWORD EventID = 0;
	sesmgr_log_event *pLE = (sesmgr_log_event *)m_LogList.LastPtr();
	if (pLE)
		EventID = pLE->m_SeqCnt +1;

	BOOL bDoUpdate = FALSE;

	while(m_pUserH)
	{
		sesmgr_log_event *p = m_pUserH->SesMgrQueryLog(EventID, 0xffffffff);
		if (p)
		{
			EventID = p->m_SeqCnt +1;
			bDoUpdate = TRUE;
			int Len = p->m_BufLen + (sizeof(sesmgr_log_event) - sizeof(WarLogHdr));
			LPCSTR UserName = NULL;

			// See if we can resolve the user name
			for(WarLinkedListItem *pItem = m_UsersOnline.First()
				; pItem; 
				pItem = m_UsersOnline.Next(pItem))
			{
				UserInfo *pUO = (UserInfo *)m_UsersOnline.Ptr(pItem);
				if (pUO->m_SessionID == p->m_SessionID)
				{
					UserName = pUO->m_Nickname;
					Len += pUO->m_Nickname.GetLength() +1;
					break;
				}
			}

			sesmgr_log_event *pp = (sesmgr_log_event *)(char *)new char[Len];
			*pp = *p;

			// Set strings
			LPSTR pBuf = ((char *)pp) + sizeof(sesmgr_log_event);
			LPSTR ppp = pBuf;
			pp->m_ProgramName = NULL;

			pp->m_SckName = ppp;
			Len = strlen(p->m_SckName) +1;
			memcpy(ppp, p->m_SckName, Len);
			ppp += Len;

			// NB: We use the program pointer for user name...
			// TODO: Change this!
			if (UserName)
			{
				pp->m_ProgramName = ppp;
				Len = strlen(UserName) +1;
				memcpy(ppp, UserName, Len);
				ppp += Len;
			}

			pp->m_Module = ppp;
			Len = strlen(p->m_Module) +1;
			memcpy(ppp, p->m_Module, Len);
			ppp += Len;

			pp->m_SystemErrText = ppp;
			Len = strlen(p->m_SystemErrText) +1;
			memcpy(ppp, p->m_SystemErrText, Len);
			ppp += Len;

			pp->m_Text = ppp;
			Len = strlen(p->m_Text) +1;
			memcpy(ppp, p->m_Text, Len);

			m_LogList.AddLast((LPVOID)pp);
			if (!pFirstNewItem)
				pFirstNewItem = m_LogList.Last();

			if (m_LogList.GetItemCount() > 1024)
			{
				pp = (sesmgr_log_event *)m_LogList.GetAndDeleteFirst();
				delete pp;
			}
		}
		else
			break;
	}

	if (bDoUpdate)
		UpdateAllViews(NULL, UH_LOG_UPDATED, (CObject *)pFirstNewItem);
	else
		UpdateAllViews(NULL, UH_LOG_UPDATED, (CObject *)0xffffffff);
}

// Refresh the user list from the server.
// Update spy windows and user list window(s)
void CWarDaemonManagerDoc::RefreshUserList()
{
	if (!m_pUserH)
	{
		// No connection..
		KillUserList();
		return;
	}

	// Mark all users as not updated
	WarLinkedListItem *pItem;
	for(pItem = m_UsersOnline.First(); pItem; pItem = m_UsersOnline.Next(pItem))
	{
		UserInfo *p = (UserInfo *)m_UsersOnline.Ptr(pItem);
		p->m_Refreshed = FALSE;
	}

	sesmgr_query_session_req Arg;
	Arg.m_ID = 0;
	Arg.m_SvrType = WST_INVALID;
	Arg.m_SvrNode = USER_NONE;
	Arg.m_UserNode = USER_NONE;
	Arg.m_Host.s_addr = INADDR_ANY;
	const sesmgr_query_session_list *pNode = m_pUserH->SesMgrQuerySessions(Arg);
	
	UserInfo *p = NULL;
	// Go trough the list and update the information
	while(m_pUserH && pNode)
	{
		BOOL bExist = FALSE;
		const sesmgr_query_session_reply *pp = &pNode->m_Data;

		// See if the info already exist
		for(pItem = m_UsersOnline.First(); pItem; pItem = m_UsersOnline.Next(pItem))
		{
			p = (UserInfo *)m_UsersOnline.Ptr(pItem);
			if (p->m_QID == pp->m_QID)
			{
				bExist = TRUE;
				break;
			}
		}

		// Update screen content
		BOOL RefreshSpy = !bExist;

#define A(a) {if (p->a != pp->a) {p->a = pp->a; RefreshSpy = TRUE;}}
#define B(a) {if (p->a != pp->a) {p->a = pp->a; if (bExist) p->InvalidateSpy((LPVOID)&p->a);}}

		if (!bExist)
		{
			// Not in list - create it.
			p = new UserInfo;
			m_UsersOnline.AddLast((LPVOID)p);
			p->m_QID = pp->m_QID;
		}

		ASSERT(p != NULL);
		{
			UserUpdateHint Hint(p, pp);
			UpdateAllViews(NULL, bExist ? UH_USER_REFRESHED : UH_USER_ADDED, &Hint);
		}
		
		// Update the internal data
		p->m_Refreshed = TRUE;
		
		A(m_SessionID);
		A(m_UserHost.s_addr);
		A(m_ServerHost.s_addr);
		A(m_DomainName);
		A(m_User);
		p->m_TimeLoggedOn = pp->m_TimeLoggedOn;
		p->m_TimeLastReq = pp->m_TimeLastReq;
		p->m_TimeoutBeforeReset = pp->m_TimeoutBeforeReset;
		A(m_Nickname);
		A(m_UserName);
		A(m_ServerType);
		A(m_ServerName);
		A(m_ProgressMeter);
		A(m_CopyProgressMeter);
		A(m_State);
		A(m_StateInfo);
		B(m_IdleTime);
		A(m_IdleTimeLimit);
		B(m_CWD);
		A(m_CurrentFilePath);
		A(m_CurrentFileURL);
		B(m_CPS);
		B(m_CurrentFileOfs);
		B(m_FileLen);
		A(m_Counters.m_OnlineNow);
		A(m_Counters.m_OnlineTotal);
		A(m_Counters.m_UplFilesNow);
		A(m_Counters.m_UplFilesTotal);
		A(m_Counters.m_DownlFilesNow);
		A(m_Counters.m_DownlFilesTotal);
		A(m_Counters.m_UplBytesTotal);
		A(m_Counters.m_DownlBytesTotal);
		A(m_TotalFilesUploaded);
		A(m_TotalFilesDownloaded);
		A(m_TotalBytesUploaded);
		A(m_TotalBytesDownloaded);
		B(m_LastCMD);
		A(m_Flags);

		if (RefreshSpy)
			p->InvalidateSpy(NULL); // Refresh the whole spy window

#undef A
#undef B

		// Get next node
		if (pNode->m_pNext)
		{
			pNode = pNode->m_pNext;
		}
		else if (pNode->m_NextIndex)
		{
			// Try new lookup
			pNode = m_pUserH->SesMgrQuerySessions(pNode->m_NextIndex);
		}
		else
			break;
	}

	// Delete unresolved items on screen
	for(pItem = m_UsersOnline.First(); pItem;)
	{
		UserInfo *p = (UserInfo *)m_UsersOnline.Ptr(pItem);
		pItem = m_UsersOnline.Next(pItem);

		if (p->m_Refreshed == FALSE)
		{
			UserUpdateHint Hint(p, NULL);
			UpdateAllViews(NULL, UH_USER_DELETED, &Hint);
			m_UsersOnline.DeletePtr((LPVOID)p);
			delete p;
		}
	}

	UpdateAllViews(NULL, UH_RECALC_SPYS, NULL);
}

void CWarDaemonManagerDoc::KillUserList()
{
	UserInfo *p;
	while(p = (UserInfo *)m_UsersOnline.GetAndDeleteFirst())
		delete p;
}

void CWarDaemonManagerDoc::KillLogList()
{
	sesmgr_log_event *p;
	while(p = (sesmgr_log_event *)m_LogList.GetAndDeleteFirst())
		delete p;
}

int CWarDaemonManagerDoc::OnGlobalTimer(LPVOID lParam)
{
	CWarDaemonManagerDoc *p = (CWarDaemonManagerDoc *)lParam;
	p->OnMyTimer();
	return TRUE; // Don't delete the timer
}

void CWarDaemonManagerDoc::OnMyTimer()
{
	// We have to double-check, as the timer might get called
	// during logon (failure dialog box)
	if (!m_pUserH || !m_pUserH->IsConnected())
		return;

	try
	{
		if (m_HeloTimer.TimeOut(60000))
		{
			RefreshServerFlags();
			m_HeloTimer.Reset();
		}

		if (m_msRefresh 
			&& m_RefreshUserList.TimeOut(m_msRefresh))
		{
			Refresh();
		}
	}
	catch(CWarDaemonManagerDoc *e)
	{
		e = e; // Compiler food
		BrutalDisconnect();
	}
}

void CWarDaemonManagerDoc::LoadSystemInfo()
{
	if (!m_pUserH)
		return;

	try
	{
		sesmgr_query_system_info_reply *p
			= m_pUserH->GetSystemInfo();
		
		if (p)
		{
			m_SystemData.m_Name = p->m_Name;
			m_SystemData.m_Flags = p->m_Flags;
			m_SystemData.m_Addr.s_addr = p->m_Addr.s_addr;
			m_SystemData.m_Domain = p->m_Domain;
			m_SystemData.m_Servers = p->m_Servers;
			m_SystemData.m_StartupTime = p->m_StartupTime;
			m_SystemData.m_OS = p->m_OS;
			m_SystemData.m_StartupPath = p->m_StartupPath;
			m_SystemData.m_NetworkInfo = p->m_NetworkInfo;
			if (m_SystemData.m_IP)
			{
				// Crashes program. Represents a inor memory leak...
				//delete m_SystemData.m_IP; 
			}
			m_SystemData.m_IP = NULL;
			if (p->m_IP.m_IP_len)
			{
				m_SystemData.m_IPlines = p->m_IP.m_IP_len;
				m_SystemData.m_IP = new SystemIPData[p->m_IP.m_IP_len];
				sesmgr_ip_info *pp = p->m_IP.m_IP_val;
				for(int i = 0; i < m_SystemData.m_IPlines; i++)
				{
					m_SystemData.m_IP[i].m_Tag = (LPCSTR)pp->m_Tag;
					m_SystemData.m_IP[i].m_Addr.s_addr = pp->m_Addr.s_addr;
					m_SystemData.m_IP[i].m_Domain = (LPCSTR)pp->m_Domain;
					
					++pp;
				}
			}
		}
	}
	catch(CWarDaemonManagerDoc *e)
	{
		e = e; // Compiler food
		BrutalDisconnect();
	}
}

void CWarDaemonManagerDoc::RefreshServerStats()
{
	if (!m_pUserH)
		return;

	try
	{
		const sesmgr_query_system_current_reply *p
			= m_pUserH->GetSystemStats();
		
		if (p)
		{
			m_SystemStats.Update(p);
		}
	}
	catch(CWarDaemonManagerDoc *e)
	{
		e = e; // Compiler food
		BrutalDisconnect();
	}

	UpdateAllViews(NULL, UH_SYSSTAT_REFRESHED, NULL);
}

void CWarDaemonManagerDoc::RefreshHostsOnline()
{
	if (!m_pUserH)
		return;
	
	// Mark all as not refreshed
	WarLinkedListItem *pItem, *pNext;
	for(pItem = m_HostsOnlineList.First()
		; pItem
		; pItem = m_HostsOnlineList.Next(pItem))
	{
		UserHostData *p = (UserHostData *)m_HostsOnlineList.Ptr(pItem);
		p->m_bIsRefreshed = FALSE;
	}

	// Refresh list

	try
	{
		int Index = 0;
		do
		{
			sesmgr_query_host_reply *pData = m_pUserH->GetHostsOnline(Index);
			if (!pData)
				break;

			Index = pData->m_NextIndex;

			for(sesmgr_query_host_list *pList = pData->m_pList
				; pList
				; pList = pList->m_pNext)
			{
				// See if we have the user
				UserHostData *p = LookupHostOnline(pList->m_Data.m_Addr);
				if (!p)
				{
					p = new UserHostData(pList->m_Data);
					m_HostsOnlineList.AddLast(p);
				}
				else
					*p = pList->m_Data;
			}

		} while(Index);
	}
	catch(CWarDaemonManagerDoc *e)
	{
		e = e; // Compiler food
		BrutalDisconnect();
		return;
	}

	// Delete unreferenced items
	for(pItem = m_HostsOnlineList.First()
		; pItem
		; pItem = pNext)
	{
		pNext = m_HostsOnlineList.Next(pItem);

		UserHostData *p = (UserHostData *)m_HostsOnlineList.Ptr(pItem);
		if (p->m_bIsRefreshed == FALSE)
		{
			m_HostsOnlineList.DeleteItem(pItem);
		}
	}

	UpdateAllViews(NULL, UH_HOSTSONLINE_REFRESHED, NULL);
}
	

UserHostData *CWarDaemonManagerDoc::LookupHostOnline(in_addr Addr)
{
	WarLinkedListItem *pItem;
	for(pItem = m_HostsOnlineList.First()
		; pItem
		; pItem = m_HostsOnlineList.Next(pItem))
	{
		UserHostData *p = (UserHostData *)m_HostsOnlineList.Ptr(pItem);
		if (p->m_Addr.s_addr == Addr.s_addr)
			return p;
	}

	return NULL;
}

void CWarDaemonManagerDoc::RefreshUsersOnline()
{
	if (!m_pUserH)
		return;
	
	// Mark all as not refreshed
	WarLinkedListItem *pItem, *pNext;
	for(pItem = m_UserOnlineList.First()
		; pItem
		; pItem = m_UserOnlineList.Next(pItem))
	{
		UserOnlineData *p = (UserOnlineData *)m_UserOnlineList.Ptr(pItem);
		p->m_bIsRefreshed = FALSE;
	}

	// Refresh list

	try
	{
		int Index = 0;
		do
		{
			sesmgr_query_usersonline_reply *pData = m_pUserH->GetUsersOnline(Index);
			if (!pData)
				break;

			Index = pData->m_NextIndex;

			for(sesmgr_query_usersonline_list *pList = pData->m_pList
				; pList
				; pList = pList->m_pNext)
			{
				// See if we have the user
				UserOnlineData *p = LookupUserOnline(pList->m_Data.m_User);
				if (!p)
				{
					p = new UserOnlineData(pList->m_Data);
					m_UserOnlineList.AddLast(p);
				}
				else
					*p = pList->m_Data;
			}

		} while(Index);
	}
	catch(CWarDaemonManagerDoc *e)
	{
		e = e; // Compiler food
		BrutalDisconnect();
		return;
	}

	// Delete unreferenced items
	for(pItem = m_UserOnlineList.First()
		; pItem
		; pItem = pNext)
	{
		pNext = m_UserOnlineList.Next(pItem);

		UserOnlineData *p = (UserOnlineData *)m_UserOnlineList.Ptr(pItem);
		if (p->m_bIsRefreshed == FALSE)
		{
			m_UserOnlineList.DeleteItem(pItem);
		}
	}

	UpdateAllViews(NULL, UH_USERSONLINE_REFRESHED, NULL);
}

UserOnlineData *CWarDaemonManagerDoc::LookupUserOnline(user User)
{
	WarLinkedListItem *pItem;
	for(pItem = m_UserOnlineList.First()
		; pItem
		; pItem = m_UserOnlineList.Next(pItem))
	{
		UserOnlineData *p = (UserOnlineData *)m_UserOnlineList.Ptr(pItem);
		if (p->m_User == User)
			return p;
	}

	return NULL;
}

void CWarDaemonManagerDoc::RefreshDomainData()
{
	if (!m_pUserH)
		return;

	if (m_DomainListLen != ((int)m_SystemStats.m_NumCounters - 1))
	{
		for(int i = 0; i < m_DomainListLen; i++)
			delete m_pDomainList[i];
		delete m_pDomainList;

		m_pDomainList = NULL;
		m_DomainListLen = 0;

		m_DomainListLen = m_SystemStats.m_NumCounters - 1;
		if (m_pUserH && m_DomainListLen >= 1)
		{
			m_pDomainList = new DomainData *[m_DomainListLen];
			for(int i = 0; i < m_DomainListLen; i++)
				m_pDomainList[i] = new DomainData;
		}
	}

	if (!m_pUserH)
		return;

	try
	{
		for(int i = 0; i < m_DomainListLen; i++)
		{
			sesmgr_query_domain_reply *p = m_pUserH->GetDomainInfo(i+1);
			if (p)
			{
				*m_pDomainList[i] = *p;
			}
		}
	}
	catch(CWarDaemonManagerDoc *e)
	{
		e = e; // Compiler food
		BrutalDisconnect();
	}

	UpdateAllViews(NULL, UH_DOMAINS_REFRESHED, NULL);
}

void CWarDaemonManagerDoc::RPCcallbackOK(void *lParam)
{
	CWarDaemonManagerDoc *pMgr = (CWarDaemonManagerDoc *)lParam;
	if (pMgr->m_pWaitDlgThread)
	{
		pMgr->m_pWaitDlgThread->PostThreadMessage(WM_QUIT, 0, 0);
		pMgr->m_pWaitDlgThread = NULL;
	}
}

cli_callbc_command CWarDaemonManagerDoc::RPCcallbackTimeout(timeval& tv, void *lParam)
{
	WarSocketTLS *pTLS = WarSocketTLS::Ptr();
	CWarDaemonManagerDoc *pMgr = (CWarDaemonManagerDoc *)lParam;
	MSG msg;
	while(::PeekMessage(&msg, NULL, WM_PAINT, WM_PAINT, PM_REMOVE)
		|| ::PeekMessage(&msg, NULL, WM_ERASEBKGND, WM_ERASEBKGND, PM_REMOVE)
		|| ::PeekMessage(&msg, NULL, WM_SHOWWINDOW, WM_SHOWWINDOW, PM_REMOVE))
	{
		::DispatchMessage(&msg);
	}

	if (pMgr->m_bAbortConnection 
		|| pTLS->m_cli_callbc_Timer2.TimeOut(1000 * 60 * 5))
	{
		if (pMgr->m_pWaitDlgThread)
		{
			pMgr->m_pWaitDlgThread->PostThreadMessage(WM_QUIT, 0, 0);
			pMgr->m_pWaitDlgThread = NULL;
		}
		throw pMgr;
	}

	if (pTLS->m_cli_callbc_Timer2.TimeOut(2000))
	{
		if (pMgr->m_pWaitDlgThread == NULL)
		{
			WaitingOnSvrDlg::CreateAsynchWaitDlg(
				&pMgr->m_bAbortConnection, &pMgr->m_pWaitDlgThread);
		}
	}

	if (pTLS->m_cli_callbc_Timer1.TimeOut(5000))
	{
		pTLS->m_cli_callbc_Timer1.Reset();
		return CLICC_TRY_AGAIN;
	}
	return CLICC_WAIT;
}

// Clear all lists. Called after disconnect, and
// when the document is closed.
// Deallocates memory.
void CWarDaemonManagerDoc::ClearAllLists()
{
	// Clear session list
	{
		UserInfo *p;
		while(p = (UserInfo *)m_UsersOnline.GetAndDeleteFirst())
			delete p;
	}

	// Clear UserOnlineList
	{
		UserOnlineData *p;
		while(p = (UserOnlineData *)m_UserOnlineList.GetAndDeleteFirst())
			delete p;
	}

	// Clear HostsOnlineList
	{
		UserHostData *p;
		while(p = (UserHostData *)m_HostsOnlineList.GetAndDeleteFirst())
			delete p;
	}

	KillLogList();

	// Clear SystemStats
	delete m_SystemStats.m_pCPSreadings;
	m_SystemStats.m_pCPSreadings = NULL;
	delete m_SystemStats.m_pSessionReadings;
	m_SystemStats.m_pSessionReadings = NULL;
	m_SystemStats.m_NumCaptures = 0;
	m_SystemStats.m_PeekCPS = 0;
	m_SystemStats.m_PeekOnline = 0;
	m_SystemStats.m_NumCounters = 0;
	m_SystemStats.m_CurrentIndex = 0;

	// Clear server list
	if (m_pDomainList)
	{
		for(int i = 0; i < m_DomainListLen; i++)
			delete m_pDomainList[i];
		delete m_pDomainList;
	}
	m_pDomainList = NULL;
	m_DomainListLen = 0;

	UpdateAllViews(NULL, UH_DESTROY, NULL);
}

/////////////////////////////////////////////////////////////////////////////
// UserInfo
/////////////////////////////////////////////////////////////////////////////

UserInfo::UserInfo()
{
	m_Refreshed = FALSE;
	m_QID = -1;
	m_SessionID = -1;
	m_UserHost.s_addr = INADDR_NONE;
	m_ServerHost.s_addr = INADDR_NONE;
	m_DomainName = "";
	m_User = USER_INVALID;
	m_Nickname = "";
	m_UserName = "";
	m_ServerType = WST_INVALID;
	m_ServerName = "";
	m_ProgressMeter = 0;
	m_CopyProgressMeter = 0;
	m_State = 0;
	m_StateInfo = "";
	m_IdleTime = 0;
	m_CWD = "";
	m_CurrentFilePath = "";
	m_CurrentFileURL = "";
}

UserInfo::~UserInfo()
{
	// Resolve spy windows
	
	SpyInfo *ps;
	UserInfo **pp;
	while(ps = (SpyInfo *)m_SpyWin.GetAndDeleteFirst())
		ps->m_pUser = NULL;

	// Resolve references
	while(pp = (UserInfo **)m_ReferencePtrs.GetAndDeleteFirst())
		*pp = NULL;
}

void UserInfo::Register(UserInfo **ppParam)
{
	m_SpyWin.DeletePtr(ppParam);
	m_SpyWin.AddLast(ppParam);
}

void UserInfo::Register(SpyInfo *pParam)
{
	m_SpyWin.DeletePtr(pParam);
	m_SpyWin.AddLast(pParam);
}

void UserInfo::Resign(UserInfo **ppParam)
{
	m_SpyWin.DeletePtr(ppParam);
}

void UserInfo::Resign(SpyInfo *pParam)
{
	m_SpyWin.DeletePtr(pParam);
}


void UserInfo::InvalidateSpy(LPVOID pVar)
{
	WarLinkedListItem *pItem;
	LPRECT pRect = NULL;
	for(pItem = m_SpyWin.First(); pItem; pItem = m_SpyWin.Next(pItem))
	{
		SpyInfo *p = (SpyInfo *)m_SpyWin.Ptr(pItem);
		if (pVar)
		{
			if (pVar == (LPVOID)&m_IdleTime)
				pRect = &p->m_IdleRect;
			else if (pVar == (LPVOID)&m_CWD)
				pRect = &p->m_CwdRect;
			else if (pVar == (LPVOID)&m_LastCMD)
				pRect = &p->m_CmdRect;
			else if (pVar == (LPVOID)&m_FileLen)
				pRect = &p->m_SizeRect;
			else if (pVar == (LPVOID)&m_CurrentFileOfs)
				pRect = &p->m_CurrentRect;
			else if (pVar == (LPVOID)&m_CPS)
				pRect = &p->m_CpsRect;
		}
		
		if (pRect)
			p->InvalidateRect(pRect);
		else
			p->Invalidate();
	}	
}


/////////////////////////////////////////////////////////////////////////////
// UserUpdateHint
/////////////////////////////////////////////////////////////////////////////

UserUpdateHint::UserUpdateHint(const UserInfo *pUserInfo,
		const sesmgr_query_session_reply *pRawInfo)
{
	m_pUserInfo = pUserInfo;
	m_pRawInfo = pRawInfo;
}


void CWarDaemonManagerDoc::OnFileProperties() 
{
	ConnectionPropDlg Dlg(this);
	Dlg.DoModal();
}


void CWarDaemonManagerDoc::OnCloseDocument() 
{
	// Cancel the timer
	if (m_pTimer)
		delete m_pTimer;
	m_pTimer = NULL;

	// Clean up all views
	ClearAllLists();
	
	CDocument::OnCloseDocument();
}


//////////////////////////////////////////////////////////////////////////////
// DomainData
//////////////////////////////////////////////////////////////////////////////

DomainData::DomainData()
{
	m_Port = NULL;
	m_Name = NULL;
	m_NodeName = NULL;
	m_DomainName = NULL;
	m_Addr.s_addr = INADDR_NONE;
	m_Type = WST_INVALID;
	m_State = 0;
	m_Flags = 0;
	m_User = USER_INVALID;
	m_Index = 0;
	memset(&m_CPSdata, 0, sizeof(m_CPSdata));
}

DomainData::~DomainData()
{
	if (m_CPSdata.m_CPSreadings.m_CPSreadings_val)
	{
		delete m_CPSdata.m_CPSreadings.m_CPSreadings_val;
		delete m_CPSdata.m_SessionReadings.m_SessionReadings_val;
	}

	if (m_Name)
		delete m_Name;

	if (m_NodeName)
		delete m_NodeName;

	if (m_DomainName)
		delete m_DomainName;
}


const DomainData& DomainData::operator = (sesmgr_query_domain_reply& Data)
{
	// Check if we have buffers...
	if (m_CPSdata.m_CPSreadings.m_CPSreadings_len 
		!= Data.m_CPSdata.m_CPSreadings.m_CPSreadings_len)
	{
		if (m_CPSdata.m_CPSreadings.m_CPSreadings_val)
		{
			delete m_CPSdata.m_CPSreadings.m_CPSreadings_val;
			delete m_CPSdata.m_SessionReadings.m_SessionReadings_val;
		}
		m_CPSdata.m_CPSreadings.m_CPSreadings_val = new u_int[Data.m_CPSdata.m_CPSreadings.m_CPSreadings_len * sizeof(u_int)];
		m_CPSdata.m_SessionReadings.m_SessionReadings_val = new u_int[Data.m_CPSdata.m_CPSreadings.m_CPSreadings_len * sizeof(u_int)];
	}

	m_CPSdata.m_CurrentIndex = Data.m_CPSdata.m_CurrentIndex;
	m_CPSdata.m_PeekCPS = Data.m_CPSdata.m_PeekCPS;
	m_CPSdata.m_PeekOnline = Data.m_CPSdata.m_PeekOnline;
	m_CPSdata.m_Resolution = Data.m_CPSdata.m_Resolution;
	m_CPSdata.m_msSampleRate = Data.m_CPSdata.m_msSampleRate;
	m_CPSdata.m_CPSreadings.m_CPSreadings_len = Data.m_CPSdata.m_CPSreadings.m_CPSreadings_len;
	memcpy(m_CPSdata.m_CPSreadings.m_CPSreadings_val, Data.m_CPSdata.m_CPSreadings.m_CPSreadings_val,
		sizeof(u_int) * m_CPSdata.m_CPSreadings.m_CPSreadings_len);
	m_CPSdata.m_SessionReadings.m_SessionReadings_len = Data.m_CPSdata.m_SessionReadings.m_SessionReadings_len;
	memcpy(m_CPSdata.m_SessionReadings.m_SessionReadings_val, Data.m_CPSdata.m_SessionReadings.m_SessionReadings_val,
		sizeof(u_int) * m_CPSdata.m_SessionReadings.m_SessionReadings_len);

	if (!m_Name || strcmp(m_Name, Data.m_Name))
	{
		if (m_Name)
			delete m_Name;
		m_Name = StrDup(Data.m_Name);
	}

	if (!m_NodeName || strcmp(m_NodeName, Data.m_NodeName))
	{
		if (m_NodeName)
			delete m_NodeName;
		m_NodeName = StrDup(Data.m_NodeName);
	}

	if (!m_DomainName || strcmp(m_DomainName, Data.m_DomainName))
	{
		if (m_DomainName)
			delete m_DomainName;
		m_DomainName = StrDup(Data.m_DomainName);
	}
	
	m_Port = Data.m_Port;
	m_Type = Data.m_Type;
	m_State = Data.m_State;
	m_Flags = Data.m_Flags;
	m_User = Data.m_User;
	m_Index = Data.m_Index;
	m_Addr = Data.m_Addr;
	m_NodeType = Data.m_NodeType;
	return *this;
}

//////////////////////////////////////////////////////////////////////
// UserOnlineData
//////////////////////////////////////////////////////////////////////

DWORD UserOnlineData::m_CNTNext;

UserOnlineData::UserOnlineData(sesmgr_query_usersonline& Data)
{
	m_Name = NULL;
	m_FullName = NULL;
	m_Domain = NULL;
	m_SubclassInfo = NULL;
	
	m_CntID = m_CNTNext++; // Just to get a unique identifier without messing with 64 bit values

	*this = Data;
}

UserOnlineData::~UserOnlineData()
{
	if (m_FullName)
		delete m_FullName;

	if (m_Name)
		delete m_Name;

	if (m_Domain)
		delete m_Domain;

	if (m_SubclassInfo)
		delete m_SubclassInfo;
}

UserOnlineData& UserOnlineData::operator = (sesmgr_query_usersonline& Data)
{
	if (m_Name && strcmp(m_Name, Data.m_Name))
	{
		delete m_Name;
		m_Name = NULL;
	}

	if (m_Domain && strcmp(m_Domain, Data.m_Domain))
	{
		delete m_Domain;
		m_Domain = NULL;
	}

	if (m_FullName && strcmp(m_FullName, Data.m_FullName))
	{
		delete m_FullName;
		m_FullName = NULL;
	}

	if (m_SubclassInfo && strcmp(m_SubclassInfo, Data.m_SubclassInfo))
	{
		delete m_SubclassInfo;
		m_SubclassInfo = NULL;
	}

	if (!m_Name)
		m_Name = StrDup(Data.m_Name);
	if (!m_FullName)
		m_FullName = StrDup(Data.m_FullName);
	if (!m_Domain)
		m_Domain = StrDup(Data.m_Domain);
	if (!m_SubclassInfo)
		m_SubclassInfo = StrDup(Data.m_SubclassInfo);

	m_Connections = Data.m_Connections;
	m_Flags = Data.m_Flags;
	m_Counters = Data.m_Counters;
	m_User = Data.m_User;
	m_NodeType = Data.m_NodeType;
	m_DomainNodeType = Data.m_DomainNodeType;
	m_DomainNode = Data.m_DomainNode;
	m_SubclassMode = Data.m_SubclassMode;

	m_bIsRefreshed = TRUE;

	return *this;
}

//////////////////////////////////////////////////////////////////////
// UserOnlineData
//////////////////////////////////////////////////////////////////////

UserHostData::UserHostData(sesmgr_query_host& Data)
{
	m_DomainName = NULL;
	m_InterfaceName = NULL;
	
	*this = Data;
}

UserHostData::~UserHostData()
{
	if (m_DomainName)
		delete m_DomainName;

	if (m_InterfaceName)
		delete m_InterfaceName;
}

UserHostData& UserHostData::operator = (sesmgr_query_host& Data)
{
	if (m_DomainName && strcmp(m_DomainName, Data.m_DomainName))
	{
		delete m_DomainName;
		m_DomainName = NULL;
	}

	if (m_InterfaceName && strcmp(m_InterfaceName, Data.m_InterfaceName))
	{
		delete m_InterfaceName;
		m_InterfaceName = NULL;
	}

	if (!m_DomainName)
		m_DomainName = StrDup(Data.m_DomainName);

	if (!m_InterfaceName)
		m_InterfaceName = StrDup(Data.m_InterfaceName);

	m_Addr.s_addr = Data.m_Addr.s_addr;
	m_bReadyForLogin = Data.m_bReadyForLogin;
	m_bIsLocal = Data.m_bIsLocal;
	m_NumSessions = Data.m_NumSessions;
	m_Flags = Data.m_Flags;
	m_Counters = Data.m_Counters;

	m_bIsRefreshed = TRUE;

	return *this;
}

 