/* !!Copyright begin */
/*
War Software series
Copyright (C) 1996 - 1998 by Jarle (jgaa) Aase (www.jgaa.com)
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation version 2
of the License - WITH THE FOLLOWING RESTRICTIONS:

  * GOVERNMENTS, GOVERNMENTAL AGENCIES OR COMMERCIAL 
    COMPANIES OWNED 50% OR MORE BY THE GOVERNMENT ARE
    NOT ALLOWED TO USE THE SOFTWARE OR SORCE CODE FOR
    ANY PURPOSE. THIS RESTRICTION ALSO APPLIES FOR
    PROGRAMS DERIVED FROM THE SOURCE CODE, OR PROGRAMS/
    SOURCE CODE THAT USE SPECIAL TECHNICS/ALOGRITHMS
    INVENTED IN THIS SOFTWARE.

  * THE FULL SOURCE CODE IS WITHELD FROM A PUBLIC RELEASE
    UNTIL THE LINUX VERISON IS RELEASED.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
 
/* !!Copyright end */

#if 0

#if defined(_DEBUG) && !defined(MFC)
#ifdef XXX
#ifdef malloc
#undef malloc
#endif
#ifdef free
#undef free
#endif
#endif // XXX

struct __debugnew_hdr
{
	LPCSTR FileName;
	int nLine;
	int size;
	QWORD nCnt;
	__debugnew_hdr *pNext;
	__debugnew_hdr *pPrev;

	// Origin
	LPCSTR FromFileName;
	int nFromLine;
};

static __debugnew_hdr *pRoot;
static QWORD Cnt;

class MemCriticalSection
{
public:
	MemCriticalSection();
	~MemCriticalSection();
	void Lock();
	void Unlock();
#ifdef _DEBUG
	int m_NumLocks;
#endif
private:
	CRITICAL_SECTION m_CriticalSection;
	BOOL m_bIsInitialized;
};

MemCriticalSection::MemCriticalSection()
{
	m_bIsInitialized = FALSE;
#ifdef _DEBUG
	m_NumLocks = 0;
#endif;
}

MemCriticalSection::~MemCriticalSection()
{
	if (m_bIsInitialized)
		DeleteCriticalSection(&m_CriticalSection);

#ifdef _DEBUG
	ASSERT(m_NumLocks == 0);
#endif;
}

void MemCriticalSection::Lock()
{
	if (!m_bIsInitialized)
	{
		InitializeCriticalSection(&m_CriticalSection);
		m_bIsInitialized = TRUE;
	}
	EnterCriticalSection(&m_CriticalSection);
#ifdef _DEBUG
	++m_NumLocks;
#endif;
}

void MemCriticalSection::Unlock()
{
#ifdef _DEBUG
	--m_NumLocks;
	ASSERT(m_NumLocks >= 0);
#endif;
	LeaveCriticalSection(&m_CriticalSection);
}


static MemCriticalSection MemLock;

void* operator new(size_t nSize, LPCSTR lpszFileName, int nLine);
void* operator new(size_t nSize, LPCSTR lpszFileName, int nLine)
{
	MemLock.Lock();
	__debugnew_hdr *p = (__debugnew_hdr *) malloc(sizeof(__debugnew_hdr) + nSize);

	if (!p)
	{
		MemLock.Unlock();
		throw p;
	}

	p->FileName = lpszFileName;
	p->nLine = nLine;
	p->size = nSize;
	p->nCnt = ++Cnt;
	p->FromFileName = NULL;
	p->nFromLine = 0;

	p->pPrev = NULL;
	p->pNext = pRoot;
	if (pRoot)
		pRoot->pPrev = p;
	pRoot = p;

	MemLock.Unlock();
	return (void *)((char *)p + sizeof(__debugnew_hdr));
}

void __mem_tag(void* ptr, LPCSTR File, int Line)
{
	__debugnew_hdr *p = (__debugnew_hdr *)((char *)ptr - sizeof(__debugnew_hdr));

	p->FromFileName = File;
	p->nFromLine = Line;
}

void operator delete(void* ptr);
void operator delete(void* ptr)
{
	MemLock.Lock();
	static QWORD FncCnt;
	FncCnt++;
	__debugnew_hdr *p = (__debugnew_hdr *)((char *)ptr - sizeof(__debugnew_hdr));

	if (p == pRoot)
	{
		pRoot = p->pNext;
		ASSERT(p->pPrev == NULL);
	}
	else
	{
		ASSERT(p->pPrev != NULL);
	}
	if (p->pPrev)
		p->pPrev->pNext = p->pNext;
	if (p->pNext)
		p->pNext->pPrev = p->pPrev;

	free(p);
	MemLock.Unlock();
}

void __db_memptr(void* ptr);
void __db_memptr(void* ptr)
{
	MemLock.Lock();
	__debugnew_hdr *p = (__debugnew_hdr *)((char *)ptr - sizeof(__debugnew_hdr));
	int i = 0;
	MemLock.Unlock();
}

void __memdmp_content(FILE *fp, __debugnew_hdr *p);
void __memdmp_content(FILE *fp, __debugnew_hdr *p)
{
	int num = min(p->size,16);
	LPCSTR pp = (LPCSTR)p + sizeof(__debugnew_hdr);
	while(num-- > 0)
		fprintf(fp, "%02x ", (unsigned)(unsigned char)*pp++);

	num = min(p->size,16);
	pp = (LPCSTR)p + sizeof(__debugnew_hdr);
	while(num-- > 0)
	{
		fprintf(fp, "%c ", isprint(*pp) ? *pp : '.');
		++pp;
	}

	fprintf(fp, "\n");
};

void __debug_dump_mem();
void __debug_dump_mem()
{
	MemLock.Lock();
	FILE *fp = fopen("warmemdump.txt", "w");
	if (!fp)
	{
		MemLock.Unlock();
		return;
	}

	for(__debugnew_hdr *p = pRoot; p; p = p->pNext)
	{
		if (p->FromFileName)
		{
			fprintf(fp, "(Origin: %s line %d) ", 
				p->FromFileName,
				p->nFromLine);
		}
#ifdef WIN32
		fprintf(fp, "%I64d %s line %d (%d bytes) ->> ",
#else
		fprintf(fp, "%qd %s line %d (%d bytes) ->> ",
#endif
			p->nCnt,
			p->FileName,
			p->nLine,
			p->size);
		__memdmp_content(fp, p);
	}

	fclose(fp);
	MemLock.Unlock();
}

#elif defined(_DEBUG)

// Need this to avoid linker error with debug and MFC
void __mem_tag(void* ptr, LPCSTR File, int Line)
{
	; // Do nothing
}


#endif

#endif // 0

