/* !!Copyright begin */
/*
War Software series
Copyright (C) 1996 - 1998 by Jarle (jgaa) Aase (www.jgaa.com)
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation version 2
of the License - WITH THE FOLLOWING RESTRICTIONS:

  * GOVERNMENTS, GOVERNMENTAL AGENCIES OR COMMERCIAL 
    COMPANIES OWNED 50% OR MORE BY THE GOVERNMENT ARE
    NOT ALLOWED TO USE THE SOFTWARE OR SORCE CODE FOR
    ANY PURPOSE. THIS RESTRICTION ALSO APPLIES FOR
    PROGRAMS DERIVED FROM THE SOURCE CODE, OR PROGRAMS/
    SOURCE CODE THAT USE SPECIAL TECHNICS/ALOGRITHMS
    INVENTED IN THIS SOFTWARE.

  * THE FULL SOURCE CODE IS WITHELD FROM A PUBLIC RELEASE
    UNTIL THE LINUX VERISON IS RELEASED.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
 
/* !!Copyright end */

#include "war.h"
#include "WarWin32Support.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


int WarDosFile::m_OverlapCallbackCnt;

WarDosFile::WarDosFile(WarFile &wFile)
: WarFile(wFile.m_phUser, wFile.m_Client, wFile.m_ConnectionID, wFile.m_Flags)
{
	Reset();
	m_pCallback = wFile.m_pCallback;
}

WarDosFile::WarDosFile(WarFile &wFile, LPCSTR URLtag)
: WarFile(wFile.m_phUser, wFile.m_Client, wFile.m_ConnectionID, wFile.m_Flags)
{
	Reset();
	m_pCallback = wFile.m_pCallback;
	m_URLtag = URLtag;
}

WarDosFile::WarDosFile(WarUserHandle *phUser, user Client, QWORD ConnectionID, DWORD Flags)
: WarFile(phUser, Client, ConnectionID, Flags)
{
	Reset();
}

// Use Destroy()
WarDosFile::~WarDosFile()
{
	ASSERT(!IsOpen());
	//ASSERT(m_bDeletePending == TRUE);
	ASSERT(m_CallbackMode == c_none);
}

void WarDosFile::Reset()
{
	m_hFile = INVALID_HANDLE_VALUE;
	m_bIsOpen = FALSE;
	m_CurrentOffset = 0;
	m_URLtag = "file";
	m_Protocol = file;

	::SetErrorMode(SEM_NOOPENFILEERRORBOX);
}


/////////////////////////////////////////////////////////////////////////
// Overrides
/////////////////////////////////////////////////////////////////////////

BOOL WarDosFile::Open(LPCSTR Path, DWORD Flags)
{
	if (IsOpen())
	{
		m_Errno = WarFileException::FILE_IS_OPEN;
		return FALSE;
	}

	m_Path = Path;
	DWORD Access = 0;
	DWORD ShareMode = 0;
	DWORD Creation = 0;
	DWORD Attributes = 0;
	DWORD Fattrs = GetFileAttributes(Path);
	BOOL bCheckBannedList = FALSE;
	BOOL bExist = (Fattrs != 0xffffffff);
    BOOL bCreatedDir = false;

	if (bExist && !(Flags & (t_dir | t_file)))
	{
		// Determine type
		if (Fattrs & FILE_ATTRIBUTE_DIRECTORY)
			Flags |= t_dir;
		else
			Flags |= t_file;
	}

	if (Flags & read)
	{
		Access |= GENERIC_READ;
		if (!(Flags & (write | append)) && 
			!(Flags & (create | notexist | mustexist | truncate)))
			Flags |= mustexist;
	}
	if (Flags & (write | append))
	{
		Access |= GENERIC_WRITE;
		if (!(Flags & (read)) && 
			!(Flags & (create | notexist | mustexist | truncate)))
			Flags |= create;
	}
	if (Flags & shread)
		ShareMode |= FILE_SHARE_READ;
	if (Flags & shwrite)
		ShareMode |= FILE_SHARE_WRITE;
	if (Flags & exclusive)
		ShareMode &= ~(shwrite | shread);
	if (Flags & sequential)
		Attributes |= FILE_FLAG_SEQUENTIAL_SCAN;
	if ((Flags & callback) && IsNT())
	{
		//Attributes |= FILE_FLAG_OVERLAPPED; Not supported yet...
	}
	
	switch(Flags & (create | notexist | mustexist | truncate))
	{
		case create:
			if (Flags & append)
				Creation = OPEN_ALWAYS;
			else
				Creation = CREATE_ALWAYS;
			break;
		case create | notexist:
			Creation = CREATE_NEW;
			break;
		case create | mustexist:
			Creation = OPEN_EXISTING;
			break;
		case truncate:
		case create | truncate:
		case create | truncate | notexist:
			// TRUNCATE_EXISTING will fail if the file don't exist.
			if (!bExist)
			{
				Creation = CREATE_ALWAYS; // File don't exist
				break;
			}
			// Fall trough
		case mustexist | truncate:
		case create | mustexist | truncate:
			Creation = TRUNCATE_EXISTING;
			bCheckBannedList = TRUE;
			break;
		case notexist:
			Creation = CREATE_ALWAYS;
			bCheckBannedList = TRUE;
			break;
		case mustexist:
			Creation = OPEN_EXISTING;
			break;
		default:
			WarFileException::Throw(WarFileException::INVALID_ARGUMENT);
	}

	if (Flags & t_dir)
	{
		// Open/Create directory
		if (Flags & (create))
		{
			if (!CreateDirectory(Path, NULL))
			{
				Log.LogMsg(LOGF_DEBUG, m_Client, NULL, m_SystemErrno = GetLastError(), 
					m_Errno = WarFileException::GENERAL_ERROR,
					"WarDosFile::Open()", "(%s,%04x) - CreateDirectory() failed.", Path, Flags);
				return FALSE;
			}
            bCreatedDir = true;
		}
		else
		{
			// Check if dir exist
			if (!bExist)
			{
				Log.LogMsg(LOGF_DEBUG, m_Client, NULL, 0,
					m_Errno = WarFileException::NO_SUCH_DIR,
					"WarDosFile::Open()", "(%s,%04x) - No such directory.", Path, Flags);
				return FALSE;
			}

			if (bExist && !(Fattrs & FILE_ATTRIBUTE_DIRECTORY))
			{
				Log.LogMsg(LOGF_DEBUG, m_Client, NULL, 0, 
					m_Errno = WarFileException::NOT_A_DIRECTORY,
					"WarDosFile::Open()", "(%s,%04x) - Not a directory.", Path, Flags);
				return FALSE;
			}


			m_DirMessage.Clear();

			if (bExist 
				&& (Flags & d_message) 
				&& !m_DirMessagePattern.IsEmpty())
			{
				// Read dir message
				WarString DirPath;
				DirPath.MkFsysPath(Path, (LPCSTR)m_DirMessagePattern, NULL);
				FILE *fp = fopen(DirPath, "r");
				if (fp)
				{
					char buf[128];
					while(fgets(buf,sizeof(buf),fp))
					{
						m_DirMessage << buf;
					}
					fclose(fp);
				}
			}
		}
		//goto done;
	}
	else if (Flags & t_file)
	{
		// Check if dir exist
		if (bExist && (Fattrs & FILE_ATTRIBUTE_DIRECTORY))
		{
			Log.LogMsg(LOGF_DEBUG, m_Client, NULL, m_SystemErrno = GetLastError(), 
				m_Errno = WarFileException::PATH_IS_A_DIR,
				"WarDosFile::Open()", "(%s,%04x) - This is a directory, not a file.", Path, Flags);
			return FALSE;
		}
	}

#ifdef WARSVRLIB
	if (m_phUser)
	{
		FLEN Length = 0;
		if (!(Flags & t_dir))
		{
			if (Flags & f_ratio)
			{
				ASSERT((Flags & (t_file | read | append | mustexist | write | create)) 
					== (t_file | read | mustexist));

				m_hFile = CreateFile(Path, Access, ShareMode, NULL, Creation, Attributes, NULL);
				if (m_hFile != INVALID_HANDLE_VALUE)
					Length = GetLength();
			}
		}

		if (!FinalPermsCheck(Path, Length, Flags, bCheckBannedList, bExist))
		{
            if (bCreatedDir)
                ::RemoveDirectory(Path);
			if (m_hFile != INVALID_HANDLE_VALUE)
			{
				CloseHandle(m_hFile);
				m_hFile = INVALID_HANDLE_VALUE;
			}
			return FALSE;
		}
	}
#endif // WARSVRLIB

	if (!(Flags & t_dir))
		Flags |= t_file; // We know for sure it's a file...

	if ((Flags & t_file) == t_file)
	{
		if (m_hFile == INVALID_HANDLE_VALUE)
			m_hFile = CreateFile(Path, Access, ShareMode, NULL, Creation, Attributes, NULL);

		if (m_hFile == INVALID_HANDLE_VALUE)
		{
			Log.LogMsg(LOGF_DEBUG, m_Client, NULL, m_SystemErrno = GetLastError(), 
				m_Errno = WarFileException::GENERAL_ERROR, 
				"WarDosFile::Open()", "(%s,%04x) - CreateFile() failed.", Path, Flags);
			return FALSE;
		}
#define FILE_TESTFLAGS (test_only|create|notexist)
        if ((Flags & FILE_TESTFLAGS) == FILE_TESTFLAGS)
        {   
            ::CloseHandle(m_hFile);
            m_hFile = INVALID_HANDLE_VALUE;
            ::DeleteFile(Path);
            m_bIsOpen = true;
            m_bDeletePending = false;
            m_Flags &= 0xfff00000;
            m_Flags |= (Flags & 0x000fffff);
            return TRUE;
        }
        
        if (Flags & append)
			SeekToEnd();
	}
    
    if (bCreatedDir && ((Flags & FILE_TESTFLAGS) == FILE_TESTFLAGS))
    {
        ::RemoveDirectory(Path);
    }

//done:
    m_Flags &= 0xfff00000;
	m_Flags |= (Flags & 0x000fffff);
	m_bIsOpen = TRUE;

	return TRUE;
}


BOOL WarDosFile::Close()
{
	if (m_hFile != INVALID_HANDLE_VALUE)
	{
		ASSERT(m_bIsOpen == TRUE);
		if (IsDir())
			FindClose(m_hFile);
		else
			CloseHandle(m_hFile);
	}
	m_Path.Clear();
	if (m_pCallback)
	{
		if (m_pCallback->m_pFile == this)
			m_pCallback->m_pFile = NULL;
	}
	if (m_CallbackMode != c_none)
	{
		//CancelIo(m_hFile); Unsupported in win95 (unresolved)
	}
	Reset();
	return TRUE;
}

void WarDosFile::Destroy()
{
	if (IsOpen())
		Close();

	m_bDeletePending = TRUE;
	if (m_CallbackMode == c_none)
		delete this;
}

BOOL WarDosFile::IsOpen()
{
	return m_bIsOpen;
}

DWORD WarDosFile::Read(LPVOID Buf, DWORD Bytes)
{
	if ((m_Flags & (read | t_file)) != (read | t_file))
		WarFileException::Throw(WarFileException::NOT_OPEN_FOR_READ, 0);

	if (!IsOpen() || (m_hFile == INVALID_HANDLE_VALUE))
		WarFileException::Throw(WarFileException::NOT_OPEN, 0);

	m_Errno = 0;

	if (IsCallback() && IsNT() && FALSE)
	{
		m_Overlapped.Internal = 0;
		m_Overlapped.InternalHigh = 0;
		m_Overlapped.Offset = (DWORD)(m_CurrentOffset & 0xffffffff);
		m_Overlapped.OffsetHigh = (DWORD)((m_CurrentOffset >> 16) & 0xffffffff);
		m_Overlapped.hEvent = (HANDLE)this;
		if (!ReadFileEx(m_hFile, Buf, Bytes, &m_Overlapped, OnIoCompletion))
		{
error:
			m_SystemErrno = GetLastError();
			switch(m_SystemErrno)
			{
				case ERROR_HANDLE_EOF:
					m_Errno = WarFileException::END_OF_FILE;
					return 0;
				default:
					WarFileException::Throw(WarFileException::GENERAL_ERROR);		
			}
		}
		m_CallbackMode = c_read;
		m_OverlapCallbackCnt++;
		WarFileException::Throw(WarFileException::CALLBACK_PENDING);
	}
	
	DWORD BytesRead;
	if (ReadFile(m_hFile, Buf, Bytes, &BytesRead, NULL))
	{
		m_SystemErrno = GetLastError();
		if (BytesRead == 0) 
		{
			if (m_SystemErrno == 0)
				m_Errno = WarFileException::END_OF_FILE;
			else
				goto error;
		}
		m_CurrentOffset += BytesRead;
		return BytesRead;
	}

	goto error;
	return 0; // Compiler food
}

void WarDosFile::Write(LPVOID Buf, DWORD Bytes)
{
	if ((m_Flags & (write | t_file)) != (write | t_file))
		WarFileException::Throw(WarFileException::NOT_OPEN_FOR_WRITE, 0);

	if (!IsOpen() || (m_hFile == INVALID_HANDLE_VALUE))
		WarFileException::Throw(WarFileException::NOT_OPEN, 0);

	if (IsCallback() && IsNT() && FALSE)
	{
		m_Overlapped.Internal = 0;
		m_Overlapped.InternalHigh = 0;
		m_Overlapped.Offset = (DWORD)(m_CurrentOffset & 0xffffffff);
		m_Overlapped.OffsetHigh = (DWORD)((m_CurrentOffset >> 16) & 0xffffffff);
		m_Overlapped.hEvent = (HANDLE)this;
		if (!WriteFileEx(m_hFile, Buf, Bytes, &m_Overlapped, OnIoCompletion))
		{
error:
			m_SystemErrno = GetLastError();
			WarFileException::Throw(WarFileException::GENERAL_ERROR);		
		}
		m_CallbackMode = c_write;
		m_OverlapCallbackCnt++;
		WarFileException::Throw(WarFileException::CALLBACK_PENDING);
	}
	
	DWORD BytesWritten;
	if (WriteFile(m_hFile, Buf, Bytes, &BytesWritten, NULL)
		&& (Bytes == BytesWritten)) 
		return;

	goto error;
}

void WarDosFile::Flush()
{
}

void WarDosFile::Seek(FLEN Offset, SeekModes Mode)
{
	if (m_hFile == INVALID_HANDLE_VALUE)
		WarFileException::Throw(WarFileException::NOT_OPEN, 0);

	DWORD Method;
	switch(Mode)
	{
		case FILE_BEGIN:
			Method = begin;
			break;
		case FILE_CURRENT:
			Method = current;
			break;
		case FILE_END:
			Method = end;
			break;
		default:
			ASSERT(TRUE);
	}

	LONG dwlow = (LONG)(Offset & 0xffffffff);
	LONG dwhigh = (LONG)((Offset >> 32) & 0xffffffff);

	dwlow = SetFilePointer(m_hFile, dwlow, &dwhigh, Method);
	if (dwlow == 0xFFFFFFFF  
		&& (m_SystemErrno = GetLastError()) != NO_ERROR )
	{
		WarFileException::Throw(WarFileException::SEEK_FAILED, m_SystemErrno);
	}

	m_CurrentOffset = dwlow;
#ifdef GNUCPP
	m_CurrentOffset |= ((FLEN)dwhigh << 32) & 0xffffffff00000000ULL;
#else
	m_CurrentOffset |= ((FLEN)dwhigh << 32) & 0xffffffff00000000;
#endif
}

void WarDosFile::SeekToBegin()
{
	Seek(0, begin);
}

void WarDosFile::SeekToEnd()
{
	Seek(0, end);
}

FLEN WarDosFile::GetLength(PositionModes Mode)
{
	FLEN CurrentPos = m_CurrentOffset, Rval;
	SeekToEnd();
	Rval = m_CurrentOffset;
	Seek(CurrentPos);
	return Rval;
}

WarTime WarDosFile::GetModifyTime()
{
	WarTime wt;
	BY_HANDLE_FILE_INFORMATION fi;

	if (GetFileInformationByHandle(m_hFile, &fi))
	{
		wt = fi.ftLastWriteTime;
		return wt;
	}

	WarFileException::Throw(WarFileException::GENERAL_ERROR, GetLastError());

	return 0;
}

void WarDosFile::SetLength(FLEN NewLen)
{
	WarFileException::Throw(WarFileException::NO_SUPPORT);
}

FLEN WarDosFile::GetPositon(PositionModes Mode)
{
	return m_CurrentOffset;
}

WarString WarDosFile::GetFileName(NameModes Mode)
{
	LPCSTR p = m_Path;
	LPCSTR pp = "";
	WarString ws;
	char buf[4];

	switch(Mode)
	{
		case PhysicalNameOnly:
			if (p && *p)
			{
				pp = p + strlen(p) - 1;
				if ((pp > p) && (*pp == '\\') || (*pp == '/'))
					--pp;
				while((pp > p) && (*pp != '\\') && (*pp != '/'))
					--pp;
				if ((*pp == '\\') || (*pp == '/'))
					pp++;
			}
			else
				pp = NULL;
			break;
		case PhysicalPath:
			pp = p;
			break;
		case PhysicalPathOnly:
			if (p && *p)
			{
				ws = p;
				pp = (LPCSTR)ws + strlen(ws) - 1;
				if ((pp > (LPCSTR)ws) && (*pp == '\\') || (*pp == '/'))
					--pp;
				while((pp > p) && (*pp != '\\') && (*pp != '/'))
					--pp;
				if ((*pp == '\\') || (*pp == '/'))
					pp++;
				if (pp)
					ws.ReleaseBuffer(pp - p);
			}
			else
				pp = NULL;
			break;
		case URL:
			ws.Printf("%s://%s", m_URLtag, p);
			pp = ws;
			break;
		case PhysicalDrive:
			if (*p && p[1] == ':')
			{
				buf[0] = *p;
				buf[1] = 0;
				pp = buf;
				break;
			}
			WarFileException::Throw(WarFileException::NOT_A_DRIVE);
		default:
			return WarFile::GetFileName(Mode);
	}

	if (!pp || !*pp)
		WarFileException::Throw(WarFileException::NOT_A_NAME);
	return pp;
}

void WarDosFile::Stat(PositionModes Mode, WarFStat &buffer)
{
	WarFileException::Throw(WarFileException::NO_SUPPORT);
}

void WarDosFile::StatInfo(WarFStat &buffer, WarStatInfo &Info, BOOL bGetAllGroups)
{
	WarFileException::Throw(WarFileException::NO_SUPPORT);
}

BOOL WarDosFile::MoveNext()
{
	if (CanBrowse())
	{
		if (FindNextFile(m_hFile, &m_FindData))
			return TRUE;
		FindClose(m_hFile);
		m_hFile = INVALID_HANDLE_VALUE;
	}

	return FALSE;
}

BOOL WarDosFile::Scan(LPCSTR Pattern, DWORD Flags, WarFileList *pFileList)
{
	if (!IsOpen() || !IsDir())
		WarFileException::Throw(WarFileException::DIR_NOT_OPEN);

	if (m_hFile != INVALID_HANDLE_VALUE)
		::FindClose(m_hFile);

	WarString wb(m_Path);
	wb.AddFsysPath(Pattern, NULL);

	m_hFile = ::FindFirstFile(wb, &m_FindData);

	if (pFileList && (m_hFile != INVALID_HANDLE_VALUE))
	{
		// Fill in list
		do
		{
			if (m_FindData.dwFileAttributes & (FILE_ATTRIBUTE_HIDDEN | FILE_ATTRIBUTE_SYSTEM | FILE_ATTRIBUTE_TEMPORARY))
				continue;

			WarFStat *p = new WarFStat;
			p->m_Name = StrDup(m_FindData.cFileName);
			p->m_Title = NULL;
			p->m_ctime = m_FindData.ftCreationTime;
			p->m_mtime = m_FindData.ftLastWriteTime;
			p->m_atime = m_FindData.ftLastAccessTime;
			p->m_size = m_FindData.nFileSizeLow;
#ifdef GNUCPP
			p->m_size |= (FLEN)(m_FindData.nFileSizeHigh << 32) & 0xffffffff00000000ULL;
#else
			p->m_size |= (FLEN)(m_FindData.nFileSizeHigh << 32) & 0xffffffff00000000;
#endif
			p->m_Owner = USER_SYSTEM;
			p->m_Group = USER_SYSTEM;
			p->m_Links = 1;
			p->m_pFather = NULL;
			p->m_IsFsAccessNode = FALSE;
			p->m_Perms = 0766;
			if (m_FindData.dwFileAttributes & FILE_ATTRIBUTE_READONLY)
			{
				p->m_Perms &= ~0222;
				p->m_Perms |= NODE_READONLY;
			}
			if (m_FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
				p->m_Perms |= NODE_DIR;

			// Now - Add the entry
			pFileList->AddUnique(p);

		} while (::FindNextFile(m_hFile, &m_FindData));
		::FindClose(m_hFile);
		m_hFile = INVALID_HANDLE_VALUE;
		return TRUE;
	}
	return m_hFile != INVALID_HANDLE_VALUE;
}


BOOL WarDosFile::CanBrowse()
{
	ClearErrors();

	if (!IsOpen())
		m_Errno = WarFileException::DIR_NOT_OPEN;
	else if (!IsDir())
		m_Errno = WarFileException::NOT_A_DIRECTORY;
	else if (m_hFile == INVALID_HANDLE_VALUE)
		m_Errno = WarFileException::END_OF_FILE;

	return m_Errno == 0;
}

void WarDosFile::DoDelete(LPCSTR Path)
{
	if (!DeleteFile(Path))
	{
		m_SystemErrno = GetLastError();
		if (m_SystemErrno == ERROR_FILE_NOT_FOUND) // Correct error ??
		{
			if (!RemoveDirectory(Path))
				WarFileException::Throw(WarFileException::GENERAL_ERROR, GetLastError());
		}
	}
}

void WarDosFile::DoDelete()
{
	if (!IsOpen() || !(m_Flags & unlink))
		WarFileException::Throw(WarFileException::FILE_NOT_OPEN);

	WarString Path = GetFileName(PhysicalPath);
	BOOL bIsDir = IsDir();

	Close();

	BOOL Success = TRUE;
	if (bIsDir)
		Success = ::RemoveDirectory(Path);
	else
		Success = ::DeleteFile(Path);

	if (!Success)
	{
		int Err = GetLastError();
		Log.LogMsg(LOGF_DEBUG,0,NULL,Err,WarFileException::GENERAL_ERROR,
			"WarDosFile::DoDelete()", "System call failed.");
		WarFileException::Throw(WarFileException::GENERAL_ERROR, Err);
	}

	Log.LogMsg(LOGF_DEBUG,0,NULL,0,0,
		"WarDosFile::DoDelete()", "Path \"%s\" is deleted.", (LPCSTR)Path);
}

// Move file. The argument is a pointer to an open file.
// Note: Both files must be open for write. The destrination file
// must be open for write, but not exist physically!)
void WarDosFile::DoMove(WarFile *pFile)
{
	ClearErrors();

	if (!pFile)
		WarFileException::Throw(WarFileException::INVALID_ARGUMENT);
	
	// Check that files are same type
	if (((m_Flags & t_file)	&& !(pFile->m_Flags & t_file))
		|| ((m_Flags & t_dir) && !(pFile->m_Flags & t_dir)))
	{
		Log.LogMsg(LOGF_DEBUG,0,NULL,m_SystemErrno, WarFileException::NOT_SAME_TYPES,
			"WarFile::DoMove()", "Source and destionation is not the same type (file/dir).");
		WarFileException::Throw(WarFileException::NOT_SAME_TYPES);
	}

	// Check that both files are open for write
	if (!IsOpen()
	 || !(m_Flags & write)
	 || !pFile->IsOpen()
	 || !(pFile->m_Flags & write))
	{
		WarFileException::Throw(WarFileException::FILE_NOT_OPEN);
	}
	
	// Check that file systems are compatible
	switch(pFile->m_Protocol)
	{
		case file:
		case vfsys:
			break;
		default:
			Log.LogMsg(LOGF_DEBUG,0,NULL,m_SystemErrno, WarFileException::UNSUPPORTED_FSYS,
				"WarFile::DoMove()", "source and destionation are incompatibe file systems.");
			pFile->DoDelete();
			WarFileException::Throw(WarFileException::UNSUPPORTED_FSYS);
	}

	// Do the job...
	WarString OldPath = GetFileName(PhysicalPath);
	WarString NewPath = pFile->GetFileName(PhysicalPath);

	Close();
	if (!::MoveFile(OldPath, NewPath))
	{
		int Err = GetLastError();
		Log.LogMsg(LOGF_DEBUG,0,NULL,Err,WarFileException::GENERAL_ERROR,
			"WarDosFile::DoDelete()", "::MoveFile(%s,%s) failed.",
			(LPCSTR)OldPath, (LPCSTR)NewPath);
		WarFileException::Throw(WarFileException::GENERAL_ERROR, Err);
	}
}

void CALLBACK WarDosFile::OnIoCompletion(DWORD dwErrorCode, // completion code
																DWORD dwNumberOfBytesTransfered,  // number of bytes transferred
																LPOVERLAPPED lpOverlapped         // pointer to structure with I/O information
																)
{
	WarDosFile *p = (WarDosFile *)lpOverlapped->hEvent;
	m_OverlapCallbackCnt--;
	ASSERT(m_OverlapCallbackCnt >= 0);
	if (p->m_bDeletePending)
	{
		p->m_CallbackMode = c_none;
		p->Destroy();
		return;
	}

	ASSERT(p->m_pCallback != NULL);
	p->m_CurrentOffset += dwNumberOfBytesTransfered;
	
	if (dwErrorCode)
	{
		p->m_Errno = WarFileException::GENERAL_ERROR_ON_CALLBACK;
		p->m_SystemErrno = GetLastError();
	}
	else 
		p->ClearErrors();

	switch(p->m_CallbackMode)
	{
		case c_read: p->m_pCallback->OnRead(dwErrorCode == 0, dwNumberOfBytesTransfered);
		case c_write: p->m_pCallback->OnWrite(dwErrorCode == 0, dwNumberOfBytesTransfered);
	}
}

BOOL WarDosFile::ExpandPath(LPCSTR Path)
{
	WarString cBuf;

	while(*Path)
	{
		if ((*Path == '\\') || (*Path == '/'))
		{
			if (!cBuf.IsEmpty())
			{
				DWORD attr = ::GetFileAttributes(cBuf);
				if (attr == 0xFFFFFFFF)
				{
					if (!::CreateDirectory(cBuf, NULL))
						return FALSE;
				}
				else if (!(attr & FILE_ATTRIBUTE_DIRECTORY))
					return FALSE;
			}
			cBuf += FSYS_SLASH;
			++Path;
			continue;
		}
		cBuf += *(Path++);
	}
	return TRUE;
}

FLEN WarDosFile::DoGetFreeDiskSpace(LPCSTR Path)
{
	WarString MyPath = Path;

	MyPath.CutAtLastCh('\\');

	DWORD SectorsPerCluster, BytesPerSector, NumberOfFreeClusters, TotalNumberOfClusters;
	if (!GetDiskFreeSpace(MyPath,&SectorsPerCluster,&BytesPerSector,&NumberOfFreeClusters,&TotalNumberOfClusters))
		return 0;

	return (FLEN)NumberOfFreeClusters *	(FLEN)SectorsPerCluster *	(FLEN)BytesPerSector;
}
 