/* !!Copyright begin */
/*
War Software series
Copyright (C) 1996 - 1998 by Jarle (jgaa) Aase (www.jgaa.com)
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation version 2
of the License - WITH THE FOLLOWING RESTRICTIONS:

  * GOVERNMENTS, GOVERNMENTAL AGENCIES OR COMMERCIAL 
    COMPANIES OWNED 50% OR MORE BY THE GOVERNMENT ARE
    NOT ALLOWED TO USE THE SOFTWARE OR SORCE CODE FOR
    ANY PURPOSE. THIS RESTRICTION ALSO APPLIES FOR
    PROGRAMS DERIVED FROM THE SOURCE CODE, OR PROGRAMS/
    SOURCE CODE THAT USE SPECIAL TECHNICS/ALOGRITHMS
    INVENTED IN THIS SOFTWARE.

  * THE FULL SOURCE CODE IS WITHELD FROM A PUBLIC RELEASE
    UNTIL THE LINUX VERISON IS RELEASED.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
 
/* !!Copyright end */

#include "war.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


WarFileException::WarFileException()
{
}

WarFileException::~WarFileException()
{
}


void WarFileException::Throw(int Errno, int SystemError)
{
	WarFileException *p = new WarFileException;
	p->m_Errno = (Errors)Errno;
	p->m_SystemError = SystemError;

	throw p;
}

////////////////////////////////////////////////////////////////////////
// WarFStat
////////////////////////////////////////////////////////////////////////

WarFStat::WarFStat()
{
	m_Name = NULL;
	m_Title = NULL;
	m_size = -1;
	m_Perms = 0;
	m_Owner = USER_INVALID;
	m_Group = USER_INVALID;
	m_MoreGroups = NULL;
	m_Links = 0;
	m_dev = 0;
	m_Inode = 0;
	m_pFather = NULL;
	m_IsFsAccessNode = FALSE;
	m_DLC = -1; // Unassigned
}

WarFStat::~WarFStat()
{
	if (m_Name)
		delete m_Name;
	if (m_Title)
		delete m_Title;
	if (m_MoreGroups)
		delete m_MoreGroups;
}

void WarFStat::MakeFileNameASCII()
{
	LPSTR p = (LPSTR)m_Name;
	
	while(*p)
	{
		if (!isprint(*p))
			*p = '?';
		++p;
	}
}

LPCSTR WarFStat::GetComment()
{
	if (m_IsFsAccessNode)
		return "";
	return m_Title ? m_Title : "";
}

////////////////////////////////////////////////////////////////////////
// WarStatInfo
////////////////////////////////////////////////////////////////////////

WarStatInfo::WarStatInfo()
{
	m_Owner = NULL;
	m_Group = NULL;
	m_MoreGroups = NULL;
	m_buffer = NULL;
}

WarStatInfo::~WarStatInfo()
{
	if (m_buffer)
		delete m_buffer;
}

////////////////////////////////////////////////////////////////////////
// WarFile
////////////////////////////////////////////////////////////////////////

WarFileCallback::WarFileCallback()
{
	m_pFile = NULL;
}

WarFileCallback::~WarFileCallback()
{
}

void WarFileCallback::SetRef(WarFile *pFile)
{
	if (m_pFile)
	{
		// dereference the old file. Panding callbacks will be ignored.
		ASSERT(m_pFile->m_pCallback == this);
		m_pFile->SetCallback(NULL);
	}

	m_pFile = pFile;
	if (pFile)
		pFile->SetCallback(this);
}

////////////////////////////////////////////////////////////////////////
// WarFile
////////////////////////////////////////////////////////////////////////

WarString WarFile::m_DirMessagePattern;

WarFile::WarFile(DWORD Flags)
{
	m_phUser = NULL;
	m_Client = USER_INVALID;
	m_ConnectionID = 0;
	m_Flags = Flags;
	m_CallbackMode = c_none;
	m_pCallback = NULL;
	m_bIsFree = FALSE;
	ClearErrors();
}

WarFile::WarFile(WarUserHandle *phUser, user Client, QWORD ConnectionID, DWORD Flags)
{
	m_phUser = phUser;
	m_Client = Client;
	m_ConnectionID = ConnectionID;
	m_Flags = Flags;
	m_CallbackMode = c_none;
	m_pCallback = NULL;
	m_bIsFree = FALSE;
	ClearErrors();
}

WarFile::~WarFile()
{
	if (IsOpen())
		Close();
	if (m_pCallback && (m_pCallback->m_pFile == this))
	{
		ASSERT(m_CallbackMode == c_none);
		//delete m_pCallback;
		m_pCallback->SetRef(NULL); // Release reference
		m_pCallback = NULL;
	}
}

LPCSTR WarFile::GetDirMessage()
{
	return m_DirMessage.IsEmpty() ? "" : m_DirMessage;
}

BOOL WarFile::SetDirMessagePattern(LPCSTR Pattern)
{
	m_DirMessagePattern = Pattern;
	return TRUE;
}
	

BOOL WarFile::Open(LPCSTR Path, DWORD Flags)
{
	m_Errno = WarFileException::NO_SUPPORT;
	return FALSE;
}

#ifdef WARSVRLIB

BOOL WarFile::AccessCheck(user User, WarSocket *pSock, DWORD OpenFlags, DWORD Flags, int& Err, LPCSTR Path)
{
	Err = 0;

	// Security check
	if (Flags & AF_DENY)
	{
		Err = WarFileException::ACCESS_DENIED;
		Log.LogMsg(LOGF_DEBUG,User,pSock,0,Err,
			"WarFile::AccessCheck()", "(%s) - Access denied (AF_DENY)", Path);
		return FALSE;
	}

	// Check perms
	if ((OpenFlags & WarFile::execute) && !(Flags & AF_EXECUTE))
	{
		Err = WarFileException::ACCESS_DENIED;
		Log.LogMsg(LOGF_DEBUG,User,pSock,0,Err,
			"WarFile::AccessCheck()", 
			"(%s) - Access denied (AF_EXECUTE)", LogicalPath);
		return FALSE;
	}

	if (OpenFlags & WarFile::t_dir)
	{
		// Special for directories
		if ((OpenFlags & WarFile::read) && !(Flags & AF_LIST))
		{
			Err = WarFileException::ACCESS_DENIED;
			Log.LogMsg(LOGF_DEBUG,User,pSock,0,Err,
				"WarFile::AccessCheck()", 
				"(%s) - Access denied (AF_LIST)", LogicalPath);
			return FALSE;
		}

		if ((OpenFlags & WarFile::create) && !(Flags & AF_CREATE))
		{
			Err = WarFileException::ACCESS_DENIED;
			Log.LogMsg(LOGF_DEBUG,User,pSock,0,Err,
				"WarFile::AccessCheck", 
				"(%s) - Access denied (AF_CREATE)", Path);
			return FALSE;
		}
	}
	else
	{
		// Special for files
		if ((OpenFlags & WarFile::read) && !(Flags & AF_READ))
		{
			Err = WarFileException::ACCESS_DENIED;
			Log.LogMsg(LOGF_DEBUG,User,pSock,0,Err,
				"WarFile::AccessCheck()", 
				"(%s) - Access denied (AF_READ)", LogicalPath);
			return FALSE;
		}

		if ((OpenFlags & WarFile::create) && !(Flags & AF_WRITE))
		{
			Err = WarFileException::ACCESS_DENIED;
			Log.LogMsg(LOGF_DEBUG,User,pSock,0,Err,
				"WarFile::AccessCheck()", 
				"(%s) Access denied (AF_WRITE)", Path);
			return FALSE;
		}
	}
	if ((OpenFlags & (WarFile::write | WarFile::append)) && !(Flags & AF_WRITE))
	{
		Err = WarFileException::ACCESS_DENIED;
		Log.LogMsg(LOGF_DEBUG,User,pSock,0,Err,
			"WarFile::AccessCheck()", 
			"(%s) - Access denied (AF_WRITE)", Path);
		return FALSE;
	}

	return TRUE;
}


BOOL WarFile::FinalPermsCheck(LPCSTR Path, 
	FLEN Length, 
	DWORD Flags, 
	BOOL bCheckBannedList, 
	BOOL bExist)
{
	ClearErrors();
	if (m_phUser)
	{
		if (!m_phUser->VerifyFileOpenAccess(m_Client, 
			m_ConnectionID,
			GetFileName(URL), 
			GetFileName(LogicalPath),
			GetFileName(PhysicalNameOnly),
			Length,	Flags, bCheckBannedList, bExist))
		{
			Log.LogMsg(LOGF_DEBUG, m_Client, NULL, m_phUser->GetLastError(), 0, 
				"WarFile::FinalPermsCheck", 
				"(%s,%04x) - Permission denied by user database.", Path, Flags);
			switch(m_phUser->GetLastError())
			{
				case USRERR_ACCES:
					m_Errno = WarFileException::ACCESS_DENIED;
					break;
				case USRERR_BANNED:
					m_Errno = WarFileException::FILE_IS_BANNED;
					break;
				case USRERR_FBIG:
					m_Errno = WarFileException::NO_CREDITS;
					break;
				default:
					m_Errno = WarFileException::COMM_ERROR;
					break;
			}
		}
		
		m_bIsFree = ((int)m_phUser->GetLastError() & AF_FREE) != 0;
	}
	
	return m_Errno == 0;
}
#endif // WARSVRLIB

BOOL WarFile::Close()
{
	m_Errno = WarFileException::NO_SUPPORT;
	return FALSE;
}

BOOL WarFile::IsOpen()
{
	return FALSE;
}

BOOL WarFile::IsFree()
{
	return m_bIsFree;
}

DWORD WarFile::Read(LPVOID Buf, DWORD Bytes)
{
	WarFileException::Throw(WarFileException::NO_SUPPORT);
	return 0; // Compiler food
}

void WarFile::Write(LPVOID Buf, DWORD Bytes)
{
	WarFileException::Throw(WarFileException::NO_SUPPORT);
}

DWORD WarFile::Printf(LPSTR Format, ...)
{
	WarString wBuf;
	va_list ap;
	va_start(ap, Format);

	int Rval = wBuf.vprintf(Format, ap);

	va_end(ap);

	Write((LPVOID)(LPCSTR)wBuf, wBuf.GetLength());

	return Rval;
}

void WarFile::Flush()
{
}

void WarFile::Seek(FLEN Offset, SeekModes Mode)
{
	WarFileException::Throw(WarFileException::NO_SUPPORT);
}

void WarFile::SeekToBegin()
{
	WarFileException::Throw(WarFileException::NO_SUPPORT);
}

void WarFile::SeekToEnd()
{
	WarFileException::Throw(WarFileException::NO_SUPPORT);
}

FLEN WarFile::GetLength(PositionModes Mode)
{
	WarFileException::Throw(WarFileException::NO_SUPPORT);
	return 0; // Compiler food
}

void WarFile::SetLength(FLEN NewLen)
{
	WarFileException::Throw(WarFileException::NO_SUPPORT);
}

FLEN WarFile::GetPositon(PositionModes Mode)
{
	WarFileException::Throw(WarFileException::NO_SUPPORT);
	return 0; // Compiler food
}


WarTime WarFile::GetModifyTime()
{
	WarFileException::Throw(WarFileException::NO_SUPPORT);
	return 0; // Compiler food
}


WarString WarFile::GetFileTitle()
{
	return GetFileName();
}

void WarFile::Stat(PositionModes Mode, WarFStat &buffer)
{
	WarFileException::Throw(WarFileException::NO_SUPPORT);
}

void WarFile::StatInfo(WarFStat &buffer, WarStatInfo &Info, BOOL bGetAllGroups)
{
	WarFileException::Throw(WarFileException::NO_SUPPORT);
}

BOOL WarFile::MoveNext()
{
	WarFileException::Throw(WarFileException::NO_SUPPORT);
	return FALSE; // Compiler food
}

BOOL WarFile::Scan(LPCSTR Pattern, DWORD Flags, WarFileList *pFileList)
{
	WarFileException::Throw(WarFileException::NO_SUPPORT);
	return FALSE; // Compiler food
}

BOOL WarFile::CanRead()
{
	return (IsOpen() && (m_Flags & (t_file | read)) == (t_file | read));
}

BOOL WarFile::CanWrite()
{
	return IsOpen() && ((m_Flags & (t_file | write)) == (t_file | write));
}

BOOL WarFile::CanBrowse()
{
	return FALSE;
}

void WarFile::DoDelete()
{
	WarFileException::Throw(WarFileException::NO_SUPPORT);
}

void WarFile::DoDelete(LPCSTR Path)
{
	WarFileException::Throw(WarFileException::NO_SUPPORT);
}

// Move file. The argument is a pointer to an open file.
void WarFile::DoMove(WarFile *pFile)
{
	WarFileException::Throw(WarFileException::NO_SUPPORT);
}

BOOL WarFile::IsSlow()
{
	return FALSE;
}

void WarFile::SetCallback(WarFileCallback *pCallback)
{
	if (!m_Flags & callback)
		WarFileException::Throw(WarFileException::NOT_CALLBACK);

	if (m_pCallback && (m_pCallback->m_pFile == this))
		m_pCallback->m_pFile = NULL; // Possible memory leak!!!

	m_pCallback = pCallback;
	if (m_pCallback)
		m_pCallback->m_pFile = this;
}


WarString WarFile::GetFileName(NameModes Mode)
{
	LPCSTR p = m_LogicalPath;
	LPCSTR pp = "";
	WarString ws;

	switch(Mode)
	{
		case LogicalPath:
			pp = p;
			break;
		case LogicalNameOnly:
			pp = strrchr(p, '/');
			if (!pp || !*(++pp))
				pp = m_LogicalPath;
			break;
		case LogicalPathOnly:
			ws = p;
			pp = strrchr(ws, '/');
			if (pp)
				ws.ReleaseBuffer(pp - p);
			break;
		default:
			WarFileException::Throw(WarFileException::NO_SUPPORT);
	}

	return pp;
}

////////////////////////////////////////////////////////////////////////
// WarFile - static functions
////////////////////////////////////////////////////////////////////////

WarFile * (*WarFile::m_CreateFunc[invalid])(WarFile& File);
LPCSTR WarFile::m_ProtocolNames[invalid] =
{
	{"none"},
	{"file"},
	{"vfsys"},
	{"cdfs"},
	{"nfs"},
	{"http"},
	{"ftp"},
	{"pop3"},
	{"smtp"},
	{"nntp"}
};

	
BOOL WarFile::Stat(LPCSTR URL, PositionModes Mode, WarFStat *buffer)
{
	return FALSE;
}

BOOL WarFile::Rename(LPCSTR URL, LPCSTR Name)
{
	return FALSE;
}

BOOL WarFile::Move(LPCSTR SrcURL, LPCSTR DstURL)
{
	return FALSE;
}

BOOL WarFile::Copy(LPCSTR ErcURL, LPCSTR DstURL)
{
	return FALSE;
}

BOOL WarFile::UnixDirListing(WarArgs Args)
{
	return FALSE;
}

BOOL WarFile::DosDirListing(WarArgs Args)
{
	return FALSE;
}

BOOL WarFile::Chdir(LPCSTR Path)
{
	return FALSE;
}

////////////////////////////////////////////////////////////////////////
// WarFile - non-static non-ovverides
////////////////////////////////////////////////////////////////////////

WarFile *WarFile::CreateForURL(LPCSTR URL, WarString& Path)
{
	WarFile *pFile = NULL;
	LPCSTR p, pp;

	for(int mode = file; mode < invalid; mode++)
	{
		p = URL, pp = m_ProtocolNames[mode];
		while(*p && *pp)
		{
			if (toupper(*p) != toupper(*pp))
				break;
			++p;
			++pp;
		}

		if (!*pp && (p[0] == ':'))
			break;
	}

	// Check that we have a valid URL
	if (*pp
		|| (mode == invalid)
		|| ((p[0] != ':') && (p[0] != '/') && (p[0] != '/')))
		WarFileException::Throw(WarFileException::INVALID_URL, 0);

	// mode is the URL type
	Path = p + 3;

	switch(mode)
	{
		case file:
			pFile = new WarDosFile(*this, "file");
			break;
		case vfsys:
			pFile = new WarDosFile(*this, "vfsys");
			break;
		default:
			if (m_CreateFunc[mode])
			{
				pFile = m_CreateFunc[mode](*this);
				break;
			}
			WarFileException::Throw(WarFileException::UNSUPPORTED_FSYS, 0);
	}

	return pFile;
}

BOOL WarFile::InstallFs(LPCSTR Name, WarFile * (*CreateFunc)(WarFile& File))
{
	for(int mode = file; mode < invalid; mode++)
	{
		if (!stricmp(m_ProtocolNames[mode], Name))
		{
			m_CreateFunc[mode] = CreateFunc;
			return TRUE;
		}
	}

	return FALSE;
}

//////////////////////////////////////////////////////////////////////////
// WarFileList
//////////////////////////////////////////////////////////////////////////

WarFileList::WarFileList(LPCSTR RootPath)
{
	m_RootPath = RootPath;
	m_bSupressTotal = FALSE;

	BOOL bIsPattern = FALSE;

	// Strip pattern
	if (!m_RootPath.IsEmpty())
	{
		LPCSTR Path = m_RootPath;
		LPCSTR p = Path + (m_RootPath.GetLength() - 1);

		while(p >= Path)
		{
			switch(*p)
			{
				case '*':
				case '?':
					 bIsPattern = TRUE;
					 break;
				case '/':
				case '\\':
					if (bIsPattern)
						m_RootPath.ReleaseBuffer((p - Path) +1);
					p = Path;
					bIsPattern = FALSE;
					break;
			}
			--p;
		}
	}

	if (bIsPattern 
		|| m_RootPath.IsEmpty())
		m_RootPath = ".";
}

WarFileList::~WarFileList()
{
	Clear();
}

void WarFileList::Clear()
{
	WarFStat *p;
	while(p = (WarFStat *)GetAndDeleteFirst())
		delete p;
}

WarLinkedListItem *WarFileList::Add(WarFStat *pFS, WarLinkedListItem *pFather)
{
	pFS->m_pFather = (LPVOID)pFather;
	AddLast((LPVOID)pFS);
	return Last();
}

void WarFileList::AddUnique(WarFStat *pFS)
{
	WarLinkedListItem *pItem;
	for(pItem = First(); pItem; pItem = Next(pItem))
	{
		WarFStat *p = (WarFStat *)Ptr(pItem);
		if (!stricmp(p->FileName(), pFS->FileName()))
		{
			// Replace interesting data.
			p->m_ctime = pFS->m_ctime;
			p->m_mtime = pFS->m_mtime;
			p->m_atime = pFS->m_atime;
			p->m_size = pFS->m_size;
			p->m_Group = pFS->m_Group;
			if (p->m_MoreGroups)
				delete p->m_MoreGroups;
			p->m_MoreGroups = pFS->m_MoreGroups;
			p->m_Links = pFS->m_Links;
			p->m_dev = pFS->m_dev;
			p->m_IsFsAccessNode = pFS->m_IsFsAccessNode;
			p->m_Perms &= ~0777;
			p->m_Perms |= pFS->m_Perms;
			if (p->m_Perms & NODE_READONLY)
				p->m_Perms &= ~0222;
			p->m_Inode = pFS->m_Inode;
			p->m_DLC = pFS->m_DLC;
			p->m_pFather = pFS->m_pFather;
			delete pFS;
			return;
		}
	}

	Add(pFS, (WarLinkedListItem *)pFS->m_pFather);
}

WarString WarFileList::GetFulllPath(WarLinkedListItem *pItem)
{
	// TODO: Add tree support
	return m_RootPath;
}

WarFStat *WarFileList::Find(LPCSTR Name)
{
	WarLinkedListItem *pItem;

	for(pItem = First(); pItem; pItem = Next(pItem))
	{
		WarFStat *p = (WarFStat *)Ptr(pItem);
		if (!stricmp(p->m_Name, Name))
			return p;
	}

	return NULL;
}

 