/* !!Copyright begin */
/*
War Software series
Copyright (C) 1996 - 1998 by Jarle (jgaa) Aase (www.jgaa.com)
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation version 2
of the License - WITH THE FOLLOWING RESTRICTIONS:

  * GOVERNMENTS, GOVERNMENTAL AGENCIES OR COMMERCIAL 
    COMPANIES OWNED 50% OR MORE BY THE GOVERNMENT ARE
    NOT ALLOWED TO USE THE SOFTWARE OR SORCE CODE FOR
    ANY PURPOSE. THIS RESTRICTION ALSO APPLIES FOR
    PROGRAMS DERIVED FROM THE SOURCE CODE, OR PROGRAMS/
    SOURCE CODE THAT USE SPECIAL TECHNICS/ALOGRITHMS
    INVENTED IN THIS SOFTWARE.

  * THE FULL SOURCE CODE IS WITHELD FROM A PUBLIC RELEASE
    UNTIL THE LINUX VERISON IS RELEASED.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
 
/* !!Copyright end */


#include "war.h"
#include "NTNetwork.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif



/*
 * API Interface
 */
usrstat WarUserHandle::m_Errno;

WarUserHandle::WarUserHandle()
{
	m_RPCclnt = NULL;
	m_pSVC = NULL;
}

WarUserHandle::~WarUserHandle()
{
	DestroyRPCClient();
	if (m_pSVC)
		delete m_pSVC;
}

// Connect to the User server
BOOL WarUserHandle::Create(WarSocket *pCaller, LPCSTR Host)
{
#ifdef WARSVR
	if (!Host && UseLocalUserdb)
	{
		m_pSVC = new svc_req; // We have to fake RPC calls
		memset(m_pSVC, 0, sizeof(svc_req));
		m_pSVC->rq_prog = WARUSER_PROGRAM;
		m_pSVC->rq_vers = WARUSER_VERSION;

		// TODO: Make a valid auth entry
		//opaque_auth rq_cred;	/* raw creds from the wire */
		//LPSTR		rq_clntcred;	/* read only cooked cred */
		m_pSVC->rq_xprt = (WarSVCSocket *)pCaller;		/* associated transport */
		m_pSVC->rq_flags |= RQ_FLAG_LOOPBACK;
		return TRUE;
	}
#endif
	return CreateRPCClient(Host);
}

BOOL WarUserHandle::ClientLogin(war_client_login_req &Req, war_user_login_ok &Handle, int &Err)
{
	war_user_client_login_result *pResult = NULL;
	Err = 0;

	if (!Req.m_ID)
		Req.m_ID = "";
	if (!Req.m_Pwd)
		Req.m_Pwd = "";

	m_Errno = USRERR_FAIL;

	if (m_RPCclnt)
		pResult = usrproc_clientlogin_1(&Req, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_clientlogin_1_svc(&Req, m_pSVC);
#endif
	if (!pResult)
		return FALSE;

	if ((m_Errno = pResult->status) != USR_OK)
	{
		if (m_Errno == USRERR_ACCES)
			Err = pResult->war_user_client_login_result_u.m_ReasonID;
		return FALSE;
	}

	Handle =  pResult->war_user_client_login_result_u.m_Data;

	return TRUE;
}

BOOL WarUserHandle::RPCLogin(LPCSTR Name, 
														 LPCSTR Passwd,
														 int *pErr,
														 user_node_type UserType, 
														 war_svr_type ServerLogonType)
{
	war_user_rpclogin_result *pAttr;
	war_rpclogin_req Arg;

	if (pErr)
		*pErr = 0;

	PrpAuthData_100(Arg, Name, Passwd, UserType, ServerLogonType);

	m_Errno = USRERR_FAIL;

	if (m_RPCclnt)
		pAttr = usrproc_rpclogin_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pAttr = usrproc_rpclogin_1_svc(&Arg, m_pSVC);
#endif

	if (!pAttr)
		return FALSE;

	if ((m_Errno = pAttr->status) != USR_OK)
	{
		if (pErr && (m_Errno == USRERR_PERM))
			*pErr = pAttr->war_user_rpclogin_result_u.m_ReasonID;
		return FALSE;
	}

	// Get RPC auth data
	war_auth_def_100 *p = (war_auth_def_100 *)pAttr->war_user_rpclogin_result_u.m_Data;
	if (!UnprpAuthData(p))
		return FALSE;
	if (m_RPCclnt)
	{
		// 'pack' the auth info into the unix data. This a _very_ poor 
		// method, but I don't have the time right now to do a 
		// good implementation.
		// TODO: This must be changed before version 2!!!
		char machname[MAX_MACHINE_NAME + 1];
		int gids[WAR_AUT_DEF_100_GLEN];
		WarAuthConvertToGids(*p, gids);
		if (gethostname(machname, MAX_MACHINE_NAME) == -1)
			return FALSE;
		machname[MAX_MACHINE_NAME] = 0;

		auth_destroy(m_RPCclnt->cl_auth);
		m_RPCclnt->cl_auth = authunix_create(machname, geteuid(), getegid(), 
			WAR_AUT_DEF_100_GLEN, gids);
	}

	return TRUE;
}

void WarUserHandle::RPCLogout()
{
	static struct timeval TIMEOUT = { 0, 0 };
	if (m_RPCclnt)
	{
		// We don't want to stall on a dead server - so we send the goodbye
		// message and continues. If the message don't make it to the server,
		// the connection will time-out anyway.
		char clnt_res;	
		void *argp = NULL;
		memset((char *)&clnt_res, 0, sizeof(clnt_res));
		clnt_call(m_RPCclnt, USERPROC_RPCLOGOUT, (xdrproc_t) xdr_void, argp, (xdrproc_t) xdr_void, &clnt_res, TIMEOUT);
	}
}

// Get information about a user
BOOL WarUserHandle::GetAttr(user User, user_attrs& Attr)
{
	user_attrs_result *pAttr;
	user Arg;
	Arg = User;

	m_Errno = USRERR_FAIL;

	if (m_RPCclnt)
		pAttr = usrproc_getattr_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pAttr = usrproc_getattr_1_svc(&Arg, m_pSVC);
#endif
	if (!pAttr)
		return FALSE;

	if ((m_Errno = pAttr->status) != USR_OK)
		return FALSE;

	Attr = pAttr->user_attrs_result_u.m_Data;
	return TRUE;
}

BOOL WarUserHandle::VerifyFileOpenAccess(user User, 
			QWORD SesID,
			LPCSTR URL, 
			LPCSTR Path, 
			LPCSTR Name,
			FLEN Size,
			DWORD WarFileFlags,
			BOOL bCheckBannedList, 
			BOOL bFileExist)
			
{
	user_generic_result *pAttr;
	user_verify_file_open_access_req Arg;
	
	Arg.m_User = User;
	Arg.m_SesID = SesID;
	Arg.m_URL = (RPC_string)(URL ? URL : "");
	Arg.m_Path = (RPC_string)(Path ? Path : "");
	Arg.m_Name = (RPC_string)(Name ? Name : "");
	Arg.m_Size = Size;
	Arg.m_WarFileFlags = WarFileFlags;
	Arg.m_bFileExist = bFileExist;
	Arg.m_bCheckBannedList = bCheckBannedList;
	Arg.m_reserved1 = 0;
	Arg.m_reserved2 = 0;

	m_Errno = USRERR_FAIL;

	if (m_RPCclnt)
		pAttr = usrproc_confirmfileopenaccess_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pAttr = usrproc_confirmfileopenaccess_1_svc(&Arg, m_pSVC);
#endif
	if (!pAttr)
		return FALSE;

	if ((m_Errno = pAttr->status) != USR_OK)
		return FALSE;

	m_Errno = (usrstat)pAttr->user_generic_result_u.m_Status;
	return TRUE;
}

BOOL WarUserHandle::CreateNew(user_create_req& Arg, user& Node)
{
	user_create_result *pResult = NULL;
	m_Errno = USRERR_FAIL;

	if (m_RPCclnt)
		pResult = usrproc_createnew_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_createnew_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	Node = pResult->user_create_result_u.m_User;
	return TRUE;
}

BOOL WarUserHandle::DeleteNode(user Node)
{
	user_generic_result *pResult = NULL;
	warusr_delete_req Arg;
	Arg.m_User = Node;

	if (m_RPCclnt)
		pResult = usrproc_delete_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_delete_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	return TRUE;
}

BOOL WarUserHandle::SetPwdMode(user User, user_pwd_type Mode)
{
	user_generic_result *pResult = NULL;
	user_generic_set_int Arg;
	Arg.m_User = User;
	Arg.m_Val = (int)Mode;

	if (m_RPCclnt)
		pResult = usrproc_setpwdmode_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_setpwdmode_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	return TRUE;
}

BOOL WarUserHandle::SetStat(user User, wdq_mode ReqMode,
		user_stat_update UpdateMode, user_stat& Stat)
{
	user_query_result *pResult = NULL;
	user_stat_req Arg;
	Arg.m_User = User;
	Arg.m_UpdateMode = UpdateMode;
	Arg.m_DataMode = ReqMode;
	Arg.m_Stat = Stat;

	if (m_RPCclnt)
		pResult = usrproc_setstat_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_setstat_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	Stat = *pResult->user_query_result_u.m_Data.wdq_result_u.m_pFTPStat;

	return TRUE;
}

BOOL WarUserHandle::SetExpiry(user User, 
		user_expiery_type Mode,
		user_expiery_method Method, 
		WarTime& Date, // Can be ex. date or when ex. countdown was activated
		long ExpireCnt,
		long RelativeDays,
		int FlagsMask,
		int Flags)
{
	user_generic_result *pResult = NULL;
	user_expiery_req Arg;
	Arg.m_User = User;
	Arg.m_Mode = Mode;
	Arg.m_Method = Method;
	Arg.m_Date = Date;
	Arg.m_ExpireCnt = ExpireCnt;
	Arg.m_RelativeDays = RelativeDays;
	Arg.m_FlagsMask = FlagsMask;
	Arg.m_Flags = Flags;

	if (m_RPCclnt)
		pResult = usrproc_setexpiery_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_setexpiery_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	return TRUE;
}

BOOL WarUserHandle::SetPrivMode(user User, user_priv_type Mode)
{
	user_generic_result *pResult = NULL;
	user_generic_set_int Arg;
	Arg.m_User = User;
	Arg.m_Val = (int)Mode;

	if (m_RPCclnt)
		pResult = usrproc_setptype_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_setptype_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	return TRUE;
}

BOOL WarUserHandle::SetUserSubclass(user User, user_subclass_data& Subclass)
{
	user_generic_result *pResult = NULL;
	user_subclass_req Arg;
	Arg.m_User = User;
	Arg.m_Data = Subclass;

	if (m_RPCclnt)
		pResult = usrproc_setusubclass_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_setusubclass_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	return TRUE;
}

BOOL WarUserHandle::SetUserGreeting(user User,
		LPCSTR Message,
		BOOL bDelWhenShown,
		time_t ExpiryDate /*0 to disable */)
{
	user_generic_result *pResult = NULL;
	user_greeting_req Arg;
	Arg.m_User = User;
	Arg.m_Expire = ExpiryDate;
	Arg.m_bDelete = bDelWhenShown;
	Arg.m_Message = (char *)(Message ? Message : "");

	if (m_RPCclnt)
		pResult = usrproc_setugreeting_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_setugreeting_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	return TRUE;
}

BOOL WarUserHandle::SetUserInfo(user User, const user_info& Info, int Mask)
{
	user_generic_result *pResult = NULL;
	user_info_req Arg;
	Arg.m_User = User;
	Arg.m_Mask = Mask;
	Arg.m_Data = Info;

	if (Arg.m_Data.m_RealName == NULL)
		Arg.m_Data.m_RealName = (RPC_string)"";

	if (Arg.m_Data.m_Email == NULL)
		Arg.m_Data.m_Email = (RPC_string)"";

	if (Arg.m_Data.m_Http == NULL)
		Arg.m_Data.m_Http = (RPC_string)"";

	if (Arg.m_Data.m_RealName == NULL)
		Arg.m_Data.m_RealName = (RPC_string)"";

	if (Arg.m_Data.m_Ftp == NULL)
		Arg.m_Data.m_Ftp = (RPC_string)"";

	if (Arg.m_Data.m_ICQ == NULL)
		Arg.m_Data.m_ICQ = (RPC_string)"";

	if (Arg.m_Data.m_IRCnick == NULL)
		Arg.m_Data.m_IRCnick = (RPC_string)"";

	if (Arg.m_Data.m_Memo == NULL)
		Arg.m_Data.m_Memo = (RPC_string)"";

	if (m_RPCclnt)
		pResult = usrproc_setuinfo_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_setuinfo_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	return TRUE;
}

BOOL WarUserHandle::QueryPath(user User, LPCSTR Path, 
		user_get_user_URL_query_mode Mode,
		WarString& EffectiveURL, WarString& EffectivePath,
		DWORD& EffectiveFlags, FLEN& Quota)
{
	user_get_user_URL_result *pResult = NULL;
	user_get_user_URL_req Arg;
	Arg.m_User = User;
	Arg.m_Path = (char *)((Path && *Path) ? Path : "");
	Arg.m_Mode = Mode;

	if (m_RPCclnt)
		pResult = usrproc_querypath_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_querypath_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	EffectiveURL = pResult->user_get_user_URL_result_u.m_FileAccess.m_Path;
	EffectivePath = pResult->user_get_user_URL_result_u.m_FileAccess.m_Alias;
	EffectiveFlags = pResult->user_get_user_URL_result_u.m_FileAccess.m_Flags;
	Quota = pResult->user_get_user_URL_result_u.m_FileAccess.m_Quota;

	return TRUE;
}

BOOL WarUserHandle::SetServerInfo(user User, war_daemon_info &Info)
{
	user_generic_result *pResult = NULL;
	war_daemon_info_req Arg;
	Arg.m_User = User;
	Arg.m_Data = Info;
	if (Arg.m_Data.m_HostName == NULL)
		Arg.m_Data.m_HostName = "";

	if (m_RPCclnt)
		pResult = usrproc_set_daemon_info_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_set_daemon_info_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	return TRUE;
}

BOOL WarUserHandle::SetFSDup(user User, wdq_mode ReqMode, fs_dup_mode Mode)
{
	user_generic_result *pResult = NULL;
	fs_dup_req Arg;
	Arg.m_User = User;
	Arg.m_Data.m_Mode = Mode;
	Arg.m_DataMode = ReqMode;

	if (m_RPCclnt)
		pResult = usrproc_setfsdup_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_setfsdup_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	return TRUE;
}

BOOL WarUserHandle::SetIPAccessList(user User, 
		ip_restrictions_mode Mode,
		LPCSTR AllowList,
		LPCSTR DenyList,
		BOOL bModeIsValid,
		BOOL bAllowListIsValid,
		BOOL bDenyListIsValid,
		ip_access_list **ppAccessList)
{
	return GenericSetList(User, Mode, AllowList, DenyList, 
		bModeIsValid, bAllowListIsValid, bDenyListIsValid, 
		ppAccessList, WDQ_IP_ACCESS_LIST);
}

BOOL WarUserHandle::SetFTPBannedFiles(user User, 
		ip_restrictions_mode Mode,
		LPCSTR AllowList,
		LPCSTR DenyList,
		BOOL bModeIsValid,
		BOOL bAllowListIsValid,
		BOOL bDenyListIsValid,
		fs_banned_files_list **ppAccessList)
{
	return GenericSetList(User, Mode, AllowList, DenyList, 
		bModeIsValid, bAllowListIsValid, bDenyListIsValid, 
		ppAccessList, WDO_FTP_BANNED_FILES);
}

BOOL WarUserHandle::GenericSetList(user User, 
		ip_restrictions_mode Mode,
		LPCSTR AllowList,
		LPCSTR DenyList,
		BOOL bModeIsValid,
		BOOL bAllowListIsValid,
		BOOL bDenyListIsValid,
		ip_access_list **ppAccessList,
		wdq_mode DataMode)
{
	user_query_result *pResult = NULL;
	generic_list_req Arg;
	Arg.m_User = User;
	Arg.m_DataMode = DataMode;

	if (Arg.m_ModeIsValid = bModeIsValid)
		Arg.m_Data.m_Mode = Mode;
	
	if (Arg.m_AllowListIsValid = bAllowListIsValid)
		Arg.m_Data.m_AllowList = (RPC_string)(AllowList ? AllowList : "");
	else
		Arg.m_Data.m_AllowList = (RPC_string)"";

	if (Arg.m_DenyListIsValid = bDenyListIsValid)
		Arg.m_Data.m_DenyList = (RPC_string)(DenyList ? DenyList : "");
	else
		Arg.m_Data.m_DenyList = (RPC_string)"";

	if (m_RPCclnt)
		pResult = usrproc_genericsetlist_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_genericsetlist_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	if (ppAccessList)
		*ppAccessList = pResult->user_query_result_u.m_Data.wdq_result_u.m_pIpAccessList;

	return TRUE;
}

BOOL WarUserHandle::SetLoginTimeMask(user User,
		wdq_mode ReqMode, /* Normally WDO_FTP_LOGIN_TIMEM */
		user_login_perms_mode Mode,
		long DayMask,
		long SaturdayMask,
		long SundayMask,
		user_login_time_result **ppNewEffective)
{
	user_query_result *pResult = NULL;
	user_login_time_req Arg;
	Arg.m_User = User;
	Arg.m_Option = ReqMode;
	Arg.m_Data.m_DayMask = DayMask;
	Arg.m_Data.m_SaturdayMask = SaturdayMask;
	Arg.m_Data.m_SundayMask = SundayMask;
	Arg.m_Data.m_Mode = Mode;
	
	if (m_RPCclnt)
		pResult = usrproc_setlogintmask_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_setlogintmask_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	if (ppNewEffective)
		*ppNewEffective = pResult->user_query_result_u.m_Data.wdq_result_u.m_pLoginTimeMask;

	return TRUE;
}


WarOption *WarUserHandle::SesMgrGetOption(LPCSTR Name, int Index)
{
	sesmgr_option_result *pResult = NULL;
	sesmgr_option_req Arg;
	
	Arg.m_Index = Index;
	Arg.m_Name = (RPC_string)(Name ? Name : "");

	if (m_RPCclnt)
		pResult = usrproc_sesmgrgetoption_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_sesmgrgetoption_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return NULL;

	return &pResult->sesmgr_option_result_u.m_Data;
}

svrmgr_sysmsg *WarUserHandle::SesMgrGetSysMsgs(int &NumReturned)
{
	NumReturned = 0;
	sesmgr_query_sysmsg_result *pResult = NULL;
	char Arg = 0;

	if (m_RPCclnt)
		pResult = usrproc_sesmgrquerysysmsgs_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_sesmgrquerysysmsgs_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return NULL;

	NumReturned = pResult->sesmgr_query_sysmsg_result_u.m_Data.m_Data_len;
	return pResult->sesmgr_query_sysmsg_result_u.m_Data.m_Data_val;
}

sesmgr_ud_restrictions_data *WarUserHandle::SesMgrGetCredits(QSWORD SessionKey)
{
	hyper Arg = SessionKey;
	sesmgr_ud_restrictions_data_result *pResult = NULL;

	if (m_RPCclnt)
		pResult = usrproc_sesmgrquerycredits_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_sesmgrquerycredits_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return NULL;

	return &pResult->sesmgr_ud_restrictions_data_result_u.m_Data;
}

BOOL WarUserHandle::SesMgrGetSysMsg(LPCSTR FileName, WarString& Text)
{
	Text = "";
	sesmgr_get_sysmsg_result *pResult = NULL;
	RPC_string Arg = (RPC_string)(FileName ? FileName : "");
	
	if (m_RPCclnt)
		pResult = usrproc_sesmgrgetsysmsg_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_sesmgrgetsysmsg_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return NULL;

	Text = pResult->sesmgr_get_sysmsg_result_u.m_Data.m_Text;

	return TRUE;
}

BOOL WarUserHandle::SesMgrSetSysMsg(LPCSTR FileName, LPCSTR Text)
{
	user_generic_result *pResult = NULL;
	svr_sysmsg_req Arg;
	
	Arg.m_MessageFileName = (RPC_string)(FileName ? FileName : "");
	Arg.m_Text = (RPC_string)(Text ? Text : "");

	if (m_RPCclnt)
		pResult = usrproc_sesmgrsetsysmsg_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_sesmgrsetsysmsg_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return NULL;

	return TRUE;
}

BOOL WarUserHandle::SesMgrSaveUserDB()
{
	user_generic_result *pResult = NULL;
	char Arg = 0;

	if (m_RPCclnt)
		pResult = usrproc_saveuserdb_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_saveuserdb_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return NULL;

	return TRUE;
}

// Return -1 on error
int WarUserHandle::SesMgrQueryOnline(sesmgr_qyery_online_mode Mode)
{
	user_generic_result *pResult = NULL;
	sesmgr_qyery_online_mode Arg = Mode;

	if (m_RPCclnt)
		pResult = usrproc_saveuserdb_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_saveuserdb_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return -1;

	return pResult->user_generic_result_u.m_Status;
}

svrmgr_macro *WarUserHandle::SesMgrGetMacros(int& NumReturned)
{
	sesmgr_query_macro_result *pResult = NULL;
	char Arg = 0;

	if (m_RPCclnt)
		pResult = usrproc_sesmgrquerymacros_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_sesmgrquerymacros_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return NULL;

	NumReturned = pResult->sesmgr_query_macro_result_u.m_Data.m_Data_len;
	return pResult->sesmgr_query_macro_result_u.m_Data.m_Data_val;
}


BOOL WarUserHandle::SesMgrSetOption(LPCSTR Name, int Index, LPCSTR NewValue)
{	
	user_generic_result *pResult = NULL;
	sesmgr_option_set Arg;
	
	Arg.m_Index = Index;
	Arg.m_Name = (RPC_string)(Name ? Name : "");
	Arg.m_NewValue = (RPC_string)(NewValue ? NewValue : "");

	if (m_RPCclnt)
		pResult = usrproc_sesmgrsetoption_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_sesmgrsetoption_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	return TRUE;
}

sesmgr_file_info_reply *WarUserHandle::SesMgrQueryFiles(sesmgr_file_info_req& Arg)
{
	sesmgr_file_info_result *pResult = NULL;

	if (m_RPCclnt)
		pResult = usrproc_sesmgrqueryfileinfo_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_sesmgrqueryfileinfo_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return NULL;

	return &pResult->sesmgr_file_info_result_u.m_Data;
}

sesmgr_query_domain_reply *WarUserHandle::GetDomainInfo(int Index)
{
	sesmgr_query_doiain_result *pResult = NULL;
	sesmgr_query_domaun_req Arg;

	Arg.m_DomainIndex = Index;

	if (m_RPCclnt)
		pResult = usrproc_sesmgrquerydomain_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_sesmgrquerydomain_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return NULL;

	return &pResult->sesmgr_query_doiain_result_u.m_Data;
}

sesmgr_query_system_info_reply *WarUserHandle::GetSystemInfo()
{
	sesmgr_query_system_info_result *pResult = NULL;
	char Arg;

	if (m_RPCclnt)
		pResult = usrproc_sesmgrgetsysinfo_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_sesmgrgetsysinfo_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return NULL;

	return &pResult->sesmgr_query_system_info_result_u.m_Data;
}

sesmgr_query_system_current_reply *WarUserHandle::GetSystemStats()
{
	sesmgr_query_system_current_result *pResult = NULL;
	char Arg;

	if (m_RPCclnt)
		pResult = usrproc_sesmggetcurrinfo_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_sesmggetcurrinfo_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return NULL;

	return &pResult->sesmgr_query_system_current_result_u.m_Data;
}

sesmgr_query_usersonline_reply *WarUserHandle::GetUsersOnline(int StartIndex) // Index start at 0
{
	sesmgr_query_usersonline_result *pResult = NULL;
	sesmgr_query_usersonline_req Arg;

	Arg.m_NextIndex = StartIndex;

	if (m_RPCclnt)
		pResult = usrproc_sesmgrqueryuseronline_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_sesmgrqueryuseronline_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return NULL;

	return &pResult->sesmgr_query_usersonline_result_u.m_Data;
}

sesmgr_query_host_reply *WarUserHandle::GetHostsOnline(int StartIndex) // Index start at 0
{
	sesmgr_query_host_result *pResult = NULL;
	sesmgr_query_hostonline_req Arg;

	Arg.m_NextIndex = StartIndex;

	if (m_RPCclnt)
		pResult = usrproc_sesmgrqueryhostonline_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_sesmgrqueryhostonline_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return NULL;

	return &pResult->sesmgr_query_host_result_u.m_Data;
}

// Change the password for the user account the current connection use
// Used to allow RPC users to change theirs password in a controlled
// manner.
BOOL WarUserHandle::ChangeSessionPwd(LPCSTR OldPwd, LPCSTR NewPwd)
{
	user_generic_result *pResult = NULL;
	warusr_setpwd_req Arg;
	
	Arg.m_OldPwd = (RPC_string)(OldPwd ? OldPwd : "");
	Arg.m_NewPwd = (RPC_string)(NewPwd ? NewPwd : "");

	if (m_RPCclnt)
		pResult = usrproc_sesmgrsetconnpwd_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_sesmgrsetconnpwd_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	return TRUE;
}

/*
 * Set/clear flags in the session manager. Depending on the mode,
 * flags can be set on connections, users, servers (domains) or
 * globally.
 * This call is primarilly used to shut down the server, or to abort
 * transfers or throw users off.
 * NB: There is no validiation of the flags set, except that the user
 * that call the fuction must be a sysadmin.
 *
 * The new effective flags can be obtained grom GetLastError() if the call succeeds.
 * Call this function with a 0 mask to query the current flags only.
 *
 * Parameters (mode)
 *		SSFM_USER			- user (userID - set on all connections from this user)
 *		SSFM_SERVER		- user (ServerNode - set on all connections to this server and the server itself)
 *									- war_svr_type
 *		SSFM_SESSION	- hyper (session ID - set on this session only)
 *		SSFM_ADDR			- in_addr (client IP - set on all connections from this IP)
 *		SSFM_GLOBAL		- void (set on all connections and servers)
 *
 * If SMF_ModeSetOnSessions bit is set in the Flag parameter, the flags are
 * set on the affetced users currently online, rather than the shared object. This
 * is how the kick flag can be set on a host or a server. If such flags was
 * set on a hosts or server object itself, all future logins would fail.
 *
 */
BOOL WarUserHandle::SesMgrSetFlag(DWORD Mask, DWORD Flag, 
		sesmgr_set_flag_mode Mode, ...)
{	
	va_list ap;
	va_start(ap, Mode);

	user_generic_result *pResult = NULL;
	smgt_set_flag_req Arg;

	Arg.m_Mask = Mask;
	Arg.m_Flag = Flag;
	switch(Arg.m_Where.Mode = Mode)
	{
		case SSFM_USER:
			Arg.m_Where.sesmgr_setflag_data_u.m_ClientUser =  va_arg(ap, user);
			break;
		case SSFM_SERVER:
			Arg.m_Where.sesmgr_setflag_data_u.m_Server.m_ServerNode = va_arg(ap, user);
			Arg.m_Where.sesmgr_setflag_data_u.m_Server.m_Type = va_arg(ap, war_svr_type);
			break;
		case SSFM_SESSION:
			Arg.m_Where.sesmgr_setflag_data_u.m_SessionID = va_arg(ap, hyper);
			break;
		case SSFM_ADDR:
			Arg.m_Where.sesmgr_setflag_data_u.m_ClientAddr.s_addr = va_arg(ap, in_addr).s_addr;
			break;
		case SSFM_GLOBAL:
			break;
		default:
			ASSERT(FALSE);
			m_Errno = USRERR_INVAL;
			va_end(ap);
			return FALSE;
	}

	va_end(ap);

	if (m_RPCclnt)
		pResult = usrproc_sesmgrsetflag_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_sesmgrsetflag_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	// Set errno to return value
	m_Errno = (usrstat)pResult->user_generic_result_u.m_Status;

	return TRUE;
}

const sesmgr_query_session_list *WarUserHandle::SesMgrQuerySessions(sesmgr_query_session_req& Arg)
{
	sesmgr_query_session_result *pResult = NULL;

	if (m_RPCclnt)
		pResult = usrproc_sesmgrquery_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_sesmgrquery_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return NULL;
	
	return &pResult->sesmgr_query_session_result_u.m_Data;
}

const sesmgr_query_session_list *WarUserHandle::SesMgrQuerySessions(int NextIndex)
{
	sesmgr_query_session_result *pResult = NULL;

	if (m_RPCclnt)
		pResult = usrproc_sesmgrquerymore_1(&NextIndex, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_sesmgrquerymore_1_svc(&NextIndex, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return NULL;
	
	return &pResult->sesmgr_query_session_result_u.m_Data;
}

sesmgr_log_event *WarUserHandle::SesMgrQueryLog(QSWORD EventID, DWORD Mask)
{
	sesmgr_query_log_result *pResult = NULL;
	sesmgr_query_log_event_req Arg;
	
	Arg.m_Event = EventID;
	Arg.m_Mask = Mask;

	if (m_RPCclnt)
		pResult = usrproc_querylog_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_querylog_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return NULL;

	return &pResult->sesmgr_query_log_result_u.m_Data;
}

cdfs_unit_list *WarUserHandle::SesMgrQueryCdFsData()
{
	sesmgr_cdfs_unit_result *pResult = NULL;
	char Arg;
	
	if (m_RPCclnt)
		pResult = usrproc_sesmgrquerycdfsdata_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_sesmgrquerycdfsdata_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return NULL;

	return &pResult->sesmgr_cdfs_unit_result_u.m_Data;
}

BOOL WarUserHandle::SesMgrCngCdFsData(cdfs_chg_mode Mode, ...)
{
	user_generic_result *pResult = NULL;
	sesmgr_cdfs_change_req Arg;
	memset(&Arg,0,sizeof(Arg));

	va_list ap;
	va_start(ap, Mode);

#define A(name, type, native)\
	Arg.sesmgr_cdfs_change_req_u.m_Unit.m_UnitData.name = (native)va_arg(ap, type);\
	if (Arg.sesmgr_cdfs_change_req_u.m_Unit.m_UnitData.name == NULL)\
		Arg.sesmgr_cdfs_change_req_u.m_Unit.m_UnitData.name = (native)"";

#define B(name, type, native)\
	Arg.sesmgr_cdfs_change_req_u.m_Device.m_DeviceData.name = (native)va_arg(ap, type);\
	if (Arg.sesmgr_cdfs_change_req_u.m_Device.m_DeviceData.name == NULL)\
		Arg.sesmgr_cdfs_change_req_u.m_Device.m_DeviceData.name = (native)"";

	switch(Arg.mode = Mode)
	{
		case CDFSM_DELUNIT:
			Arg.sesmgr_cdfs_change_req_u.m_UnitName = (RPC_string)va_arg(ap, LPCSTR);
			break;
		case CDFSM_CHGUNIT:
			Arg.sesmgr_cdfs_change_req_u.m_Unit.m_UnitName = (RPC_string)va_arg(ap, LPCSTR);
			// Fall trough
		case CFDSM_ADDUNIT:
			if (!Arg.sesmgr_cdfs_change_req_u.m_Unit.m_UnitName)
				Arg.sesmgr_cdfs_change_req_u.m_Unit.m_UnitName = "";
			A(m_Name, LPCSTR, RPC_string);
			A(m_CacheDir, LPCSTR, RPC_string);
			A(m_MaxSimAccess, int, int);
			A(m_CacheTimeout, int, int);
			A(m_LRUnum, int, int);
			A(m_Flags, LPCSTR, DWORD);
			break;
		case CDFSM_CHGDEV:
			Arg.sesmgr_cdfs_change_req_u.m_Device.m_UnitName = (RPC_string)"";
			Arg.sesmgr_cdfs_change_req_u.m_Device.m_DeviceName = (RPC_string)va_arg(ap, LPCSTR);
			B(m_Name, LPCSTR, RPC_string);
			B(m_PhysPath, LPCSTR, RPC_string);
			B(m_Label, LPCSTR, RPC_string);
			B(m_DirMessageFile, LPCSTR, RPC_string);
			B(m_Flags, LPCSTR, DWORD);
			break;
		case CDFSM_ADDDEV:
			Arg.sesmgr_cdfs_change_req_u.m_Device.m_UnitName = (RPC_string)va_arg(ap, LPCSTR);			
			Arg.sesmgr_cdfs_change_req_u.m_Device.m_DeviceName = (RPC_string)"";
			B(m_Name, LPCSTR, RPC_string);
			B(m_PhysPath, LPCSTR, RPC_string);
			B(m_Label, LPCSTR, RPC_string);
			B(m_DirMessageFile, LPCSTR, RPC_string);
			B(m_Flags, LPCSTR, DWORD);
			break;
		case CDFSM_DELDEV:
		case CDFSM_RESCAN:
		case CDFSM_INIT:
			Arg.sesmgr_cdfs_change_req_u.m_DeviceName = (RPC_string)va_arg(ap, LPCSTR);
			break;
		default:
			m_Errno = USRERR_INVAL;
			return FALSE;
	}

#undef A
#undef B
	va_end(ap);
	
	if (m_RPCclnt)
		pResult = usrproc_sesmgrcngcdfsdata_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_sesmgrcngcdfsdata_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	return TRUE;
}

// Used to update session data and user database after a successful transfer.
// must NOT be used from plugins or RPC sessions!
BOOL WarUserHandle::SesMgrPostTransfer(hyper ID, BOOL IsTmp, BOOL IsFree,
	BOOL IsSending, LPCSTR URL, 
	LPCSTR FsysPath, LPCSTR UserPath, hyper Bytes,
	int TransferSeconds, unsigned int CPS, int FTPType,
	 int& ReturnStatus)
{
	user_generic_result *pResult = NULL;
	sesmgr_transfer_done_req Arg;
	ReturnStatus = -1; // Error
	
	Arg.m_ID = ID;
	Arg.m_bIsTmp = IsTmp;
	Arg.m_bIsFree = IsFree;
	Arg.m_bIsSending = IsSending;
	Arg.m_URL = (RPC_string)(LPCSTR)(URL ? URL : "");
	Arg.m_FsysPath = (RPC_string)(LPCSTR)(FsysPath ? FsysPath : "");
	Arg.m_FileBytes = Bytes;
	Arg.m_TransferSeconds = TransferSeconds;
	Arg.m_CPS = CPS;
	Arg.m_UserPath = (RPC_string)(LPCSTR)(UserPath ? UserPath : "");
	Arg.m_FTPtype = FTPType;

	if (m_RPCclnt)
		pResult = usrproc_sesmgrposttransfer_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_sesmgrposttransfer_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	ReturnStatus = (int)pResult->user_generic_result_u.m_Status;
	
	return TRUE;
}

/*
 * This call is used to tell the session manager that a session
 * is active, and to query the manger for special actions, like
 * shutdown, chat, abort etc. If a session not receive a helo
 * in 5 minutes, the manager assumes that the connection is
 * broken, and removes the session from the current session list.
 * FTP server connections and user connections need to send the
 * helo message frequently. RPC connections will also need to send
 * helo messages to avoid the timeout.
 *
 * The result is returned in the Flags argument
 *
 * Note: The SesMgrSet() call returns a call to SesMgrHelo(), so
 * if a client have something to update, it don't have to call
 * SesMgrHelo() in addition to the update call.
 */

BOOL WarUserHandle::SesMgrHelo(QWORD SessionKey, DWORD& Flags)
{
	user_generic_result *pResult = NULL;
	hyper Arg;
	Flags = 0;
	
	Arg = SessionKey;

	if (m_RPCclnt)
		pResult = usrproc_sesmgrhelo_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_sesmgrhelo_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	Flags = (DWORD)pResult->user_generic_result_u.m_Status;
	
	return TRUE;
}

BOOL WarUserHandle::QueryHost(in_addr& Host)
{
	sesmgr_query_host_req Arg;
	user_generic_result *pResult = NULL;
	
	Arg.m_Host = Host;

	if (m_RPCclnt)
		pResult = usrproc_sesmgrqueryhost_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_sesmgrqueryhost_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	return TRUE;
}

/* Used to update the session manager about a client's status.
	Args:
		SDT_LoginName		- LPCSTR LoginName (visual name, listed in the manager)
		SDT_CWD					- LPCSTR CWD
		SDT_Info				- LPCSTR InfoString
		SDT_CurrentFileInfo - LPCSTR Path,
													LPCSTR URL,
													FLEN FileLength,
													FLEN CurPos,
													int Progress
													int CPS
													int CopyProgress,
		SDT_Progress				- FLEN FileLength,
													FLEN CurPos,
													int Progress
													int CPS
													int CopyProgress,
		SDT_State				- int State
		SDT_Progress		- int Progress
		SDT_CPS					- int CPS
		SDT_IdleTime		- void
		SDT_LastCMD			- LPCSTR LastCommand
											BOOL Reset idle time

 * If OK, GetLastError() will return the flags returned by SesMgrHelo()
 */

BOOL WarUserHandle::SesMgrSet(QWORD SessionKey, 
	QSWORD NewIdleTime, 
	sesmgr_sdt Mode, ...)
{
	user_generic_result *pResult = NULL;
	sesmgr_set_data_req Arg;
	sesmgt_set_file_info FI;
	memset(&FI,0,sizeof(FI));
	sesmgt_set_lastcmd_info LCI;

	va_list ap;
	va_start(ap, Mode);

	Arg.m_ID = SessionKey;
	Arg.m_NewIdleTime = NewIdleTime;

	switch(Arg.m_Data.mode = Mode)
	{
		case SDT_LoginName:
		case SDT_CWD:
		case SDT_Info:
			Arg.m_Data.sesmgr_set_data_u.m_Str = (RPC_string) va_arg(ap, LPCSTR);
			if (Arg.m_Data.sesmgr_set_data_u.m_Str == NULL)
				Arg.m_Data.sesmgr_set_data_u.m_Str = "";
			break;
		case SDT_CurrentFileInfo:
			Arg.m_Data.sesmgr_set_data_u.m_pFileInfo = &FI;
			FI.m_Path = (RPC_string) va_arg(ap, LPCSTR); 
			if (!FI.m_Path) 
				FI.m_Path = "";
			FI.m_URL = (RPC_string) va_arg(ap, LPCSTR); 
			if (!FI.m_URL) 
				FI.m_URL = "";
			goto progress;
		case SDT_CurrentFileProgress:
			Arg.m_Data.sesmgr_set_data_u.m_pProgress = &FI.m_Progress;
progress:
			FI.m_Progress.m_FileLength = va_arg(ap, FLEN);
			FI.m_Progress.m_CurPos = va_arg(ap, FLEN);
			FI.m_Progress.m_Progress = va_arg(ap, int);
			FI.m_Progress.m_CPS = va_arg(ap, int);
			FI.m_Progress.m_CopyProgress = va_arg(ap, int);
			break;
		case SDT_CPS:
		case SDT_State:
		case SDT_Progress:
			Arg.m_Data.sesmgr_set_data_u.m_iVal = va_arg(ap, int);
			break;
		case SDT_LastCMD:
			LCI.m_CMD = (RPC_string) va_arg(ap, LPCSTR);
			LCI.m_IdleTime = va_arg(ap, BOOL);
			Arg.m_Data.sesmgr_set_data_u.m_pLastCmdInfo = &LCI;
			break;
		case SDT_IdleTime:
		case SDT_Logout:
			// Void parameter
			break;
	}
	
	va_end(ap);

	if (m_RPCclnt)
		pResult = usrproc_sesmgrupdate_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_sesmgrupdate_1_svc(&Arg, m_pSVC);
#endif
	
	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	// Set errno to return value
	m_Errno = (usrstat)pResult->user_generic_result_u.m_Status;

	return TRUE;
}

BOOL WarUserHandle::SetLoginPerm(user User, 
									wdq_mode Where, 
									long Data, 
									user_login_perms_mode Mode)
{
	user_generic_result *pResult = NULL;
	m_Errno = USRERR_FAIL;
	login_perms_req Arg;
	Arg.m_User = User;
	Arg.m_Option = Where;
	Arg.m_Data.m_Data = Data;
	Arg.m_Data.m_Mode = Mode;

	if (m_RPCclnt)
		pResult = usrproc_setloginperms_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_setloginperms_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	return TRUE;
}

BOOL WarUserHandle::ResynchUser(user User)
{
	user_generic_result *pResult = NULL;
	m_Errno = USRERR_FAIL;
	user_generic_user_req Arg;
	Arg.m_User = User;
	Arg.m_Flags = 0; // Reserved

	if (m_RPCclnt)
		pResult = usrproc_resynchuser_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_resynchuser_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	return TRUE;
}

BOOL WarUserHandle::SetFTPUDR(user User, const ud_restrictions& UDR)
{
	user_generic_result *pResult = NULL;
	m_Errno = USRERR_FAIL;
	ud_restrictions_req Arg;
	Arg.m_User = User;
	Arg.m_Data = UDR;

	if (m_RPCclnt)
		pResult = usrproc_setftpudr_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_setftpudr_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	return TRUE;
}

BOOL WarUserHandle::SetAccess(user User, user_access_type Mode, user_login_access Access)
{
	user_generic_result *pResult = NULL;
	m_Errno = USRERR_FAIL;
	user_generic_set_int Arg;
	Arg.m_User = User;
	Arg.m_Val = (int)Access;
	Arg.m_OptClass = (int)Mode;

	if (m_RPCclnt)
		pResult = usrproc_setaccess_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_setaccess_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	return TRUE;
}

BOOL WarUserHandle::ReadBuffer(int Handle, int From, int Bytes, LPVOID *ppBuffer, int& BytesRead, BOOL& IsEOF)
{
	warsvr_read_buffer_result *pResult = NULL;
	m_Errno = USRERR_FAIL;
	warsvr_read_buffer_req Arg;
	Arg.m_Handle = Handle;
	Arg.m_ReadFrom = From;
	Arg.m_ReadBytes = Bytes;

	if (m_RPCclnt)
		pResult = usrproc_readbuffer_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_readbuffer_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	ASSERT(ppBuffer != NULL);

	BytesRead = pResult->warsvr_read_buffer_result_u.m_Data.m_BytesRead;
	IsEOF = pResult->warsvr_read_buffer_result_u.m_Data.m_IsEOF;
	*ppBuffer = pResult->warsvr_read_buffer_result_u.m_Data.m_Data.m_Data_val;

	return TRUE;
}

BOOL WarUserHandle::SetPwd(user User, LPCSTR Pwd)
{
	user_generic_result *pResult = NULL;
	m_Errno = USRERR_FAIL;
	user_generic_set_str Arg;
	Arg.m_User = User;
	Arg.m_Val = (char *)Pwd;

	if (m_RPCclnt)
		pResult = usrproc_setpwd_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_setpwd_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	return TRUE;
}

BOOL WarUserHandle::SetFileAttrs(user User, LPCSTR Path, 
		LPCSTR Alias, FLEN Quota, 
		DWORD PermMask, DWORD Perms)
{
	user_generic_result *pResult = NULL;
	m_Errno = USRERR_FAIL;
	user_set_fs_access_req Arg;
	Arg.m_User = User;
	Arg.m_Data.m_Flags = (u_int)Perms;
	Arg.m_Data.m_Path = (RPC_string) (Path ? Path : "");
	Arg.m_Data.m_Alias = (RPC_string) (Alias ? Alias : "");
	Arg.m_Data.m_Quota = (hyper)Quota;
	Arg.m_FlagMask = (u_int)PermMask;
	Arg.m_Action = SFA_UPDATE;
	

	if (m_RPCclnt)
		pResult = usrproc_setfileattr_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_setfileattr_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	return TRUE;
}

BOOL WarUserHandle::DeleteFileAttrs(user User, LPCSTR Path)
{
	user_generic_result *pResult = NULL;
	m_Errno = USRERR_FAIL;
	user_set_fs_access_req Arg;
	Arg.m_User = User;
	Arg.m_Data.m_Flags = 0;
	Arg.m_Data.m_Path = (RPC_string) (Path ? Path : "");
	Arg.m_Data.m_Alias = (RPC_string)"";
	Arg.m_Data.m_Quota = 0;
	Arg.m_FlagMask = 0;
	Arg.m_Action = SFA_DELETE;
	

	if (m_RPCclnt)
		pResult = usrproc_setfileattr_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_setfileattr_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	return TRUE;
}

BOOL WarUserHandle::MoveUser(user SrcUser, user Dst, int Flags)
{
	user_generic_result *pResult = NULL;
	m_Errno = USRERR_FAIL;
	move_user_req Arg;
	Arg.m_SrcUser = SrcUser;
	Arg.m_DstUser = Dst;
	Arg.m_Flags = Flags;

	if (m_RPCclnt)
		pResult = usrproc_moveuser_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_moveuser_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	return TRUE;
}

BOOL WarUserHandle::FindUser(user Origin, 
		user_node_type Type,
		LPCSTR Pattern, 
		user_find_direction Direction,
		user_attrs& Attr)
{
	user_find_req Arg;
	Arg.m_Origin = Origin;
	Arg.m_Pattern = (char *)Pattern;
	Arg.m_Direction = Direction;
	Arg.m_Type = Type;

	user_attrs_result *pResult = NULL;
	m_Errno = USRERR_FAIL;

	if (m_RPCclnt)
		pResult = usrproc_find_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_find_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE; 

	Attr = pResult->user_attrs_result_u.m_Data;
	return TRUE;
}

/*
 * General query function
 */
LPCVOID WarUserHandle::Query(user User, 
														 wdq_qmode QueryMode, 
														 wdq_mode DataMode)
{
	war_user_query Arg;
	Arg.m_QueryMode = QueryMode;
	Arg.m_Mode = DataMode;
	Arg.m_User = User;

	user_query_result *pResult = NULL;
	m_Errno = USRERR_FAIL;


	if (m_RPCclnt)
		pResult = usrproc_query_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_query_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	// All possible values are pointers 
	// - so we can return any of them.
	ASSERT(pResult->user_query_result_u.m_Data.wdq_result_u.m_pDaemonInfo != NULL);
	return pResult->user_query_result_u.m_Data.wdq_result_u.m_pDaemonInfo;
}


BOOL WarUserHandle::FetchDomains(WarLinkedList& Domains)
{
	char *Arg;
	user_fetcth_domains_result *pResult = NULL;
	m_Errno = USRERR_FAIL;

	if (m_RPCclnt)
		pResult = usrproc_fetchdomains_1((void *)&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_fetchdomains_1_svc((void *)&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	for(u_int i = 0; 
		i < pResult->user_fetcth_domains_result_u.m_Domains.m_Domains_len;
		i++)
	{
		user *pUser = new user;
		*pUser = pResult->user_fetcth_domains_result_u.m_Domains.m_Domains_val[i];
		Domains.AddLast(pUser);
	}

	return TRUE;
}


/*
 * Internal support functions
 */

BOOL WarUserHandle::CreateRPCClient(LPCSTR Host)
{
	void  *result_1;
	char*  nfsproc_null_1_arg;
	if (!Host)
		Host = "127.0.0.1";
	m_RPCclnt = clnt_create(Host, WARUSER_PROGRAM, WARUSER_VERSION, "UDP");
	if (m_RPCclnt == NULL) 
	{
		m_Errno = USRERR_COMMFAIL;
		clnt_pcreateerror(Host ? Host : "127.0.0.1");
		return FALSE;
	}
	result_1 = nfsproc_null_1((void*)&nfsproc_null_1_arg, m_RPCclnt);
	if (result_1 == NULL) 
	{
		Log.LogMsg(LOGF_ERROR, 0, NULL, 0, 0, 
			"WarUserHandle::CreateRPCClient()", 
			"(%s) - Communication failed test", Host);
		DestroyRPCClient();
		return FALSE;
	}

	return TRUE;
}


BOOL WarUserHandle::WarNTAddGroup(LPCSTR Host, LPCSTR Name, LPCSTR Comment, BOOL bMakeGlobal)
{
	warusr_NT_add_group_req Arg;
	user_generic_result *pResult = NULL;
	m_Errno = USRERR_FAIL;
	Arg.m_Host = (RPC_string)Host;
	Arg.m_Name = (RPC_string)Name;
	Arg.m_Comment = (RPC_string)Comment;
	Arg.m_bMakeGlobal = bMakeGlobal;

	if (m_RPCclnt)
		pResult = usrproc_nt_add_group_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_nt_add_group_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return NULL;

	return TRUE;
}

WarString WarUserHandle::WarNTGetDC(LPCSTR Machine, LPCSTR Domain)
{
	warusr_NT_get_DC_req Arg;
	warsvr_generic_str_result *pResult = NULL;
	m_Errno = USRERR_FAIL;
	Arg.m_Machine = (RPC_string)Machine;
	Arg.m_Domain = (RPC_string)Domain;

	if (m_RPCclnt)
		pResult = usrproc_nt_get_dc_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_nt_get_dc_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return WarString("");

	return WarString(pResult->warsvr_generic_str_result_u.m_Data);
}

BOOL WarUserHandle::WarNTExchangeUsers(user StartNode)
{
	warusr_NT_exchange_req Arg;
	user_generic_result *pResult = NULL;
	m_Errno = USRERR_FAIL;
	Arg.m_StartNode = StartNode;

	if (m_RPCclnt)
		pResult = usrproc_nt_exchange_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_nt_exchange_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return FALSE;

	return TRUE;
}

WarLinkedList *WarUserHandle::WarNTGetDomains(LPCSTR RunOnMachine)
{
	warusr_NT_get_domains_req Arg;
	warsvr_buffer_result *pResult = NULL;
	m_Errno = USRERR_FAIL;
	Arg.m_RunOnMachine = RunOnMachine ? (RPC_string)RunOnMachine : "";

	if (m_RPCclnt)
		pResult = usrproc_nt_get_domains_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_nt_get_domains_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return NULL;

	// Read the list
	int ID = pResult->warsvr_buffer_result_u.m_Handle;
	BOOL IsEOF;
	int GotBytes, StartPos = 0;
	WarString wBuf;
	LPVOID pBuf;
	do
	{
		if (!ReadBuffer(ID, StartPos, 1024, &pBuf, GotBytes, IsEOF))
			return NULL;

		wBuf.AddData(pBuf, GotBytes);
		StartPos += GotBytes;

	} while(!IsEOF);

	wBuf.ReleaseBuffer(StartPos);
	WarLinkedList *pList = new WarLinkedList;
	LPCSTR p = wBuf;
	while(*p)
	{
		WarString wName, wComment;
		int Type;

		while(*p != '\t')
			wName << *p++;

		ASSERT(*p == '\t');
		++p;

		while(*p != '\t')
			wComment << *p++;

		ASSERT(*p == '\t');
		++p;

		Type = atoi(p);

		pList->AddLast((LPVOID)new WarNTServerInfo(wName, wComment, Type));

		while(*p != '\n')
			++p;
		++p;
	}

	return pList;
}

WarLinkedList *WarUserHandle::WarNTGetServers(LPCSTR RunOnMachine, LPCSTR Domain)
{
	warusr_NT_get_servers_req Arg;
	warsvr_buffer_result *pResult = NULL;
	m_Errno = USRERR_FAIL;
	Arg.m_RunOnMachine = RunOnMachine ? (RPC_string)RunOnMachine : "";
	Arg.m_Domain = Domain ? (RPC_string)Domain : "";

	if (m_RPCclnt)
		pResult = usrproc_nt_get_servers_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_nt_get_servers_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return NULL;

	// Read the list
	int ID = pResult->warsvr_buffer_result_u.m_Handle;
	BOOL IsEOF;
	int GotBytes, StartPos = 0;
	WarString wBuf;
	LPVOID pBuf;
	do
	{
		if (!ReadBuffer(ID, StartPos, 1024, &pBuf, GotBytes, IsEOF))
			return NULL;

		wBuf.AddData(pBuf, GotBytes);
		StartPos += GotBytes;

	} while(!IsEOF);

	wBuf.ReleaseBuffer(StartPos);
	WarLinkedList *pList = new WarLinkedList;
	LPCSTR p = wBuf;
	while(*p)
	{
		WarString wName, wComment;
		int Type;

		while(*p != '\t')
			wName << *p++;

		ASSERT(*p == '\t');
		++p;

		while(*p != '\t')
			wComment << *p++;

		ASSERT(*p == '\t');
		++p;

		Type = atoi(p);

		pList->AddLast((LPVOID)new WarNTServerInfo(wName, wComment, Type));

		while(*p != '\n')
			++p;
		++p;
	}

	return pList;
}

WarLinkedList *WarUserHandle::WarNTGetGroups(LPCSTR Host)
{
	warusr_NT_get_groups_req Arg;
	warsvr_buffer_result *pResult = NULL;
	m_Errno = USRERR_FAIL;
	Arg.m_RunOnMachine = Host ? (RPC_string)Host : "";

	if (m_RPCclnt)
		pResult = usrproc_nt_get_groups_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_nt_get_groups_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return NULL;

	// Read the list
	int ID = pResult->warsvr_buffer_result_u.m_Handle;
	BOOL IsEOF;
	int GotBytes, StartPos = 0;
	WarString wBuf;
	LPVOID pBuf;
	do
	{
		if (!ReadBuffer(ID, StartPos, 1024, &pBuf, GotBytes, IsEOF))
			return NULL;

		wBuf.AddData(pBuf, GotBytes);
		StartPos += GotBytes;

	} while(!IsEOF);

	wBuf.ReleaseBuffer(StartPos);
	WarLinkedList *pList = new WarLinkedList;
	LPCSTR p = wBuf;
	while(*p)
	{
		WarString wName, wComment;
		int Global;

		while(*p != '\t')
			wName << *p++;

		ASSERT(*p == '\t');
		++p;

		while(*p != '\t')
			wComment << *p++;

		ASSERT(*p == '\t');
		++p;

		Global = atoi(p);

		pList->AddLast((LPVOID)new WarNTGroupInfo(wName, wComment, Global));

		while(*p != '\n')
			++p;
		++p;
	}

	return pList;
}

sesmgr_svr_startup_options *WarUserHandle::SvrGetStartupOptions()
{
	sesmgr_svr_startup_options_result *pResult = NULL;
	char Arg = 0;
	m_Errno = USRERR_FAIL;

	if (m_RPCclnt)
		pResult = usrproc_svr_query_startupoptions_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_svr_query_startupoptions_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return NULL;

	return &pResult->sesmgr_svr_startup_options_result_u.m_Data;
}

sesmgr_svr_startup_options *WarUserHandle::SvrSetStartupOptions(
	sesmgr_svr_startup_req& Arg)
{
	sesmgr_svr_startup_options_result *pResult = NULL;
	m_Errno = USRERR_FAIL;

	if (Arg.m_UsePasswd == NULL)
		Arg.m_UsePasswd = (RPC_string)"";

	if (Arg.m_UseName == NULL)
		Arg.m_UseName = (RPC_string)"";

	if (m_RPCclnt)
		pResult = usrproc_svr_set_startupoptions_1(&Arg, m_RPCclnt);
#ifdef WARSVR
	else
		pResult = usrproc_svr_set_startupoptions_1_svc(&Arg, m_pSVC);
#endif

	if (!pResult || ((m_Errno = pResult->status) != USR_OK))
		return NULL;

	return &pResult->sesmgr_svr_startup_options_result_u.m_Data;
}



void WarUserHandle::DestroyRPCClient()
{
	if (m_RPCclnt)
	{	
		RPCLogout();
		clnt_destroy(m_RPCclnt);
	}
	m_RPCclnt = NULL;
}

// Unpack a login request reply
BOOL UnprpAuthData(LPVOID Raw)
{	
	war_auth_def_100 *p = (war_auth_def_100 *)Raw;
	switch(p->m_EncrLevel)
	{
		case WARAC_LEVEL_100:
			// Decrypt from user and down
			ecb_crypt(WeakEncrKey, (char *)&p->m_User, WAR_AUT_DEF_100_CLEN, DES_DECRYPT);
			break;
		default:
			return FALSE;
	}

	return TRUE;
}

 